@@ -1,6 +1,6 @@
 AIC.rma <- function(object, ..., k=2, correct=FALSE) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(object), must="rma")
 
@@ -39,9 +39,9 @@ AIC.rma <- function(object, ..., k=2, correct=FALSE) {
 
       ### check that all models were fitted to the same data
 
-      yis <- lapply(list(object, ...), function(x) as.vector(x$yi))
+      chksums <- sapply(list(object, ...), function(x) x$chksumyi)
 
-      if (!all(sapply(yis[-1], function(x) identical(x, yis[[1]]))))
+      if (any(chksums[1] != chksums))
          warning(mstyle$warning("Models not all fitted to the same data."), call.=FALSE)
 
    }
