@@ -1,9 +1,8 @@
 BIC.rma <- function(object, ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
-   if (!inherits(object, "rma"))
-      stop(mstyle$stop("Argument 'object' must be an object of class \"rma\"."))
+   .chkclass(class(object), must="rma")
 
    if (missing(...)) {
 
@@ -35,9 +34,10 @@ BIC.rma <- function(object, ...) {
 
       ### check that all models were fitted to the same data
 
-      yis <- lapply(list(object, ...), function(x) as.vector(x$yi))
-      if (!all(sapply(yis[-1], function(x) identical(x, yis[[1]]))))
-         warning(mstyle$warning("Models not all fitted to the same data."))
+      chksums <- sapply(list(object, ...), function(x) x$chksumyi)
+
+      if (any(chksums[1] != chksums))
+         warning(mstyle$warning("Models not all fitted to the same data."), call.=FALSE)
 
    }
 
