@@ -2,7 +2,7 @@ aggregate.escalc <- function(x, cluster, time, obs, V, struct="CS", rho, phi,
                              weighted=TRUE, checkpd=TRUE, fun, na.rm=TRUE,
                              addk=FALSE, subset, select, digits, var.names, ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(x), must="escalc")
 
@@ -10,10 +10,9 @@ aggregate.escalc <- function(x, cluster, time, obs, V, struct="CS", rho, phi,
       stop(mstyle$stop("Unknown 'struct' specified."))
 
    if (missing(cluster))
-      stop(mstyle$stop("Must specify 'cluster' variable."))
+      stop(mstyle$stop("Must specify the 'cluster' variable."))
 
-   if (length(na.rm) == 1L)
-      na.rm <- c(na.rm, na.rm)
+   na.rm <- .expand1(na.rm, 2L)
 
    k <- nrow(x)
 
@@ -37,7 +36,7 @@ aggregate.escalc <- function(x, cluster, time, obs, V, struct="CS", rho, phi,
       stop(mstyle$stop("No missing values allowed in 'cluster' variable."))
 
    if (length(cluster) != k)
-      stop(mstyle$stop(paste0("Length of variable specified via 'cluster' (", length(cluster), ") does not match length of data (", k, ").")))
+      stop(mstyle$stop(paste0("Length of the variable specified via 'cluster' (", length(cluster), ") does not match the length of the data (", k, ").")))
 
    ucluster <- unique(cluster)
    n <- length(ucluster)
@@ -102,8 +101,7 @@ aggregate.escalc <- function(x, cluster, time, obs, V, struct="CS", rho, phi,
          if (missing(rho))
             stop(mstyle$stop("Must specify 'rho' for this var-cov structure."))
 
-         if (length(rho) == 1L)
-            rho <- rep(rho, n)
+         rho <- .expand1(rho, n)
 
          if (length(rho) != n)
             stop(mstyle$stop(paste0("Length of 'rho' (", length(rho), ") does not match the number of clusters (", n, ").")))
@@ -118,8 +116,7 @@ aggregate.escalc <- function(x, cluster, time, obs, V, struct="CS", rho, phi,
          if (missing(phi))
             stop(mstyle$stop("Must specify 'phi' for this var-cov structure."))
 
-         if (length(phi) == 1L)
-            phi <- rep(phi, n)
+         phi <- .expand1(phi, n)
 
          if (length(phi) != n)
             stop(mstyle$stop(paste0("Length of 'phi' (", length(phi), ") does not match the number of clusters (", n, ").")))
@@ -130,20 +127,20 @@ aggregate.escalc <- function(x, cluster, time, obs, V, struct="CS", rho, phi,
          ### checks on time variable
 
          if (!is.element("time", names(mf)))
-            stop(mstyle$stop("Must specify 'time' variable for this var-cov structure."))
+            stop(mstyle$stop("Must specify a 'time' variable for this var-cov structure."))
 
          if (length(time) != k)
-            stop(mstyle$stop(paste0("Length of variable specified via 'time' (", length(time), ") does not match length of data (", k, ").")))
+            stop(mstyle$stop(paste0("Length of the variable specified via 'time' (", length(time), ") does not match the length of the data (", k, ").")))
 
          if (struct == "CS*CAR") {
 
             ### checks on obs variable
 
          if (!is.element("obs", names(mf)))
-            stop(mstyle$stop("Must specify 'obs' variable for this var-cov structure."))
+            stop(mstyle$stop("Must specify an 'obs' variable for this var-cov structure."))
 
             if (length(obs) != k)
-               stop(mstyle$stop(paste0("Length of variable specified via 'obs' (", length(obs), ") does not match length of data (", k, ").")))
+               stop(mstyle$stop(paste0("Length of the variable specified via 'obs' (", length(obs), ") does not match the length of the data (", k, ").")))
 
          }
 
@@ -195,11 +192,10 @@ aggregate.escalc <- function(x, cluster, time, obs, V, struct="CS", rho, phi,
 
       if (.is.vector(V)) {
 
-         if (length(V) == 1L)
-            V <- rep(V, k)
+         V <- .expand1(V, k)
 
          if (length(V) != k)
-            stop(mstyle$stop(paste0("Length of 'V' (", length(V), ") does not match length of data frame (", k, ").")))
+            stop(mstyle$stop(paste0("Length of 'V' (", length(V), ") does not match the length of the data frame (", k, ").")))
 
          V <- diag(as.vector(V), nrow=k, ncol=k)
 
@@ -218,7 +214,7 @@ aggregate.escalc <- function(x, cluster, time, obs, V, struct="CS", rho, phi,
          stop(mstyle$stop("'V' must be a symmetric matrix."))
 
       if (nrow(V) != k)
-         stop(mstyle$stop(paste0("Dimensions of 'V' (", nrow(V), "x", ncol(V), ") do not match length of data frame (", k, ").")))
+         stop(mstyle$stop(paste0("Dimensions of 'V' (", nrow(V), "x", ncol(V), ") do not match the length of the data frame (", k, ").")))
 
       ### check that covariances are really 0 for estimates belonging to different clusters
       ### note: if na.rm[1] is FALSE, there may be missings in V, so skip check in those clusters
@@ -417,7 +413,7 @@ aggregate.escalc <- function(x, cluster, time, obs, V, struct="CS", rho, phi,
       attr(xagg, "digits") <- attr(x, "digits")
    }
 
-   if (is.null(attr(xagg, "digits"))) ### in case x no longer has a 'digits' attribute
+   if (is.null(attr(xagg, "digits"))) # in case x no longer has a 'digits' attribute
       attr(xagg, "digits") <- 4
 
    class(xagg) <- c("escalc", "data.frame")
