@@ -1,6 +1,6 @@
 confint.rma.mh <- function(object, parm, level, digits, transf, targs, ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(object), must="rma.mh")
 
@@ -33,7 +33,7 @@ confint.rma.mh <- function(object, parm, level, digits, transf, targs, ...) {
 
    #########################################################################
 
-   level <- ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > .5, 1-level, level)))
+   level <- .level(level)
    crit  <- qnorm(level/2, lower.tail=FALSE)
 
    beta <- x$beta
@@ -42,7 +42,7 @@ confint.rma.mh <- function(object, parm, level, digits, transf, targs, ...) {
 
    ### if requested, apply transformation function
 
-   if (.isTRUE(transf) && is.element(x$measure, c("OR","RR","IRR"))) ### if transf=TRUE, apply exp transformation to ORs, RRs, and IRRs
+   if (.isTRUE(transf) && is.element(x$measure, c("OR","RR","IRR"))) # if transf=TRUE, apply exp transformation to ORs, RRs, and IRRs
       transf <- exp
 
    if (is.function(transf)) {
@@ -51,6 +51,8 @@ confint.rma.mh <- function(object, parm, level, digits, transf, targs, ...) {
          ci.lb <- sapply(ci.lb, transf)
          ci.ub <- sapply(ci.ub, transf)
       } else {
+         if (!is.primitive(transf) && !is.null(targs) && length(formals(transf)) == 1L)
+            stop(mstyle$stop("Function specified via 'transf' does not appear to have an argument for 'targs'."))
          beta  <- sapply(beta, transf, targs)
          ci.lb <- sapply(ci.lb, transf, targs)
          ci.ub <- sapply(ci.ub, transf, targs)
