@@ -2,7 +2,7 @@ conv.fivenum <- function(min, q1, median, q3, max, n, data, include,
                          method="default", dist="norm", transf=TRUE, test=TRUE,
                          var.names=c("mean","sd"), append=TRUE, replace="ifna", ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    if (missing(min) && missing(q1) && missing(median) && missing(q3) && missing(max))
       stop(mstyle$stop("Must specify at least some of these arguments: 'min', 'q1', 'median', 'q3', 'max'."))
@@ -23,11 +23,7 @@ conv.fivenum <- function(min, q1, median, q3, max, n, data, include,
 
    .chkdots(ddd, c("verbose", "seed"))
 
-   if (is.null(ddd$verbose)) {
-      verbose <- FALSE
-   } else {
-      verbose <- .isTRUE(ddd$verbose)
-   }
+   verbose <- .chkddd(ddd$verbose, FALSE, .isTRUE(ddd$verbose))
 
    if (!is.null(ddd$seed))
       set.seed(ddd$seed)
@@ -91,11 +87,10 @@ conv.fivenum <- function(min, q1, median, q3, max, n, data, include,
 
    ### handle dist argument
 
-   if (length(dist) == 1L)
-      dist <- rep(dist, k)
+   dist <- .expand1(dist, k)
 
    if (length(dist) != k)
-      stop(mstyle$stop(paste0("Length of 'dist' argument (", length(dist), ") does not match length of data (", k, ").")))
+      stop(mstyle$stop(paste0("Length of the 'dist' argument (", length(dist), ") does not match the length of the data (", k, ").")))
 
    dist <- c("norm","lnorm")[pmatch(dist, c("norm","lnorm"), duplicates.ok=TRUE)]
 
@@ -107,7 +102,7 @@ conv.fivenum <- function(min, q1, median, q3, max, n, data, include,
    if (is.null(include))
       include <- rep(TRUE, k)
 
-   ### turn numeric include vector into logical vector
+   ### turn numeric include vector into a logical vector
 
    include <- .chksubset(include, k, stoponk0=FALSE)
 
@@ -136,8 +131,7 @@ conv.fivenum <- function(min, q1, median, q3, max, n, data, include,
 
    method <- tolower(method)
 
-   if (length(method) == 1L)
-      method <- c(method, method)
+   method <- .expand1(method, 2L)
 
    method1.options <- c("default", "luo/wan/shi", "qe", "bc", "mln", "blue", "hozo2005", "wan2014", "bland2015", "luo2016", "walter2007")
    method2.options <- c("default", "luo/wan/shi", "qe", "bc", "mln", "blue", "hozo2005", "wan2014", "bland2015", "shi2020", "walter2007")
@@ -188,7 +182,7 @@ conv.fivenum <- function(min, q1, median, q3, max, n, data, include,
    tval  <- rep(NA_real_, k)
    crit  <- rep(NA_real_, k)
    sig   <- rep(NA,       k)
-   dists <- rep("norm", k)
+   dists <- rep("norm",   k)
 
    for (i in seq_len(k)) {
 
