@@ -1,6 +1,6 @@
 fitstats.rma <- function(object, ..., REML) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(object), must="rma")
 
@@ -47,8 +47,9 @@ fitstats.rma <- function(object, ..., REML) {
 
       ### check that all models were fitted to the same data
 
-      yis <- lapply(list(object, ...), function(x) as.vector(x$yi))
-      if (!all(sapply(yis[-1], function(x) identical(x, yis[[1]]))))
+      chksums <- sapply(list(object, ...), function(x) x$chksumyi)
+
+      if (any(chksums[1] != chksums))
          warning(mstyle$warning("Models not all fitted to the same data."), call.=FALSE)
 
    }
@@ -56,7 +57,7 @@ fitstats.rma <- function(object, ..., REML) {
    rownames(out) <- c("logLik:", "deviance:", "AIC:", "BIC:", "AICc:")
    return(out)
 
-   #print(.fcf(out, object$digits[["fit"]]), quote=FALSE)
+   #print(fmtx(out, object$digits[["fit"]]), quote=FALSE)
    #invisible(out)
 
 }
