@@ -1,14 +1,14 @@
 forest.default   <- function(x, vi, sei, ci.lb, ci.ub,
-annotate=TRUE,                             showweights=FALSE, header=FALSE,
+annotate=TRUE,                                            showweights=FALSE, header=TRUE,
 xlim, alim, olim, ylim, at, steps=5, level=95,      refline=0, digits=2L, width,
-xlab, slab,       ilab, ilab.xpos, ilab.pos, order, subset,
+xlab, slab,       ilab, ilab.lab, ilab.xpos, ilab.pos, order, subset,
 transf, atransf, targs, rows,
 efac=1, pch, psize, plim=c(0.5,1.5),         col,         shade, colshade,
 lty, fonts, cex, cex.lab, cex.axis, ...) {
 
    #########################################################################
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    na.act <- getOption("na.action")
 
@@ -40,6 +40,9 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
    if (missing(ilab))
       ilab <- NULL
 
+   if (missing(ilab.lab))
+      ilab.lab <- NULL
+
    if (missing(ilab.xpos))
       ilab.xpos <- NULL
 
@@ -103,8 +106,9 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
    ### vertical expansion factor: 1st = CI end lines, 2nd = arrows
 
-   if (length(efac) == 1L)
-      efac <- rep(efac, 2L)
+   efac <- .expand1(efac, 2L)
+
+   efac[efac == 0] <- NA
 
    ### annotation symbols vector
 
@@ -140,10 +144,11 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
    ### column header
 
    estlab <- .setlab(measure, transf.char, atransf.char, gentype=3, short=TRUE)
+
    if (is.expression(estlab)) {
-      header.right <- str2lang(paste0("bold(", estlab, " * '", annosym[1], "' * '", 100*(1-level), "% CI'", " * '", annosym[3], "')"))
+      header.right <- str2lang(paste0("bold(", estlab, " * '", annosym[1], "' * '", round(100*(1-level),digits[[1]]), "% CI'", " * '", annosym[3], "')"))
    } else {
-      header.right <- paste0(estlab, annosym[1], 100*(1-level), "% CI", annosym[3])
+      header.right <- paste0(estlab, annosym[1], round(100*(1-level),digits[[1]]), "% CI", annosym[3])
    }
 
    if (is.logical(header)) {
@@ -167,11 +172,7 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
    if (!annotate)
       header.right <- NULL
 
-   if (is.null(ddd$decreasing)) {
-      decreasing <- FALSE
-   } else {
-      decreasing <- ddd$decreasing
-   }
+   decreasing <- .chkddd(ddd$decreasing, FALSE)
 
    if (!is.null(ddd$clim))
       olim <- ddd$clim
@@ -188,11 +189,7 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
          rowadj <- c(rowadj,0) # if two values are specified, use them for 1&2
    }
 
-   if (is.null(ddd$top)) {
-      top <- 3
-   } else {
-      top <- ddd$top
-   }
+   top <- .chkddd(ddd$top, 3)
 
    if (is.null(ddd$xlabadj)) {
       xlabadj <- c(NA,NA)
@@ -202,51 +199,55 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
          xlabadj <- c(xlabadj, 1-xlabadj)
    }
 
-   if (is.null(ddd$xlabfont)) {
-      xlabfont <- 1
-   } else {
-      xlabfont <- ddd$xlabfont
-   }
+   xlabfont <- .chkddd(ddd$xlabfont, 1)
 
-   lplot     <- function(..., textpos, decreasing, clim, rowadj, annosym, tabfig, top, xlabadj, xlabfont) plot(...)
-   labline   <- function(..., textpos, decreasing, clim, rowadj, annosym, tabfig, top, xlabadj, xlabfont) abline(...)
-   lsegments <- function(..., textpos, decreasing, clim, rowadj, annosym, tabfig, top, xlabadj, xlabfont) segments(...)
-   laxis     <- function(..., textpos, decreasing, clim, rowadj, annosym, tabfig, top, xlabadj, xlabfont) axis(...)
-   lmtext    <- function(..., textpos, decreasing, clim, rowadj, annosym, tabfig, top, xlabadj, xlabfont) mtext(...)
-   lpolygon  <- function(..., textpos, decreasing, clim, rowadj, annosym, tabfig, top, xlabadj, xlabfont) polygon(...)
-   ltext     <- function(..., textpos, decreasing, clim, rowadj, annosym, tabfig, top, xlabadj, xlabfont) text(...)
-   lpoints   <- function(..., textpos, decreasing, clim, rowadj, annosym, tabfig, top, xlabadj, xlabfont) points(...)
+   lplot     <- function(..., textpos, decreasing, clim, rowadj, annosym, tabfig, top, xlabadj, xlabfont, at.lab) plot(...)
+   labline   <- function(..., textpos, decreasing, clim, rowadj, annosym, tabfig, top, xlabadj, xlabfont, at.lab) abline(...)
+   lsegments <- function(..., textpos, decreasing, clim, rowadj, annosym, tabfig, top, xlabadj, xlabfont, at.lab) segments(...)
+   laxis     <- function(..., textpos, decreasing, clim, rowadj, annosym, tabfig, top, xlabadj, xlabfont, at.lab) axis(...)
+   lmtext    <- function(..., textpos, decreasing, clim, rowadj, annosym, tabfig, top, xlabadj, xlabfont, at.lab) mtext(...)
+   lpolygon  <- function(..., textpos, decreasing, clim, rowadj, annosym, tabfig, top, xlabadj, xlabfont, at.lab) polygon(...)
+   ltext     <- function(..., textpos, decreasing, clim, rowadj, annosym, tabfig, top, xlabadj, xlabfont, at.lab) text(...)
+   lpoints   <- function(..., textpos, decreasing, clim, rowadj, annosym, tabfig, top, xlabadj, xlabfont, at.lab) points(...)
 
    #########################################################################
 
    ### extract data, study labels, and other arguments
 
    if (!missing(vi) && is.function(vi)) # if vi is utils::vi()
-      stop(mstyle$stop("Cannot find variable specified for 'vi' argument."), call.=FALSE)
+      stop(mstyle$stop("Cannot find variable specified for the 'vi' argument."))
+
+   if (hasArg(ci.lb) && hasArg(ci.ub) && !is.null(ci.lb) && !is.null(ci.ub)) { # CI bounds are specified by user
 
-   if (hasArg(ci.lb) && hasArg(ci.ub)) {     # CI bounds are specified by user
       if (length(ci.lb) != length(ci.ub))
-         stop(mstyle$stop("Length of 'ci.lb' and 'ci.ub' is not the same."))
+         stop(mstyle$stop("Length of 'ci.lb' and 'ci.ub' are not the same."))
+
       if (missing(vi) && missing(sei)) {     # vi/sei not specified, so calculate vi based on CI
          vi <- ((ci.ub - ci.lb) / (2*qnorm(level/2, lower.tail=FALSE)))^2
       } else {
          if (missing(vi))                    # vi not specified, but sei is, so set vi = sei^2
             vi <- sei^2
       }
+
       if (length(ci.lb) != length(vi))
-         stop(mstyle$stop("Length of 'vi' (or 'sei') does not match length of ('ci.lb', 'ci.ub') pairs."))
+         stop(mstyle$stop("Length of 'vi' (or 'sei') does not match the length of ('ci.lb','ci.ub')."))
+
    } else {                                  # CI bounds are not specified by user
+
       if (missing(vi)) {
          if (missing(sei)) {
-            stop(mstyle$stop("Must specify either 'vi', 'sei', or ('ci.lb', 'ci.ub') pairs."))
+            stop(mstyle$stop("Must specify either 'vi', 'sei', or ('ci.lb','ci.ub')."))
          } else {
             vi <- sei^2
          }
       }
+
       if (length(yi) != length(vi)) # need to do this here to avoid warning when calculating 'ci.lb' and 'ci.ub'
-         stop(mstyle$stop("Length of 'vi' (or 'sei') does not match length of 'yi'."))
+         stop(mstyle$stop("Length of 'vi' (or 'sei') does not match the length of 'yi'."))
+
       ci.lb <- yi - qnorm(level/2, lower.tail=FALSE) * sqrt(vi)
       ci.ub <- yi + qnorm(level/2, lower.tail=FALSE) * sqrt(vi)
+
    }
 
    ### check length of yi and vi
@@ -254,7 +255,7 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
    k <- length(yi)
 
    if (length(vi) != k)
-      stop(mstyle$stop("Length of 'yi' does not match the length of 'vi', 'sei', or the ('ci.lb', 'ci.ub') pairs."))
+      stop(mstyle$stop("Length of 'yi' does not match the length of 'vi', 'sei', or the ('ci.lb','ci.ub')."))
 
    ### note: slab (if specified), ilab (if specified), pch (if vector), psize (if
    ###       vector), col (if vector), subset (if specified), order (if vector)
@@ -288,22 +289,20 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
    }
 
-   if (length(pch) == 1L)
-      pch <- rep(pch, k)                        # pch can be a single value (which is then repeated)
+   pch <- .expand1(pch, k) # pch can be a single value (which is then repeated)
 
    if (length(pch) != k)
       stop(mstyle$stop(paste0("Length of the 'pch' argument (", length(pch), ") does not correspond to the number of outcomes (", k, ").")))
 
    if (!is.null(psize)) {
-      if (length(psize) == 1L)                  # psize can be a single value (which is then repeated)
-         psize <- rep(psize, k)
+      if (length(psize) == 1L)
+      psize <- .expand1(psize, k) # psize can be a single value (which is then repeated)
       if (length(psize) != k)
          stop(mstyle$stop(paste0("Length of the 'psize' argument (", length(psize), ") does not correspond to the number of outcomes (", k, ").")))
    }
 
    if (!is.null(col)) {
-      if (length(col) == 1L)                    # col can be a single value (which is then repeated)
-         col <- rep(col, k)
+      col <- .expand1(col, k) # col can be a single value (which is then repeated)
       if (length(col) != k)
          stop(mstyle$stop(paste0("Length of the 'col' argument (", length(col), ") does not correspond to the number of outcomes (", k, ").")))
    } else {
@@ -473,6 +472,8 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
          ci.lb <- sapply(ci.lb, transf)
          ci.ub <- sapply(ci.ub, transf)
       } else {
+         if (!is.primitive(transf) && !is.null(targs) && length(formals(transf)) == 1L)
+            stop(mstyle$stop("Function specified via 'transf' does not appear to have an argument for 'targs'."))
          yi    <- sapply(yi, transf, targs)
          ci.lb <- sapply(ci.lb, transf, targs)
          ci.ub <- sapply(ci.ub, transf, targs)
@@ -491,10 +492,9 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
       if (length(olim) != 2L)
          stop(mstyle$stop("Argument 'olim' must be of length 2."))
       olim <- sort(olim)
-      yi[yi < olim[1]] <- olim[1]
-      yi[yi > olim[2]] <- olim[2]
-      ci.lb[ci.lb < olim[1]] <- olim[1]
-      ci.ub[ci.ub > olim[2]] <- olim[2]
+      yi    <- .applyolim(yi, olim)
+      ci.lb <- .applyolim(ci.lb, olim)
+      ci.ub <- .applyolim(ci.ub, olim)
    }
 
    if (showweights) {                           # inverse variance weights after ordering/subsetting and
@@ -578,16 +578,24 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
    ### x-axis labels (apply transformation to axis labels if requested)
 
-   at.lab <- at
+   if (is.null(ddd$at.lab)) {
 
-   if (is.function(atransf)) {
-      if (is.null(targs)) {
-         at.lab <- fmtx(sapply(at.lab, atransf), digits[[2]], drop0ifint=TRUE)
+      at.lab <- at
+
+      if (is.function(atransf)) {
+         if (is.null(targs)) {
+            at.lab <- fmtx(sapply(at.lab, atransf), digits[[2]], drop0ifint=TRUE)
+         } else {
+            at.lab <- fmtx(sapply(at.lab, atransf, targs), digits[[2]], drop0ifint=TRUE)
+         }
       } else {
-         at.lab <- fmtx(sapply(at.lab, atransf, targs), digits[[2]], drop0ifint=TRUE)
+         at.lab <- fmtx(at.lab, digits[[2]], drop0ifint=TRUE)
       }
+
    } else {
-      at.lab <- fmtx(at.lab, digits[[2]], drop0ifint=TRUE)
+
+      at.lab <- ddd$at.lab
+
    }
 
    ### set plot limits (xlim)
@@ -639,6 +647,9 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
       xlim <- round(xlim, digits[[2]])
       #xlim[1] <- xlim[1]*max(1, digits[[2]]/2)
       #xlim[2] <- xlim[2]*max(1, digits[[2]]/2)
+   } else {
+      if (length(xlim) != 2L)
+         stop(mstyle$stop("Argument 'xlim' must be of length 2."))
    }
 
    xlim <- sort(xlim)
@@ -660,11 +671,7 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
    ### allow adjustment of position of study labels and annotations via textpos argument
 
-   if (is.null(ddd$textpos)) {
-      textpos <- xlim
-   } else {
-      textpos <- ddd$textpos
-   }
+   textpos <- .chkddd(ddd$textpos, xlim)
 
    if (length(textpos) != 2L)
       stop(mstyle$stop("Argument 'textpos' must be of length 2."))
@@ -678,9 +685,13 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
    ### set y-axis limits
 
    if (missing(ylim)) {
-      ylim <- c(0.5, max(rows, na.rm=TRUE)+top)
+      ylim <- c(0, max(rows, na.rm=TRUE)+top)
    } else {
-      ylim <- sort(ylim)
+      if (length(ylim) == 1L) {
+         ylim <- c(ylim, max(rows, na.rm=TRUE)+top)
+      } else {
+         ylim <- sort(ylim)
+      }
    }
 
    #########################################################################
@@ -707,12 +718,12 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
    par.mar <- par("mar")
    par.mar.adj <- par.mar - c(0,3,1,1)
    par.mar.adj[par.mar.adj < 0] <- 0
-   par(mar = par.mar.adj)
-   on.exit(par(mar = par.mar), add=TRUE)
+   par(mar=par.mar.adj)
+   on.exit(par(mar=par.mar), add=TRUE)
 
    ### start plot
 
-   lplot(NA, NA, xlim=xlim, ylim=ylim, xlab="", ylab="", yaxt="n", xaxt="n", xaxs="i", bty="n", ...)
+   lplot(NA, NA, xlim=xlim, ylim=ylim, xlab="", ylab="", yaxt="n", xaxt="n", xaxs="i", yaxs="i", bty="n", ...)
 
    ### add shading
 
@@ -765,7 +776,7 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
       cex <- par("cex") * cex.adj
    } else {
       if (is.null(cex.lab))
-         cex.lab <- cex
+         cex.lab <- par("cex") * cex
       if (is.null(cex.axis))
          cex.axis <- cex
    }
@@ -800,6 +811,10 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
    ### add CI ends (either | or <> if outside of axis limits)
 
+   ciendheight <- height / 150 * cex * efac[1]
+   arrowwidth  <- 1.4    / 100 * cex * (xlim[2]-xlim[1])
+   arrowheight <- height / 150 * cex * efac[2]
+
    for (i in seq_len(k)) {
 
       ### need to skip missings (if check below will otherwise throw an error)
@@ -808,28 +823,32 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
       ### if the lower bound is actually larger than upper x-axis limit, then everything is to the right and just draw a polygon pointing in that direction
       if (ci.lb[i] >= alim[2]) {
-         lpolygon(x=c(alim[2], alim[2]-(1.4/100)*cex*(xlim[2]-xlim[1]), alim[2]-(1.4/100)*cex*(xlim[2]-xlim[1]), alim[2]), y=c(rows[i], rows[i]+(height/150)*cex*efac[2], rows[i]-(height/150)*cex*efac[2], rows[i]), col=col[i], border=col[i], ...)
+         lpolygon(x=c(alim[2], alim[2]-arrowwidth, alim[2]-arrowwidth, alim[2]),
+                  y=c(rows[i], rows[i]+arrowheight, rows[i]-arrowheight, rows[i]), col=col[i], border=col[i], ...)
          next
       }
 
       ### if the upper bound is actually lower than lower x-axis limit, then everything is to the left and just draw a polygon pointing in that direction
       if (ci.ub[i] <= alim[1]) {
-         lpolygon(x=c(alim[1], alim[1]+(1.4/100)*cex*(xlim[2]-xlim[1]), alim[1]+(1.4/100)*cex*(xlim[2]-xlim[1]), alim[1]), y=c(rows[i], rows[i]+(height/150)*cex*efac[2], rows[i]-(height/150)*cex*efac[2], rows[i]), col=col[i], border=col[i], ...)
+         lpolygon(x=c(alim[1], alim[1]+arrowwidth, alim[1]+arrowwidth, alim[1]),
+                  y=c(rows[i], rows[i]+arrowheight, rows[i]-arrowheight, rows[i]), col=col[i], border=col[i], ...)
          next
       }
 
       lsegments(max(ci.lb[i], alim[1]), rows[i], min(ci.ub[i], alim[2]), rows[i], lty=lty[1], col=col[i], ...)
 
       if (ci.lb[i] >= alim[1]) {
-         lsegments(ci.lb[i], rows[i]-(height/150)*cex*efac[1], ci.lb[i], rows[i]+(height/150)*cex*efac[1], col=col[i], ...)
+         lsegments(ci.lb[i], rows[i]-ciendheight, ci.lb[i], rows[i]+ciendheight, col=col[i], ...)
       } else {
-         lpolygon(x=c(alim[1], alim[1]+(1.4/100)*cex*(xlim[2]-xlim[1]), alim[1]+(1.4/100)*cex*(xlim[2]-xlim[1]), alim[1]), y=c(rows[i], rows[i]+(height/150)*cex*efac[2], rows[i]-(height/150)*cex*efac[2], rows[i]), col=col[i], border=col[i], ...)
+         lpolygon(x=c(alim[1], alim[1]+arrowwidth, alim[1]+arrowwidth, alim[1]),
+                  y=c(rows[i], rows[i]+arrowheight, rows[i]-arrowheight, rows[i]), col=col[i], border=col[i], ...)
       }
 
       if (ci.ub[i] <= alim[2]) {
-         lsegments(ci.ub[i], rows[i]-(height/150)*cex*efac[1], ci.ub[i], rows[i]+(height/150)*cex*efac[1], col=col[i], ...)
+         lsegments(ci.ub[i], rows[i]-ciendheight, ci.ub[i], rows[i]+ciendheight, col=col[i], ...)
       } else {
-         lpolygon(x=c(alim[2], alim[2]-(1.4/100)*cex*(xlim[2]-xlim[1]), alim[2]-(1.4/100)*cex*(xlim[2]-xlim[1]), alim[2]), y=c(rows[i], rows[i]+(height/150)*cex*efac[2], rows[i]-(height/150)*cex*efac[2], rows[i]), col=col[i], border=col[i], ...)
+         lpolygon(x=c(alim[2], alim[2]-arrowwidth, alim[2]-arrowwidth, alim[2]),
+                  y=c(rows[i], rows[i]+arrowheight, rows[i]-arrowheight, rows[i]), col=col[i], border=col[i], ...)
       }
 
    }
@@ -842,7 +861,7 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
    if (!is.null(ilab)) {
       if (is.null(ilab.xpos)) {
-         #stop(mstyle$stop("Must specify 'ilab.xpos' argument when adding information with 'ilab'."))
+         #stop(mstyle$stop("Must specify the 'ilab.xpos' argument when adding information with 'ilab'."))
          dist <- min(ci.lb, na.rm=TRUE) - xlim[1]
          if (ncol.ilab == 1L)
             ilab.xpos <- xlim[1] + dist*0.75
@@ -853,13 +872,17 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
          if (ncol.ilab >= 4L)
             ilab.xpos <- seq(xlim[1] + dist*0.5, xlim[1] + dist*0.9, length.out=ncol.ilab)
       }
-      if (length(ilab.xpos) != ncol(ilab))
-         stop(mstyle$stop(paste0("Number of 'ilab' columns (", ncol(ilab), ") does not match length of 'ilab.xpos' argument (", length(ilab.xpos), ").")))
+      if (length(ilab.xpos) != ncol.ilab)
+         stop(mstyle$stop(paste0("Number of 'ilab' columns (", ncol.ilab, ") do not match the length of the 'ilab.xpos' argument (", length(ilab.xpos), ").")))
       if (!is.null(ilab.pos) && length(ilab.pos) == 1L)
-         ilab.pos <- rep(ilab.pos, ncol(ilab))
+         ilab.pos <- rep(ilab.pos, ncol.ilab)
+      if (!is.null(ilab.lab) && length(ilab.lab) != ncol.ilab)
+         stop(mstyle$stop(paste0("Number of 'ilab' columns (", ncol.ilab, ") do not match the length of the 'ilab.lab' argument (", length(ilab.lab), ").")))
       par(family=names(fonts)[3], font=fonts[3])
-      for (l in seq_len(ncol(ilab))) {
+      for (l in seq_len(ncol.ilab)) {
          ltext(ilab.xpos[l], rows+rowadj[3], ilab[,l], pos=ilab.pos[l], cex=cex, ...)
+         if (!is.null(ilab.lab))
+            ltext(ilab.xpos[l], ylim[2]-(top-1)+1+rowadj[3], ilab.lab[l], pos=ilab.pos[l], font=2, cex=cex, ...)
       }
       par(family=names(fonts)[1], font=fonts[1])
    }
@@ -896,10 +919,9 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
       if (missing(width)) {
          width <- apply(annotext, 2, function(x) max(nchar(x)))
       } else {
-         if (length(width) == 1L)
-            width <- rep(width, ncol(annotext))
+         width <- .expand1(width, ncol(annotext))
          if (length(width) != ncol(annotext))
-            stop(mstyle$stop(paste0("Length of 'width' argument (", length(width), ") does not match the number of annotation columns (", ncol(annotext), ").")))
+            stop(mstyle$stop(paste0("Length of the 'width' argument (", length(width), ") does not match the number of annotation columns (", ncol(annotext), ").")))
       }
 
       for (j in seq_len(ncol(annotext))) {
@@ -907,7 +929,7 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
       }
 
       if (showweights) {
-         annotext <- cbind(annotext[,1], paste0("%", rep(substr(annosym[1],1,1),3)), annotext[,2], annosym[1], annotext[,3], annosym[2], annotext[,4], annosym[3])
+         annotext <- cbind(annotext[,1], paste0("%", paste0(rep(substr(annosym[1],1,1),3), collapse="")), annotext[,2], annosym[1], annotext[,3], annosym[2], annotext[,4], annosym[3])
       } else {
          annotext <- cbind(annotext[,1], annosym[1], annotext[,2], annosym[2], annotext[,3], annosym[3])
       }
@@ -938,22 +960,24 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
    }
 
-   #lpoints(x=yi, y=rows, pch=pch, cex=cex*psize, ...)
-
    ### add header
 
-   ltext(textpos[1], ylim[2]-(top-1)+1, header.left,  pos=4, font=2, cex=cex, ...)
-   ltext(textpos[2], ylim[2]-(top-1)+1, header.right, pos=2, font=2, cex=cex, ...)
+   ltext(textpos[1], ylim[2]-(top-1)+1+rowadj[1], header.left,  pos=4, font=2, cex=cex, ...)
+   ltext(textpos[2], ylim[2]-(top-1)+1+rowadj[2], header.right, pos=2, font=2, cex=cex, ...)
 
    #########################################################################
 
    ### return some information about plot invisibly
 
-   res <- list(xlim=par("usr")[1:2], alim=alim, at=at, ylim=ylim, rows=rows, cex=cex, cex.lab=cex.lab, cex.axis=cex.axis, ilab.xpos=ilab.xpos, ilab.pos=ilab.pos, textpos=textpos)
+   res <- list(xlim=par("usr")[1:2], alim=alim, at=at, ylim=ylim, rows=rows,
+               cex=cex, cex.lab=cex.lab, cex.axis=cex.axis,
+               ilab.xpos=ilab.xpos, ilab.pos=ilab.pos, textpos=textpos)
 
-   ### add some additional stuff to be put into .metafor environment, so that it can be used by addpoly()
+   ### put some additional stuff into .metafor, so that it can be used by addpoly()
 
-   sav <- c(res, list(level=level, annotate=annotate, digits=digits[[1]], width=width, transf=transf, atransf=atransf, targs=targs, fonts=fonts[1:2], annosym=annosym))
+   sav <- c(res, list(level=level, annotate=annotate, digits=digits[[1]], width=width,
+                      transf=transf, atransf=atransf, targs=targs,
+                      fonts=fonts[1:2], annosym=annosym))
    try(assign("forest", sav, envir=.metafor), silent=TRUE)
 
    invisible(res)
