@@ -30,14 +30,59 @@ fmtp <- function(p, digits=4, pname="", equal=FALSE, sep=FALSE, add0=FALSE, quot
 
 }
 
+fmtp2 <- function(p, cutoff=c(0.001,0.06), pname="p", sep=TRUE, add0=FALSE, quote=FALSE) {
+
+   p[p < 0] <- 0
+   p[p > 1] <- 1
+
+   if (length(cutoff) == 1L)
+      stop("Argument 'cutoff' must be of length 2.")
+
+   cutoff <- sort(cutoff)
+
+   if (cutoff[1] == 0)
+      stop("The lower 'cutoff' value must be > 0.")
+
+   digits1 <- nchar(formatC(cutoff[1], format="f", digits=10, drop0trailing=T))-2
+   digits2 <- nchar(formatC(cutoff[2], format="f", digits=10, drop0trailing=T))-2
+
+   if (sep) {
+      if (pname != "")
+         pname <- paste0(pname, " ")
+      sep <- " "
+   } else {
+      sep <- ""
+   }
+
+   out <- sapply(p, function(x) {
+      if (is.na(x))
+         return(paste0(pname, "=", sep, "NA"))
+      if (x < cutoff[1]) {
+         return(paste0(pname, "<", sep, formatC(cutoff[1], digits=digits1, format="f")))
+      }
+      if (x < cutoff[2]) {
+         return(paste0(pname, "=", sep, formatC(x, digits=digits1, format="f")))
+      }
+      return(paste0(pname, "=", sep, formatC(x, digits=digits2, format="f")))
+   })
+
+   if (!add0)
+      out <- gsub("0.", ".", fixed=TRUE, out)
+
+   if (!quote)
+      out <- noquote(out)
+
+   return(out)
+
+}
+
 fmtx <- function(x, digits=4, flag="", quote=FALSE, ...) {
 
    # in case x is a data frame / matrix with two dimensions
 
    if (length(dim(x)) == 2L) {
 
-      if (length(digits) == 1L)
-         digits <- rep(digits, ncol(x))
+      digits <- .expand1(digits, ncol(x))
 
       out <- matrix("", nrow=nrow(x), ncol=ncol(x))
 
@@ -56,23 +101,9 @@ fmtx <- function(x, digits=4, flag="", quote=FALSE, ...) {
 
    ddd <- list(...)
 
-   if (is.null(ddd$addwidth)) {
-      width <- NULL
-   } else {
-      width <- digits + ddd$addwidth
-   }
-
-   if (is.null(ddd$drop0ifint)) {
-      drop0ifint <- FALSE
-   } else {
-      drop0ifint <- ddd$drop0ifint
-   }
-
-   if (is.null(ddd$add0)) {
-      add0 <- TRUE
-   } else {
-      add0 <- ddd$add0
-   }
+   width      <- .chkddd(ddd$addwidth,   NULL, digits + ddd$addwidth)
+   drop0ifint <- .chkddd(ddd$drop0ifint, FALSE)
+   add0       <- .chkddd(ddd$add0,       TRUE)
 
    if (!is.null(ddd$thresh)) {
       if (length(x) != 1L)
@@ -81,11 +112,7 @@ fmtx <- function(x, digits=4, flag="", quote=FALSE, ...) {
          digits <- 0
    }
 
-   if (is.null(ddd$postfix)) {
-      postfix <- ""
-   } else {
-      postfix <- ddd$postfix
-   }
+   postfix <- .chkddd(ddd$postfix, "")
 
    out <- sapply(x, function(x) {
       if (is.na(x))
@@ -127,11 +154,7 @@ fmtt <- function(val, tname, df, df1, df2, pval, digits=4, pname="p-val", format
 
    ddd <- list(...)
 
-   if (is.null(ddd$flag)) {
-      flag <- ""
-   } else {
-      flag <- ddd$flag
-   }
+   flag <- .chkddd(ddd$flag, "")
 
    if (length(digits) == 1L)
       digits <- c(test = digits, pval = digits)
