@@ -3,7 +3,7 @@ fsn <- function(x, vi, sei, subset, data, type, alpha=.05, target,
 
    #########################################################################
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    na.act <- getOption("na.action")
 
@@ -19,37 +19,15 @@ fsn <- function(x, vi, sei, subset, data, type, alpha=.05, target,
 
    .chkdots(ddd, c("pool", "mumiss", "interval", "maxint", "tol", "maxiter", "tau2", "test", "weighted"))
 
-   if (is.null(ddd$pool)) {
-      pool <- "stouffer"
-   } else {
-      pool <- match.arg(tolower(ddd$pool), c("stouffer", "fisher"))
-   }
+   pool <- .chkddd(ddd$pool, "stouffer", match.arg(tolower(ddd$pool), c("stouffer", "fisher")))
 
-   if (is.null(ddd$mumiss)) {
-      mumiss <- 0
-   } else {
-      mumiss <- ddd$mumiss
-   }
+   mumiss <- .chkddd(ddd$mumiss, 0)
 
    # note: default interval set below; see [a] (based on k)
 
-   if (is.null(ddd$maxint)) {
-      maxint <- 10^7
-   } else {
-      maxint <- ddd$maxint
-   }
-
-   if (is.null(ddd$tol)) {
-      tol <- .Machine$double.eps^0.25
-   } else {
-      tol <- ddd$tol
-   }
-
-   if (is.null(ddd$maxiter)) {
-      maxiter <- 1000
-   } else {
-      maxiter <- ddd$maxiter
-   }
+   maxint  <- .chkddd(ddd$maxint, 10^7)
+   tol     <- .chkddd(ddd$tol, .Machine$double.eps^0.25)
+   maxiter <- .chkddd(ddd$maxiter, 1000)
 
    ### observed values (to be replaced as needed)
 
@@ -157,13 +135,13 @@ fsn <- function(x, vi, sei, subset, data, type, alpha=.05, target,
       }
 
       if (type %in% c("Rosenthal", "Rosenberg", "General") && is.null(vi))
-         stop(mstyle$stop("Must specify 'vi' or 'sei' argument."))
+         stop(mstyle$stop("Must specify the 'vi' or 'sei' argument."))
 
       ### ensure backwards compatibility with the 'weighted' argument when type="Orwin"
 
       if (type == "Orwin") {
          if (isTRUE(ddd$weighted) && is.null(vi)) # if weighted=TRUE, then check that the vi's are available
-            stop(mstyle$stop("Must specify 'vi' or 'sei' argument."))
+            stop(mstyle$stop("Must specify the 'vi' or 'sei' argument."))
          if (isFALSE(ddd$weighted)) # if weighted=FALSE, then set vi <- 1 for unweighted
             vi <- 1
          if (is.null(ddd$weighted) && is.null(vi)) # if weighted is unspecified, set vi <- 1 if vi's are unspecified
@@ -172,13 +150,12 @@ fsn <- function(x, vi, sei, subset, data, type, alpha=.05, target,
 
       ### allow easy setting of vi to a single value
 
-      if (length(vi) == 1L)
-         vi <- rep(vi, length(yi))
+      vi <- .expand1(vi, length(yi))
 
       ### check length of yi and vi
 
       if (length(yi) != length(vi))
-         stop(mstyle$stop("Length of 'yi' and 'vi' (or 'sei') is not the same."))
+         stop(mstyle$stop("Length of 'yi' and 'vi' (or 'sei') are not the same."))
 
       ### check 'vi' argument for potential misuse
 
@@ -228,16 +205,12 @@ fsn <- function(x, vi, sei, subset, data, type, alpha=.05, target,
 
    k <- length(yi)
 
-   if (k == 1)
+   if (k == 1L)
       stop(mstyle$stop("Stopped because k = 1."))
 
    ### set interval for uniroot() [a]
 
-   if (is.null(ddd$interval)) {
-      interval <- c(0, k*50)
-   } else {
-      interval <- ddd$interval
-   }
+   interval  <- .chkddd(ddd$interval, c(0,k*50))
 
    #########################################################################
 
@@ -247,6 +220,7 @@ fsn <- function(x, vi, sei, subset, data, type, alpha=.05, target,
       z.avg  <- abs(sum(zi) / sqrt(k))
       pval   <- pnorm(z.avg, lower.tail=FALSE)
       fsnum  <- max(0, k * (z.avg / qnorm(alpha, lower.tail=FALSE))^2 - k)
+      fsnum  <- .rnd.fsn(fsnum)
       target <- NA_real_
 
    }
@@ -264,6 +238,7 @@ fsn <- function(x, vi, sei, subset, data, type, alpha=.05, target,
          if (inherits(fsnum, "try-error"))
             stop(mstyle$stop("Could not find fail-safe N using Fisher's method for pooling p-values."))
       }
+      fsnum  <- .rnd.fsn(fsnum)
       target <- NA_real_
 
    }
@@ -282,6 +257,7 @@ fsn <- function(x, vi, sei, subset, data, type, alpha=.05, target,
             pvalnew <- binom.test(kpos + fsnum/2, k + fsnum)$p.value
          }
       }
+      fsnum  <- .rnd.fsn(fsnum)
       target <- NA_real_
 
    }
@@ -302,6 +278,8 @@ fsn <- function(x, vi, sei, subset, data, type, alpha=.05, target,
          fsnum <- max(0, k * (est - target) / target)
       }
 
+      fsnum <- .rnd.fsn(fsnum)
+
    }
 
    if (type == "Rosenberg") {
@@ -314,6 +292,7 @@ fsn <- function(x, vi, sei, subset, data, type, alpha=.05, target,
       #w.p   <- (sum(wi*yi) / qnorm(alpha/2, lower.tail=FALSE))^2 - sum(wi)
       #fsnum <- max(0, k*w.p/sum(wi))
       fsnum  <- max(0, ((sum(wi*yi) / qnorm(alpha/2, lower.tail=FALSE))^2 - sum(wi)) * vt)
+      fsnum  <- .rnd.fsn(fsnum)
       target <- NA_real_
 
    }
@@ -358,7 +337,7 @@ fsn <- function(x, vi, sei, subset, data, type, alpha=.05, target,
       tmp <- try(rma(yi, vi, method=method, tau2=tau2fix, test=test, weighted=weighted, verbose=verbose), silent=!verbose)
 
       if (inherits(tmp, "try-error"))
-         stop(mstyle$stop("Could not fit random-effects model (use verbose=TRUE for more info)."), call.=FALSE)
+         stop(mstyle$stop("Could not fit random-effects model (use verbose=TRUE for more info)."))
 
       vt   <- 1 / mean(1/vi)
       est  <- tmp$beta[1]
@@ -389,6 +368,8 @@ fsn <- function(x, vi, sei, subset, data, type, alpha=.05, target,
             if (fsnum > maxint)
                fsnum <- maxint
 
+            fsnum <- .rnd.fsn(fsnum)
+
             tmp <- .fsn.gen(fsnum, yi=yi, vi=vi, vt=vt, est=est, tau2=tau2, tau2fix=tau2fix,
                             test=test, weighted=weighted, target=target, alpha=alpha, exact=exact,
                             method=method, mumiss=mumiss, upperint=max(interval), maxint=maxint, newest=TRUE)
@@ -419,6 +400,8 @@ fsn <- function(x, vi, sei, subset, data, type, alpha=.05, target,
             if (fsnum > maxint)
                fsnum <- maxint
 
+            fsnum <- .rnd.fsn(fsnum)
+
             tmp <- .fsn.gen(fsnum, yi=yi, vi=vi, vt=vt, est=est, tau2=tau2, tau2fix=tau2fix,
                             test=test, weighted=weighted, target=target, alpha=alpha, exact=exact,
                             method=method, mumiss=mumiss, upperint=max(interval), maxint=maxint, newest=TRUE)
@@ -444,12 +427,6 @@ fsn <- function(x, vi, sei, subset, data, type, alpha=.05, target,
 
    #########################################################################
 
-   if (is.finite(fsnum) && abs(fsnum - round(fsnum)) >= .Machine$double.eps^0.5) {
-      fsnum <- ceiling(fsnum)
-   } else {
-      fsnum <- round(fsnum)
-   }
-
    res <- list(type=type, fsnum=fsnum, est=est, tau2=tau2, meanes=est, pval=pval, alpha=alpha, target=target,
                method=ifelse(type=="General", method, NA), est.fsn=est.fsn, tau2.fsn=tau2.fsn, pval.fsn=pval.fsn,
                ub.sign=ub.sign, digits=digits)
