@@ -1,32 +1,31 @@
 influence.rma.uni <- function(model, digits, progbar=FALSE, ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
-   .chkclass(class(model), must="rma.uni", notav=c("rma.ls", "rma.uni.selmodel"))
+   .chkclass(class(model), must="rma.uni", notav=c("rma.ls", "rma.gen", "rma.uni.selmodel"))
 
    na.act <- getOption("na.action")
-   on.exit(options(na.action=na.act))
+   on.exit(options(na.action=na.act), add=TRUE)
 
    if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
       stop(mstyle$stop("Unknown 'na.action' specified under options()."))
 
+   if (is.null(model$yi) || is.null(model$vi))
+      stop(mstyle$stop("Information needed is not available in the model object."))
+
    x <- model
 
-   if (x$k == 1)
+   if (x$k == 1L)
       stop(mstyle$stop("Stopped because k = 1."))
 
    ddd <- list(...)
 
-   .chkdots(ddd, c("btt", "measure", "time"))
+   .chkdots(ddd, c("btt", "measure", "time", "code1", "code2"))
 
-   btt <- .set.btt(ddd$btt, x$p, int.incl=FALSE, Xnames=colnames(x$X))
+   btt <- .set.btt(ddd$btt, x$p, int.incl=FALSE, Xnames=colnames(x$X)) # note: 1:p by default (also in models with intercept)
    m <- length(btt)
 
-   if (is.null(ddd$measure)) {
-      measure <- "all"
-   } else {
-      measure <- ddd$measure
-   }
+   measure <- .chkddd(ddd$measure, "all")
 
    if (missing(digits)) {
       digits <- .get.digits(xdigits=x$digits, dmiss=TRUE)
@@ -40,6 +39,9 @@ influence.rma.uni <- function(model, digits, progbar=FALSE, ...) {
    if (.isTRUE(ddd$time))
       time.start <- proc.time()
 
+   if (!is.null(ddd[["code1"]]))
+      eval(expr = parse(text = ddd[["code1"]]))
+
    #########################################################################
 
    tau2.del <- rep(NA_real_, x$k)
@@ -80,6 +82,10 @@ influence.rma.uni <- function(model, digits, progbar=FALSE, ...) {
    hat <- hatvalues(x)
    options(na.action = na.act)
 
+   ### elements that need to be returned
+
+   outlist <- "coef.na=coef.na, tau2=tau2, QE=QE, beta=beta, vb=vb, s2w=s2w"
+
    ### note: skipping NA cases
    ### also: it is possible that model fitting fails, so that generates more NAs (these NAs will always be shown in output)
 
@@ -91,7 +97,12 @@ influence.rma.uni <- function(model, digits, progbar=FALSE, ...) {
       if (progbar)
          pbapply::setpb(pbar, i)
 
-      res <- try(suppressWarnings(rma.uni(x$yi, x$vi, weights=x$weights, mods=x$X, intercept=FALSE, method=x$method, weighted=x$weighted, test=x$test, level=x$level, tau2=ifelse(x$tau2.fix, x$tau2, NA), control=x$control, subset=-i, skipr2=TRUE)), silent=TRUE)
+      if (!is.null(ddd[["code2"]]))
+         eval(expr = parse(text = ddd[["code2"]]))
+
+      args <- list(yi=x$yi, vi=x$vi, weights=x$weights, mods=x$X, intercept=FALSE, method=x$method, weighted=x$weighted,
+                   test=x$test, level=x$level, tau2=ifelse(x$tau2.fix, x$tau2, NA), control=x$control, subset=-i, skipr2=TRUE, outlist=outlist)
+      res <- try(suppressWarnings(.do.call(rma.uni, args)), silent=TRUE)
 
       if (inherits(res, "try-error"))
          next
@@ -154,7 +165,8 @@ influence.rma.uni <- function(model, digits, progbar=FALSE, ...) {
 
       cook.d[i] <- crossprod(dfb,svb) %*% dfb # / x$p
 
-      #cook.d[i] <- sum(1/(x$vi+tau2.del[i]) * (pred.full - x$X %*% res$beta)^2) / x$p
+      #cook.d[i] <- sum(1/(x$vi+tau2.del[i]) * (pred.full - x$X %*% res$beta)^2) # / x$p
+      #cook.d[i] <- sum(1/(x$vi+x$tau2) * (pred.full - x$X %*% res$beta)^2) # / x$p
 
       ### compute covariance ratio
 
@@ -169,9 +181,9 @@ influence.rma.uni <- function(model, digits, progbar=FALSE, ...) {
 
    resid <- x$yi - delpred
    resid[abs(resid) < 100 * .Machine$double.eps] <- 0
-   #resid[abs(resid) < 100 * .Machine$double.eps * median(abs(resid), na.rm=TRUE)] <- 0 ### see lm.influence
+   #resid[abs(resid) < 100 * .Machine$double.eps * median(abs(resid), na.rm=TRUE)] <- 0 # see lm.influence
    #seresid <- sqrt(x$vi + vdelpred + tau2.del)
-   seresid <- sqrt(x$vi * s2w.del + vdelpred + tau2.del * s2w.del) ### this yields the same results as a mean shift outlier model when using test="knha"
+   seresid <- sqrt(x$vi * s2w.del + vdelpred + tau2.del * s2w.del) # this yields the same results as a mean shift outlier model when using test="knha"
    stresid <- resid / seresid
 
    ### extract weights
@@ -198,13 +210,13 @@ influence.rma.uni <- function(model, digits, progbar=FALSE, ...) {
    ### determine "influential" cases
 
    is.infl <-
-      #abs(inf$rstudent) > qnorm(.975) |
+      #abs(inf$rstudent) > qnorm(0.975) |
       abs(inf$dffits) > 3*sqrt(x$p/(x$k-x$p)) |
-      pchisq(inf$cook.d, df=m) > .50 |
+      pchisq(inf$cook.d, df=m) > 0.50 |
       #inf$cov.r > 1 + 3*m/(x$k-m) |
       #inf$cov.r < 1 - 3*m/(x$k-m) |
       inf$hat > 3*x$p/x$k |
-      apply(abs(dfbs) > 1, 1, any) ### consider using rowAnys() from matrixStats package
+      apply(abs(dfbs) > 1, 1, any) # consider using rowAnys() from matrixStats package
 
    #print(is.infl)
 
@@ -254,18 +266,18 @@ influence.rma.uni <- function(model, digits, progbar=FALSE, ...) {
    if (measure == "rstudent") {
 
       if (na.act == "na.omit") {
-         resid.f   <- resid
-         seresid.f <- seresid
-         stresid.f <- stresid
+         resid.f   <- c(resid)
+         seresid.f <- c(seresid)
+         stresid.f <- c(stresid)
       }
 
       if (na.act == "na.exclude" || na.act == "na.pass") {
          resid.f   <- rep(NA_real_, x$k.f)
          seresid.f <- rep(NA_real_, x$k.f)
          stresid.f <- rep(NA_real_, x$k.f)
-         resid.f[x$not.na]   <- resid
-         seresid.f[x$not.na] <- seresid
-         stresid.f[x$not.na] <- stresid
+         resid.f[x$not.na]   <- c(resid)
+         seresid.f[x$not.na] <- c(seresid)
+         stresid.f[x$not.na] <- c(stresid)
       }
 
       out <- list(resid=resid.f, se=seresid.f, z=stresid.f, slab=out$inf$slab, digits=digits)
