@@ -4,9 +4,9 @@
 ### Liao, J. G. & Rosen, O. (2001). Fast and stable algorithms for computing and sampling from the
 ### noncentral hypergeometric distribution. The American Statistician, 55, 366-369.
 
-.dnoncenhypergeom <- function (x=NA_real_, n1, n2, m1, psi) { ### x=ai, n1=ai+bi, n2=ci+di, m1=ai+ci, psi=ORi
+.dnoncenhypergeom <- function (x=NA_real_, n1, n2, m1, psi) { # x=ai, n1=ai+bi, n2=ci+di, m1=ai+ci, psi=ORi
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    mode.compute <- function(n1, n2, m1, psi, ll, uu) {
       a <- psi - 1
@@ -67,7 +67,7 @@
 
 .dnchgi <- function(logOR, ai, bi, ci, di, mu.i, tau2, random, dnchgcalc, dnchgprec) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    k <- length(logOR)
    dnchgi <- rep(NA_real_, k)
@@ -110,12 +110,12 @@
 
 .dnchg <- function(parms, ai, bi, ci, di, X.fit, random, verbose=FALSE, digits, dnchgcalc, dnchgprec, intCtrl) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    p    <- ncol(X.fit)
    k    <- length(ai)
-   beta <- parms[seq_len(p)]                  ### first p elemenets in parms are the model coefficients
-   tau2 <- ifelse(random, exp(parms[p+1]), 0) ### next value is tau^2 -- optimize over exp(tau^2) value or hold at 0 if random=FALSE
+   beta <- parms[seq_len(p)]                  # first p elemenets in parms are the model coefficients
+   tau2 <- ifelse(random, exp(parms[p+1]), 0) # next value is tau^2 -- optimize over exp(tau^2) value or hold at 0 if random=FALSE
    mu.i <- X.fit %*% cbind(beta)
 
    lli  <- rep(NA_real_, k)
@@ -136,10 +136,12 @@
       for (i in seq_len(k)) {
 
          res <- try(integrate(.dnchgi, lower=intCtrl$lower, upper=intCtrl$upper, ai=ai[i], bi=bi[i], ci=ci[i], di=di[i], mu.i=mu.i[i], tau2=tau2, random=random, dnchgcalc=dnchgcalc, dnchgprec=dnchgprec, rel.tol=intCtrl$rel.tol, subdivisions=intCtrl$subdivisions, stop.on.error=FALSE), silent=!verbose)
+         #res <- try(cubintegrate(.dnchgi, lower=intCtrl$lower, upper=intCtrl$upper, ai=ai[i], bi=bi[i], ci=ci[i], di=di[i], mu.i=mu.i[i], tau2=tau2, random=random, dnchgcalc=dnchgcalc, dnchgprec=dnchgprec), silent=!verbose)
 
          if (inherits(res, "try-error")) {
             stop(mstyle$stop(paste0("Could not integrate over density of non-central hypergeometric distribution in study ", i, ".")), call.=FALSE)
          } else {
+            #res$value <- res$integral
             if (res$value > 0) {
                lli[i] <- log(res$value)
             } else {
