@@ -37,8 +37,8 @@
       return(vcvals)
 
    if (length(ids) != length(vcvals)) {
-      mstyle <- .get.mstyle("crayon" %in% .packages())
-      stop(mstyle$stop(paste0("Length of 'vccon$", vcname, "' (", length(ids), ") does not match length of ", vcname, " (", length(vcvals), ").")), call.=FALSE)
+      mstyle <- .get.mstyle()
+      stop(mstyle$stop(paste0("Length of 'vccon$", vcname, "' (", length(ids), ") does not match the length of ", vcname, " (", length(vcvals), ").")), call.=FALSE)
    }
 
    for (id in unique(ids))
@@ -56,7 +56,7 @@
 
 .process.G.aftersub <- function(mf.g, struct, formula, tau2, rho, isG, k, sparse, verbose) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    if (verbose > 1)
       message(mstyle$message(paste0("Processing '", paste0(formula, collapse=""), "' term (#1) ...")))
@@ -74,7 +74,7 @@
 
    ### get variables names in mf.g
 
-   g.names <- names(mf.g) ### names for inner and outer factors/variables
+   g.names <- names(mf.g) # names for inner and outer factors/variables
 
    ### check that inner variable is a factor (or character variable) for structures that require this (no longer required)
 
@@ -107,7 +107,7 @@
 
    ### get number of levels of each variable in mf.g (vector with two values, for the inner and outer factor)
 
-   #g.nlevels <- c(nlevels(mf.g[[1]]), nlevels(mf.g[[2]])) ### works only for factors
+   #g.nlevels <- c(nlevels(mf.g[[1]]), nlevels(mf.g[[2]])) # works only for factors
    if (is.element(struct, c("SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYBM","PHYPL","PHYPD","GEN","GDIAG"))) {
       g.nlevels <- c(length(unique(apply(mf.g[-nvars], 1, paste, collapse=" + "))), length(unique(mf.g[[nvars]])))
    } else {
@@ -116,7 +116,7 @@
 
    ### get levels of each variable in mf.g
 
-   #g.levels <- list(levels(mf.g[[1]]), levels(mf.g[[2]])) ### works only for factors
+   #g.levels <- list(levels(mf.g[[1]]), levels(mf.g[[2]])) # works only for factors
    if (is.element(struct, c("SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYBM","PHYPL","PHYPD","GEN","GDIAG"))) {
       g.levels <- list(sort(unique(apply(mf.g[-nvars], 1, paste, collapse=" + "))), sort(unique((mf.g[[nvars]]))))
    } else {
@@ -167,20 +167,18 @@
 
    ### allow quickly setting all tau2 values to a fixed value
 
-   if (length(tau2) == 1L)
-      tau2 <- rep(tau2, tau2s)
+   tau2 <- .expand1(tau2, tau2s)
 
    ### allow quickly setting all rho values to a fixed value
 
-   if (length(rho) == 1L)
-      rho <- rep(rho, rhos)
+   rho <- .expand1(rho, rhos)
 
    ### check if tau2 and rho are of correct length
 
    if (length(tau2) != tau2s)
-      stop(mstyle$stop(paste0("Length of ", ifelse(isG, 'tau2', 'gamma2'), " argument (", length(tau2), ") does not match actual number of variance components (", tau2s, ").")), call.=FALSE)
+      stop(mstyle$stop(paste0("Length of the ", ifelse(isG, 'tau2', 'gamma2'), " argument (", length(tau2), ") does not match the actual number of variance components (", tau2s, ").")), call.=FALSE)
    if (length(rho) != rhos)
-      stop(mstyle$stop(paste0("Length of ", ifelse(isG, 'rho', 'phi'), " argument (", length(rho), ") does not match actual number of correlations (", rhos, ").")), call.=FALSE)
+      stop(mstyle$stop(paste0("Length of the ", ifelse(isG, 'rho', 'phi'), " argument (", length(rho), ") does not match the actual number of correlations (", rhos, ").")), call.=FALSE)
 
    ### checks on any fixed values of tau2 and rho arguments
 
@@ -252,9 +250,9 @@
 
 ############################################################################
 
-.process.G.afterrmna <- function(mf.g, g.nlevels, g.levels, g.values, struct, formula, tau2, rho, Z.G1, Z.G2, isG, sparse, distspec, verbose) {
+.process.G.afterrmna <- function(mf.g, g.nlevels, g.levels, g.values, struct, formula, tau2, rho, Z.G1, Z.G2, isG, sparse, distspec, check.k.gtr.1, verbose) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    if (verbose > 1)
       message(mstyle$message(paste0("Processing '", paste0(formula, collapse=""), "' term (#2) ...")))
@@ -279,7 +277,7 @@
 
    ### redo: get number of levels of each variable in mf.g (vector with two values, for the inner and outer factor)
 
-   #g.nlevels <- c(nlevels(mf.g[[1]]), nlevels(mf.g[[2]])) ### works only for factors
+   #g.nlevels <- c(nlevels(mf.g[[1]]), nlevels(mf.g[[2]])) # works only for factors
    if (is.element(struct, c("SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYBM","PHYPL","PHYPD","GEN","GDIAG"))) {
       g.nlevels <- c(length(unique(apply(mf.g[-nvars], 1, paste, collapse=" + "))), length(unique(mf.g[[nvars]])))
    } else {
@@ -288,7 +286,7 @@
 
    ### redo: get levels of each variable in mf.g
 
-   #g.levels <- list(levels(mf.g[[1]]), levels(mf.g[[2]])) ### works only for factors
+   #g.levels <- list(levels(mf.g[[1]]), levels(mf.g[[2]])) # works only for factors
    if (is.element(struct, c("SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYBM","PHYPL","PHYPD","GEN","GDIAG"))) {
       g.levels <- list(sort(unique(apply(mf.g[-nvars], 1, paste, collapse=" + "))), sort(unique((mf.g[[nvars]]))))
    } else {
@@ -327,18 +325,18 @@
    if (is.element(struct, c("SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYBM","PHYPL","PHYPD","GEN","GDIAG"))) {
       g.levels.k <- table(factor(apply(mf.g[-nvars], 1, paste, collapse=" + "), levels=g.levels.f[[1]]))
    } else {
-      g.levels.k <- table(factor(mf.g[[1]], levels=g.levels.f[[1]]))
+      #g.levels.k <- table(factor(mf.g[[1]], levels=g.levels.f[[1]]))
+      g.levels.k <- apply(table(factor(mf.g[[1]], levels=g.levels.f[[1]]), mf.g[[2]]), 1, function(x) sum(x>0L))
    }
 
    ### for "HCS","UN","DIAG","HAR": if a particular level of the inner factor only occurs once, then set corresponding tau2 value to 0 (if not already fixed)
-   ### note: no longer done; variance component should still be (weakly) identifiable
 
-   #if (is.element(struct, c("HCS","UN","DIAG","HAR"))) {
-   #   if (any(is.na(tau2) & g.levels.k == 1)) {
-   #      tau2[is.na(tau2) & g.levels.k == 1] <- 0
-   #      warning(mstyle$warning("Inner factor has k=1 for one or more levels. Corresponding 'tau2' value(s) fixed to 0."), call.=FALSE)
-   #   }
-   #}
+   if (is.element(struct, c("HCS","UN","DIAG","HAR")) && check.k.gtr.1) {
+      if (any(is.na(tau2) & g.levels.k == 1)) {
+         tau2[is.na(tau2) & g.levels.k == 1] <- 0
+         warning(mstyle$warning("Inner factor has k=1 for one or more levels. Corresponding 'tau2' value(s) fixed to 0."), call.=FALSE)
+      }
+   }
 
    ### check if each study has only a single arm (could be different arms!)
    ### for "CS","HCS","AR","HAR","CAR" must then fix rho to 0 (if not already fixed)
@@ -613,7 +611,7 @@
    if (any(g.levels.r) && struct == "UNR") {
       G[g.levels.r,] <- 0
       G[,g.levels.r] <- 0
-      diag(G) <- tau2 ### don't really need this
+      diag(G) <- tau2 # don't really need this
       rho <- G[lower.tri(G)]
       warning(mstyle$warning(paste0("Fixed ", ifelse(isG, 'rho', 'phi'), " value(s) to 0 for removed level(s).")), call.=FALSE)
    }
@@ -689,23 +687,23 @@
 
 ### function to construct var-cov matrix (G or H) for '~ inner | outer' terms
 
-.con.E <- function(v, r, v.val, r.val, Z1, Z2, levels.r, values, Dmat, struct, cholesky, vctransf, vccov, nearpd, sparse) {
+.con.E <- function(v, r, v.arg, r.arg, Z1, Z2, levels.r, values, Dmat, struct, cholesky, vctransf, vccov, nearpd, sparse) {
 
    ### if cholesky=TRUE, back-transformation/substitution is done below; otherwise, back-transform and replace fixed values
 
    if (!cholesky) {
       if (vctransf) {
-         v <- ifelse(is.na(v.val), exp(v), v.val)           # variances are optimized in log space, so exponentiate
+         v <- ifelse(is.na(v.arg), exp(v), v.arg)           # variances are optimized in log space, so exponentiate
          if (struct == "CAR")
-            r <- ifelse(is.na(r.val), plogis(r), r.val)     # CAR correlation is optimized in qlogis space, so use plogis
+            r <- ifelse(is.na(r.arg), plogis(r), r.arg)     # CAR correlation is optimized in qlogis space, so use plogis
          if (is.element(struct, c("SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYBM","PHYPL","PHYPD")))
-            r <- ifelse(is.na(r.val), exp(r), r.val)        # spatial and phylogenetic 'correlation' parameter is optimized in log space, so exponentiate
+            r <- ifelse(is.na(r.arg), exp(r), r.arg)        # spatial and phylogenetic 'correlation' parameter is optimized in log space, so exponentiate
          if (!is.element(struct, c("CAR","SPEXP","SPGAU","SPLIN","SPRAT","SPSPH","PHYBM","PHYPL","PHYPD")))
-            r <- ifelse(is.na(r.val), tanh(r), r.val)       # other correlations are optimized in atanh space, so use tanh
+            r <- ifelse(is.na(r.arg), tanh(r), r.arg)       # other correlations are optimized in atanh space, so use tanh
       } else {
          ### for Hessian computation, can choose to leave as is
-         v <- ifelse(is.na(v.val), v, v.val)
-         r <- ifelse(is.na(r.val), r, r.val)
+         v <- ifelse(is.na(v.arg), v, v.arg)
+         r <- ifelse(is.na(r.arg), r, r.arg)
          v[v < 0] <- 0
          if (struct == "CAR") {
             r[r < 0] <- 0
@@ -741,8 +739,8 @@
          E <- .con.vcov.UN.chol(v, r)
          v <- diag(E)                  # need this, so correct values are shown when verbose=TRUE
          r <- cov2cor(E)[lower.tri(E)] # need this, so correct values are shown when verbose=TRUE
-         v[!is.na(v.val)] <- v.val[!is.na(v.val)] # replace any fixed values
-         r[!is.na(r.val)] <- r.val[!is.na(r.val)] # replace any fixed values
+         v[!is.na(v.arg)] <- v.arg[!is.na(v.arg)] # replace any fixed values
+         r[!is.na(r.arg)] <- r.arg[!is.na(r.arg)] # replace any fixed values
       }
       E <- .con.vcov.UN(v, r, vccov)
       if (nearpd) {
@@ -757,8 +755,8 @@
          E <- .con.vcov.UNR.chol(v, r)
          v <- diag(E)[1,1]             # need this, so correct values are shown when verbose=TRUE
          r <- cov2cor(E)[lower.tri(E)] # need this, so correct values are shown when verbose=TRUE
-         v[!is.na(v.val)] <- v.val[!is.na(v.val)] # replace any fixed values
-         r[!is.na(r.val)] <- r.val[!is.na(r.val)] # replace any fixed values
+         v[!is.na(v.arg)] <- v.arg[!is.na(v.arg)] # replace any fixed values
+         r[!is.na(r.arg)] <- r.arg[!is.na(r.arg)] # replace any fixed values
       }
       E <- .con.vcov.UNR(v, r)
       if (nearpd) {
@@ -854,29 +852,35 @@
 
 ############################################################################
 
-### -1 times the log likelihood (regular or restricted) for rma.mv models
+### -1 times the log-likelihood (regular or restricted) for rma.mv models
 
 .ll.rma.mv <- function(par, reml, Y, M, A, X, k, pX, # note: pX due to nlm(); M=V to begin with
                        D.S, Z.G1, Z.G2, Z.H1, Z.H2, g.Dmat, h.Dmat,
-                       sigma2.val, tau2.val, rho.val, gamma2.val, phi.val, beta.val,
+                       sigma2.arg, tau2.arg, rho.arg, gamma2.arg, phi.arg, beta.arg,
                        sigma2s, tau2s, rhos, gamma2s, phis,
                        withS, withG, withH,
                        struct, g.levels.r, h.levels.r, g.values, h.values,
                        sparse, cholesky, nearpd, vctransf, vccov, vccon,
-                       verbose, digits, REMLf, dofit=FALSE, hessian=FALSE) {
+                       verbose, digits, REMLf,
+                       dofit=FALSE, hessian=FALSE, optbeta=FALSE, lambda=0, intercept=TRUE) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
-   ### only NA values in sigma2.val, tau2.val, rho.val, gamma2.val, phi.val should be estimated; otherwise, replace with fixed values
+   if (optbeta) {
+      beta <- par[1:pX]
+      par  <- par[-c(1:pX)]
+   }
+
+   ### only NA values in sigma2.arg, tau2.arg, rho.arg, gamma2.arg, phi.arg should be estimated; otherwise, replace with fixed values
 
    if (withS) {
 
       vars <- par[seq_len(sigma2s)]
 
       if (vctransf) {
-         sigma2 <- ifelse(is.na(sigma2.val), exp(vars), sigma2.val) # sigma2 is optimized in log space, so exponentiate
+         sigma2 <- ifelse(is.na(sigma2.arg), exp(vars), sigma2.arg) # sigma2 is optimized in log space, so exponentiate
       } else {
-         sigma2 <- ifelse(is.na(sigma2.val), vars, sigma2.val)      # for Hessian computation, can choose to leave as is
+         sigma2 <- ifelse(is.na(sigma2.arg), vars, sigma2.arg)      # for Hessian computation, can choose to leave as is
          sigma2[sigma2 < 0] <- 0
       }
 
@@ -903,7 +907,7 @@
       cors <- par[(sigma2s+tau2s+1):(sigma2s+tau2s+rhos)]
 
       resG <- .con.E(v=vars, r=cors,
-                     v.val=tau2.val, r.val=rho.val, Z1=Z.G1, Z2=Z.G2, levels.r=g.levels.r, values=g.values, Dmat=g.Dmat,
+                     v.arg=tau2.arg, r.arg=rho.arg, Z1=Z.G1, Z2=Z.G2, levels.r=g.levels.r, values=g.values, Dmat=g.Dmat,
                      struct=struct[1], cholesky=cholesky[1], vctransf=vctransf, vccov=vccov, nearpd=nearpd, sparse=sparse)
       tau2 <- resG$v
       rho  <- resG$r
@@ -923,7 +927,7 @@
          }
 
          resG <- .con.E(v=tau2, r=rho,
-                        v.val=tau2.val, r.val=rho.val, Z1=Z.G1, Z2=Z.G2, levels.r=g.levels.r, values=g.values, Dmat=g.Dmat,
+                        v.arg=tau2.arg, r.arg=rho.arg, Z1=Z.G1, Z2=Z.G2, levels.r=g.levels.r, values=g.values, Dmat=g.Dmat,
                         struct=struct[1], cholesky=FALSE, vctransf=FALSE, vccov=vccov, nearpd=nearpd, sparse=sparse)
          tau2 <- resG$v
          rho  <- resG$r
@@ -941,7 +945,7 @@
       cors <- par[(sigma2s+tau2s+rhos+gamma2s+1):(sigma2s+tau2s+rhos+gamma2s+phis)]
 
       resH <- .con.E(v=vars, r=cors,
-                     v.val=gamma2.val, r.val=phi.val, Z1=Z.H1, Z2=Z.H2, levels.r=h.levels.r, values=h.values, Dmat=h.Dmat,
+                     v.arg=gamma2.arg, r.arg=phi.arg, Z1=Z.H1, Z2=Z.H2, levels.r=h.levels.r, values=h.values, Dmat=h.Dmat,
                      struct=struct[2], cholesky=cholesky[2], vctransf=vctransf, vccov=vccov, nearpd=nearpd, sparse=sparse)
       gamma2 <- resH$v
       phi    <- resH$r
@@ -962,7 +966,7 @@
          }
 
          resH <- .con.E(v=gamma2, r=phi,
-                        v.val=gamma2.val, r.val=phi.val, Z1=Z.H1, Z2=Z.H2, levels.r=h.levels.r, values=h.values, Dmat=h.Dmat,
+                        v.arg=gamma2.arg, r.arg=phi.arg, Z1=Z.H1, Z2=Z.H2, levels.r=h.levels.r, values=h.values, Dmat=h.Dmat,
                         struct=struct[2], cholesky=FALSE, vctransf=FALSE, vccov=vccov, nearpd=nearpd, sparse=sparse)
          gamma2 <- resH$v
          phi    <- resH$r
@@ -990,6 +994,8 @@
    if (nearpd)
       M <- as.matrix(nearPD(M)$mat)
 
+   ### compute W = M^-1 via Cholesky decomposition
+
    if (verbose > 1) {
       W <- try(chol2inv(chol(M)), silent=FALSE)
    } else {
@@ -1000,93 +1006,86 @@
 
    if (inherits(W, "try-error")) {
 
-      ### if M is not positive-definite, set the (restricted) log likelihood to -Inf
+      ### if M is not positive-definite, set the (restricted) log-likelihood to -Inf
       ### this idea is based on: https://stats.stackexchange.com/q/11368/1934 (this is crude, but should
       ### move the parameter estimates away from values that create the non-positive-definite M matrix)
 
       if (dofit) {
-         stop(mstyle$stop("Final variance-covariance matrix not positive definite."), call.=FALSE)
+         stop(mstyle$stop("Final variance-covariance matrix is not positive definite."), call.=FALSE)
       } else {
          llval <- -Inf
       }
 
    } else {
 
-      if (verbose > 1) {
-         U <- try(chol(W), silent=FALSE)
+      if (!dofit || is.null(A)) {
+
+         stXWX <- chol2inv(chol(as.matrix(t(X) %*% W %*% X))) # TODO: catch if this fails
+         if (!optbeta)
+            beta <- matrix(stXWX %*% crossprod(X,W) %*% Y, ncol=1)
+         beta  <- ifelse(is.na(beta.arg), beta, beta.arg)
+         RSS   <- as.vector(t(Y - X %*% beta) %*% W %*% (Y - X %*% beta))
+         if (optbeta && lambda > 0) {
+            if (intercept) {
+               RSS <- RSS + c(lambda * crossprod(beta[-1]))
+               #RSS <- RSS + c(lambda * sum(abs(beta[-1])))
+            } else {
+               RSS <- RSS + c(lambda * crossprod(beta))
+               #RSS <- RSS + c(lambda * sum(abs(beta)))
+            }
+         }
+         vb    <- stXWX
+
       } else {
-         U <- try(suppressWarnings(chol(W)), silent=TRUE)
+
+         stXAX <- chol2inv(chol(as.matrix(t(X) %*% A %*% X))) # TODO: catch if this fails
+         beta  <- matrix(stXAX %*% crossprod(X,A) %*% Y, ncol=1)
+         beta  <- ifelse(is.na(beta.arg), beta, beta.arg)
+         RSS   <- as.vector(t(Y - X %*% beta) %*% W %*% (Y - X %*% beta))
+         if (optbeta && lambda > 0) {
+            if (intercept) {
+               RSS <- RSS + c(lambda * crossprod(beta[-1]))
+            } else {
+               RSS <- RSS + c(lambda * crossprod(beta))
+            }
+         }
+         vb    <- matrix(stXAX %*% t(X) %*% A %*% M %*% A %*% X %*% stXAX, nrow=pX, ncol=pX)
+
       }
 
-      ### Y ~ N(Xbeta, M), so UY ~ N(UXbeta, UMU) where UMU = I
-      ### return(U %*% M %*% U)
+      llvals <- c(NA_real_, NA_real_)
 
-      if (inherits(U, "try-error")) {
+      if (dofit || !reml)
+         llvals[1]  <- -1/2 * (k) * log(2*base::pi) - 1/2 * determinant(M, logarithm=TRUE)$modulus - 1/2 * RSS
 
-         if (dofit) {
-            stop(mstyle$stop("Cannot fit model based on estimated marginal variance-covariance matrix."), call.=FALSE)
-         } else {
-            llval <- -Inf
+      if (dofit || reml)
+         llvals[2]  <- -1/2 * (k-pX) * log(2*base::pi) + ifelse(REMLf, 1/2 * determinant(crossprod(X), logarithm=TRUE)$modulus, 0) +
+                       -1/2 * determinant(M, logarithm=TRUE)$modulus - 1/2 * determinant(crossprod(X,W) %*% X, logarithm=TRUE)$modulus - 1/2 * RSS
+
+      if (dofit) {
+
+         res <- list(beta=beta, vb=vb, M=M, llvals=llvals)
+
+         if (withS)
+            res$sigma2 <- sigma2
+
+         if (withG) {
+            res$G <- G
+            res$tau2 <- tau2
+            res$rho <- rho
          }
 
+         if (withH) {
+            res$H <- H
+            res$gamma2 <- gamma2
+            res$phi <- phi
+         }
+
+         return(res)
+
       } else {
 
-         if (!dofit || is.null(A)) {
-
-            sX   <- U %*% X
-            sY   <- U %*% Y
-            beta <- solve(crossprod(sX), crossprod(sX, sY))
-            beta <- ifelse(is.na(beta.val), beta, beta.val)
-            RSS  <- sum(as.vector(sY - sX %*% beta)^2)
-            if (dofit)
-               vb <- matrix(solve(crossprod(sX)), nrow=pX, ncol=pX)
-
-         } else {
-
-            stXAX <- chol2inv(chol(as.matrix(t(X) %*% A %*% X)))
-            #stXAX <- tcrossprod(qr.solve(sX, diag(k)))
-            beta  <- matrix(stXAX %*% crossprod(X,A) %*% Y, ncol=1)
-            beta  <- ifelse(is.na(beta.val), beta, beta.val)
-            RSS   <- as.vector(t(Y - X %*% beta) %*% W %*% (Y - X %*% beta))
-            vb    <- matrix(stXAX %*% t(X) %*% A %*% M %*% A %*% X %*% stXAX, nrow=pX, ncol=pX)
-
-         }
-
-         llvals <- c(NA_real_, NA_real_)
-
-         if (dofit || !reml)
-            llvals[1]  <- -1/2 * (k) * log(2*base::pi) - 1/2 * determinant(M, logarithm=TRUE)$modulus - 1/2 * RSS
-
-         if (dofit || reml)
-            llvals[2]  <- -1/2 * (k-pX) * log(2*base::pi) + ifelse(REMLf, 1/2 * determinant(crossprod(X), logarithm=TRUE)$modulus, 0) +
-                          -1/2 * determinant(M, logarithm=TRUE)$modulus - 1/2 * determinant(crossprod(X,W) %*% X, logarithm=TRUE)$modulus - 1/2 * RSS
-
-         if (dofit) {
-
-            res <- list(beta=beta, vb=vb, M=M, llvals=llvals)
-
-            if (withS)
-               res$sigma2 <- sigma2
-
-            if (withG) {
-               res$G <- G
-               res$tau2 <- tau2
-               res$rho <- rho
-            }
-
-            if (withH) {
-               res$H <- H
-               res$gamma2 <- gamma2
-               res$phi <- phi
-            }
-
-            return(res)
-
-         } else {
-
-            llval <- ifelse(reml, llvals[2], llvals[1])
-
-         }
+         llval <- ifelse(reml, llvals[2], llvals[1])
 
       }
 
@@ -1099,7 +1098,6 @@
          iteration <- .getfromenv("iteration", default=NULL)
 
          if (!is.null(iteration)) {
-            #cat(mstyle$verbose(paste0("Iteration ", iteration, "\t")))
             cat(mstyle$verbose(paste0("Iteration ", formatC(iteration, width=5, flag="-", format="f", digits=0), " ")))
             try(assign("iteration", iteration+1, envir=.metafor), silent=TRUE)
          }
@@ -1127,11 +1125,14 @@
 
 ############################################################################
 
-.cooks.distance.rma.mv <- function(i, obj, parallel, svb, cluster, ids, reestimate, btt) {
+.cooks.distance.rma.mv <- function(i, obj, parallel, svb, cluster, ids, reestimate, btt, code2=NULL) {
 
    if (parallel == "snow")
       library(metafor)
 
+   if (!is.null(code2))
+      eval(expr = parse(text = code2))
+
    incl <- cluster %in% ids[i]
 
    ### elements that need to be returned
@@ -1157,7 +1158,7 @@
                    test=obj$test, dfs=obj$dfs, level=obj$level, R=obj$R, Rscale=obj$Rscale,
                    sigma2=ifelse(obj$vc.fix$sigma2, obj$sigma2, NA), tau2=ifelse(obj$vc.fix$tau2, obj$tau2, NA), rho=ifelse(obj$vc.fix$rho, obj$rho, NA),
                    gamma2=ifelse(obj$vc.fix$gamma2, obj$gamma2, NA), phi=ifelse(obj$vc.fix$phi, obj$phi, NA),
-                   sparse=obj$sparse, dist=obj$dist, vccon=obj$vccon, control=control, subset=!incl, outlist=outlist)
+                   sparse=obj$sparse, dist=obj$dist, vccon=obj$vccon, optbeta=obj$optbeta, control=control, subset=!incl, outlist=outlist)
       res <- try(suppressWarnings(.do.call(rma.mv, args)), silent=TRUE)
 
    } else {
@@ -1167,7 +1168,7 @@
       args <- list(yi=obj$yi, V=obj$V, W=obj$W, mods=obj$X, random=obj$random, struct=obj$struct, intercept=FALSE, data=obj$mf.r, method=obj$method,
                    test=obj$test, dfs=obj$dfs, level=obj$level, R=obj$R, Rscale=obj$Rscale,
                    sigma2=obj$sigma2, tau2=obj$tau2, rho=obj$rho, gamma2=obj$gamma2, phi=obj$phi,
-                   sparse=obj$sparse, dist=obj$dist, vccon=obj$vccon, control=obj$control, subset=!incl, outlist=outlist)
+                   sparse=obj$sparse, dist=obj$dist, vccon=obj$vccon, optbeta=obj$optbeta, control=obj$control, subset=!incl, outlist=outlist)
       res <- try(suppressWarnings(.do.call(rma.mv, args)), silent=TRUE)
 
    }
@@ -1190,11 +1191,14 @@
 
 }
 
-.rstudent.rma.mv <- function(i, obj, parallel, cluster, ids, reestimate) {
+.rstudent.rma.mv <- function(i, obj, parallel, cluster, ids, reestimate, code2=NULL) {
 
    if (parallel == "snow")
       library(metafor)
 
+   if (!is.null(code2))
+      eval(expr = parse(text = code2))
+
    incl <- cluster %in% ids[i]
 
    k.id <- sum(incl)
@@ -1220,7 +1224,7 @@
                    test=obj$test, dfs=obj$dfs, level=obj$level, R=obj$R, Rscale=obj$Rscale,
                    sigma2=ifelse(obj$vc.fix$sigma2, obj$sigma2, NA), tau2=ifelse(obj$vc.fix$tau2, obj$tau2, NA), rho=ifelse(obj$vc.fix$rho, obj$rho, NA),
                    gamma2=ifelse(obj$vc.fix$gamma2, obj$gamma2, NA), phi=ifelse(obj$vc.fix$phi, obj$phi, NA),
-                   sparse=obj$sparse, dist=obj$dist, vccon=obj$vccon, control=control, subset=!incl, outlist=outlist)
+                   sparse=obj$sparse, dist=obj$dist, vccon=obj$vccon, optbeta=obj$optbeta, control=control, subset=!incl, outlist=outlist)
       res <- try(suppressWarnings(.do.call(rma.mv, args)), silent=TRUE)
 
    } else {
@@ -1230,7 +1234,7 @@
       args <- list(yi=obj$yi, V=obj$V, W=obj$W, mods=obj$X, random=obj$random, struct=obj$struct, intercept=FALSE, data=obj$mf.r, method=obj$method,
                    test=obj$test, dfs=obj$dfs, level=obj$level, R=obj$R, Rscale=obj$Rscale,
                    sigma2=obj$sigma2, tau2=obj$tau2, rho=obj$rho, gamma2=obj$gamma2, phi=obj$phi,
-                   sparse=obj$sparse, dist=obj$dist, vccon=obj$vccon, control=obj$control, subset=!incl, outlist=outlist)
+                   sparse=obj$sparse, dist=obj$dist, vccon=obj$vccon, optbeta=obj$optbeta, control=obj$control, subset=!incl, outlist=outlist)
       res <- try(suppressWarnings(.do.call(rma.mv, args)), silent=TRUE)
 
    }
@@ -1252,7 +1256,7 @@
    args <- list(yi=obj$yi, V=obj$V, W=obj$W, mods=obj$X, random=obj$random, struct=obj$struct, intercept=FALSE, data=obj$mf.r, method=obj$method,
                 test=obj$test, dfs=obj$dfs, level=obj$level, R=obj$R, Rscale=obj$Rscale,
                 sigma2=res$sigma2, tau2=res$tau2, rho=res$rho, gamma2=res$gamma2, phi=res$phi,
-                sparse=obj$sparse, dist=obj$dist, vccon=obj$vccon, control=obj$control, outlist=outlist)
+                sparse=obj$sparse, dist=obj$dist, vccon=obj$vccon, optbeta=obj$optbeta, control=obj$control, outlist=outlist)
    tmp <- try(suppressWarnings(.do.call(rma.mv, args)), silent=TRUE)
 
    Xi <- obj$X[incl,,drop=FALSE]
@@ -1276,11 +1280,14 @@
 
 }
 
-.dfbetas.rma.mv <- function(i, obj, parallel, cluster, ids, reestimate) {
+.dfbetas.rma.mv <- function(i, obj, parallel, cluster, ids, reestimate, code2=NULL) {
 
    if (parallel == "snow")
       library(metafor)
 
+   if (!is.null(code2))
+      eval(expr = parse(text = code2))
+
    incl <- cluster %in% ids[i]
 
    ### elements that need to be returned
@@ -1304,7 +1311,7 @@
                    test=obj$test, dfs=obj$dfs, level=obj$level, R=obj$R, Rscale=obj$Rscale,
                    sigma2=ifelse(obj$vc.fix$sigma2, obj$sigma2, NA), tau2=ifelse(obj$vc.fix$tau2, obj$tau2, NA), rho=ifelse(obj$vc.fix$rho, obj$rho, NA),
                    gamma2=ifelse(obj$vc.fix$gamma2, obj$gamma2, NA), phi=ifelse(obj$vc.fix$phi, obj$phi, NA),
-                   sparse=obj$sparse, dist=obj$dist, vccon=obj$vccon, control=control, subset=!incl, outlist=outlist)
+                   sparse=obj$sparse, dist=obj$dist, vccon=obj$vccon, optbeta=obj$optbeta, control=control, subset=!incl, outlist=outlist)
       res <- try(suppressWarnings(.do.call(rma.mv, args)), silent=TRUE)
 
    } else {
@@ -1314,7 +1321,7 @@
       args <- list(yi=obj$yi, V=obj$V, W=obj$W, mods=obj$X, random=obj$random, struct=obj$struct, intercept=FALSE, data=obj$mf.r, method=obj$method,
                    test=obj$test, dfs=obj$dfs, level=obj$level, R=obj$R, Rscale=obj$Rscale,
                    sigma2=obj$sigma2, tau2=obj$tau2, rho=obj$rho, gamma2=obj$gamma2, phi=obj$phi,
-                   sparse=obj$sparse, dist=obj$dist, vccon=obj$vccon, control=obj$control, subset=!incl, outlist=outlist)
+                   sparse=obj$sparse, dist=obj$dist, vccon=obj$vccon, optbeta=obj$optbeta, control=obj$control, subset=!incl, outlist=outlist)
       res <- try(suppressWarnings(.do.call(rma.mv, args)), silent=TRUE)
 
    }
@@ -1336,7 +1343,7 @@
    args <- list(yi=obj$yi, V=obj$V, W=obj$W, mods=obj$X, random=obj$random, struct=obj$struct, intercept=FALSE, data=obj$mf.r, method=obj$method,
                 test=obj$test, dfs=obj$dfs, level=obj$level, R=obj$R, Rscale=obj$Rscale,
                 sigma2=res$sigma2, tau2=res$tau2, rho=res$rho, gamma2=res$gamma2, phi=res$phi,
-                sparse=obj$sparse, dist=obj$dist, vccon=obj$vccon, control=obj$control, outlist=outlist)
+                sparse=obj$sparse, dist=obj$dist, vccon=obj$vccon, optbeta=obj$optbeta, control=obj$control, outlist=outlist)
    tmp <- try(suppressWarnings(.do.call(rma.mv, args)), silent=TRUE)
 
    ### compute dfbeta value(s)
@@ -1355,16 +1362,15 @@
 
 .ddf.calc <- function(dfs, X, k, p, mf.s=NULL, mf.g=NULL, mf.h=NULL, beta=TRUE) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    if (beta) {
 
       if (is.numeric(dfs)) {
          ddf <- dfs
-         if (length(ddf) == 1L)
-            ddf <- rep(ddf, p)
+         ddf <- .expand1(ddf, p)
          if (length(ddf) != p)
-            stop(mstyle$stop(paste0("Length of 'dfs' argument (", length(dfs), ") does not match the number of model coefficient (", p, ").")), call.=FALSE)
+            stop(mstyle$stop(paste0("Length of the 'dfs' argument (", length(dfs), ") does not match the number of model coefficient (", p, ").")), call.=FALSE)
       }
 
       if (is.character(dfs) && dfs == "residual")
