@@ -4,18 +4,18 @@
 
 .set.btt <- function(btt, p, int.incl, Xnames, fixed=FALSE) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    if (missing(btt) || is.null(btt)) {
 
-      if (p > 1L) {                       ### if the model matrix has more than one column
+      if (p > 1L) {                       # if the model matrix has more than one column
          if (int.incl) {
-            btt <- seq.int(from=2, to=p)     ### and the model has an intercept term, test all coefficients except the intercept
+            btt <- seq.int(from=2, to=p)     # and the model has an intercept term, test all coefficients except the intercept
          } else {
-            btt <- seq_len(p)                ### and the model does not have an intercept term, test all coefficients
+            btt <- seq_len(p)                # and the model does not have an intercept term, test all coefficients
          }
       } else {
-         btt <- 1L                        ### if the model matrix has a single column, test that single coefficient
+         btt <- 1L                        # if the model matrix has a single column, test that single coefficient
       }
 
    } else {
@@ -44,7 +44,7 @@
 
          ### make sure that at least one valid value is left
          if (length(btt) == 0L)
-            stop(mstyle$stop("Non-existent coefficients specified via 'btt'."), call.=FALSE)
+            stop(mstyle$stop("Non-existent coefficient(s) specified via 'btt'."), call.=FALSE)
 
       }
 
@@ -96,38 +96,83 @@
 
 ### pairwise sorting of the elements of two vectors
 
-.psort <- function(x,y) {
+#.psort.old <- function(x, y) {
+#
+#   if (is.null(x) || length(x) == 0L) # need to catch this
+#      return(NULL)
+#
+#   if (missing(y)) {
+#      if (is.matrix(x)) {
+#         xy <- x
+#      } else {
+#         xy <- rbind(x) # in case x is just a vector
+#      }
+#   } else {
+#      xy <- cbind(x,y)
+#   }
+#
+#   n <- nrow(xy)
+#
+#   for (i in seq_len(n)) {
+#      if (anyNA(xy[i,]))
+#         next
+#      xy[i,] <- sort(xy[i,])
+#   }
+#
+#   colnames(xy) <- NULL
+#
+#   return(xy)
+#
+#}
 
-   ### t(apply(xy, 1, sort)) would be okay, but problematic if there are NAs;
-   ### either they are removed completely (na.last=NA) or they are always put
-   ### first/last (na.last=FALSE/TRUE); but we just want to leave the NAs in
-   ### their position!
+.psort <- function(x, y, as.list=FALSE) {
 
-   if (is.null(x) || length(x) == 0L) ### need to catch this
+   # simpler / vectorized version that also deals with x and y being matrices
+   # (of the same dimensions) for elementwise swapping of pairs as needed
+
+   # t(apply(xy, 1, sort)) would be okay, but problematic if there are NAs;
+   # either they are removed completely (na.last=NA) or they are always put
+   # first/last (na.last=FALSE/TRUE); but we just want to leave the NAs in
+   # their position!
+
+   if (is.null(x) || length(x) == 0L) # need to catch this
       return(NULL)
 
    if (missing(y)) {
       if (is.matrix(x)) {
-         xy <- x
+         y <- x[,2]
+         x <- x[,1]
       } else {
-         xy <- rbind(x) ### in case x is just a vector
+         y <- x[2]
+         x <- x[1]
       }
+   }
+
+   flip <- x > y
+   flip[is.na(flip)] <- FALSE
+
+   x.flip <- x
+   y.flip <- y
+
+   x.flip[flip] <- y[flip]
+   y.flip[flip] <- x[flip]
+
+   if (as.list) {
+      return(list(x=x.flip, y=y.flip))
    } else {
-      xy <- cbind(x,y)
+      return(unname(cbind(x.flip, y.flip)))
    }
 
-   n <- nrow(xy)
+}
 
-   for (i in seq_len(n)) {
-      if (anyNA(xy[i,]))
-         next
-      xy[i,] <- sort(xy[i,])
-   }
+############################################################################
 
-   colnames(xy) <- NULL
-
-   return(xy)
+### function for applying observation limits
 
+.applyolim <- function(x, olim) {
+   x[x < olim[1]] <- olim[1]
+   x[x > olim[2]] <- olim[2]
+   return(x)
 }
 
 ############################################################################
@@ -192,11 +237,72 @@
    is.numeric(x)
 }
 
+### sapply()-like function but for matrices that always preserves the matrix dimensions (used in traceplot.rma.uni())
+
+.matapply <- function(x, FUN, targs=NULL) {
+   if (is.null(x))
+      return(NULL)
+   if (is.null(targs)) {
+      x[] <- sapply(x, FUN)
+   } else {
+      x[] <- sapply(x, FUN, targs)
+   }
+   return(x)
+}
+
+### check if ddd element is NULL; if so, return ifnull, otherwise the ddd element or ifnot
+
+.chkddd <- function(x, ifnull=NULL, ifnot=NULL) {
+
+   if (is.null(x)) {
+      return(ifnull)
+   } else {
+      if (is.null(ifnot)) {
+         return(x)
+      } else {
+         return(ifnot)
+      }
+   }
+
+}
+
+### function that expands a scalar to length k; can also expand a scalar to
+### the maximum length of the list elements given to k
+
+.expand1 <- function(x, k) {
+
+   if (is.list(k))
+      k <- max(lengths(k, use.names=FALSE))
+
+   if (length(x) == 1L)
+      x <- rep(x, k)
+
+   return(x)
+
+}
+
+### function that takes a vector as input and creates an expanded vector of
+### the length of 'fill' of all NAs but where the fill values are given by x
+### (can also take an entire list as input)
+
+.expandna <- function(x, fill) {
+   if (is.list(x)) {
+      return(lapply(x, function(xi) .expandna(xi, fill)))
+   } else {
+      if (!is.logical(fill))
+         stop("Argument 'fill' is not a logical vector.")
+      k <- length(fill)
+      out <- rep(NA_real_, k)
+      out[fill] <- x
+      return(out)
+   }
+}
+
 ############################################################################
 
 ### function to format p-values (no longer used; use fmtp() instead)
-### if showeq=FALSE, c(.001, .00001) becomes c("0.0010", "<.0001")
-### if showeq=TRUE,  c(.001, .00001) becomes c("=0.0010", "<.0001")
+### if showeq=FALSE, c(0.001, 0.00001) becomes c("0.0010", "<.0001")
+### if showeq=TRUE,  c(0.001, 0.00001) becomes c("=0.0010", "<.0001")
 ### if add0=FALSE, "<.0001"; if add0=TRUE, "<0.0001"
 
 .pval <- function(p, digits=4, showeq=FALSE, sep="", add0=FALSE) {
@@ -225,19 +331,26 @@
 
 ### function to handle 'level' argument
 
-.level <- function(level, allow.vector=FALSE) {
+.level <- function(level, allow.vector=FALSE, argname="level", stopon100=FALSE) {
 
-   if (!allow.vector && length(level) != 1L) {
-      mstyle <- .get.mstyle("crayon" %in% .packages())
-      stop(mstyle$stop("Argument 'level' must specify a single value."), call.=FALSE)
-   }
+   if (is.null(level))
+      return(NULL)
 
-   if (!is.numeric(level)) {
-      mstyle <- .get.mstyle("crayon" %in% .packages())
-      stop(mstyle$stop("The 'level' argument must be numeric."), call.=FALSE)
-   }
+   mstyle <- .get.mstyle()
 
-   ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > .5, 1-level, level)))
+   if (any(level > 100) || any(level < 0))
+      stop(mstyle$stop(paste0("Argument '", argname, "' must be between 0 and 100.")), call.=FALSE)
+
+   if (isTRUE(stopon100) && any(level==100))
+      stop(mstyle$stop(paste0("Argument '", argname, "' cannot be equal to 100.")), call.=FALSE)
+
+   if (!allow.vector && length(level) != 1L)
+      stop(mstyle$stop(paste0("Argument '", argname, "' must specify a single value.")), call.=FALSE)
+
+   if (!is.numeric(level))
+      stop(mstyle$stop(paste0("The '", argname, "' argument must be numeric.")), call.=FALSE)
+
+   ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > 0.5, 1-level, level)))
 
 }
 
@@ -334,7 +447,7 @@
 
 .print.time <- function(x) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    hours   <- floor(x/60/60)
    minutes <- floor(x/60) - hours*60
@@ -360,7 +473,8 @@
    ux <- unique(x)
 
    for (i in seq_along(ux)) {
-      xiTF <- x == ux[i]
+      #xiTF <- x == ux[i]
+      xiTF <- x %in% ux[i] # works also with NAs in vector (multiple NAs are then NA.1, NA.2, ...)
       xi <- x[xiTF]
       if (length(xi) == 1L)
          next
@@ -381,7 +495,7 @@
       ddd[okargs[i]] <- NULL
 
    if (length(ddd) > 0L) {
-      mstyle <- .get.mstyle("crayon" %in% .packages())
+      mstyle <- .get.mstyle()
       warning(mstyle$warning(paste0("Extra argument", ifelse(length(ddd) > 1L, "s ", " "), "(", paste0("'", names(ddd), "'", collapse=", "), ") disregarded.")), call.=FALSE)
    }
 
@@ -391,7 +505,7 @@
 
 .getx <- function(x, mf, data, enclos=sys.frame(sys.parent(n=2)), checknull=TRUE, checknumeric=FALSE, default) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    mf.getx <- match.call()
    dname <- deparse1(mf.getx[[match("data", names(mf.getx))]])
@@ -477,7 +591,7 @@
 
 .chkclass <- function(class, must, notap, notav, type="Method") {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    obj <- as.character(match.call()[2])
    obj <- substr(obj, 7, nchar(obj)-1)
@@ -533,8 +647,8 @@
          ok <- TRUE
 
       if (!ok) {
-         mstyle <- .get.mstyle("crayon" %in% .packages())
-         warning(mstyle$warning(paste0("The 'vi' argument is for specifying the sampling variances,\nbut '", x, "' sounds like this variable may contain standard\nerrors (maybe use 'sei=", x, "' instead?).")), call.=FALSE)
+         mstyle <- .get.mstyle()
+         warning(mstyle$warning(paste0("The 'vi' argument should be used to specify sampling variances,\nbut '", x, "' sounds like this variable may contain standard\nerrors (maybe use 'sei=", x, "' instead?).")), call.=FALSE)
          try(assign("runvicheck", FALSE, envir=.metafor), silent=TRUE)
       }
 
@@ -553,7 +667,7 @@
    if (all(ks == 0L)) { # if all elements have length 0 (are NULL), return TRUE
       return(TRUE)
    } else {
-      ks <- ks[ks > 0L]  # keep the non-zero lengths
+      ks <- ks[ks > 0L] # keep the non-zero lengths
       return(length(unique(ks)) == 1L) # check that they are all identical
    }
 
@@ -816,14 +930,14 @@
          }
       }
       ######################################################################
-      if (measure == "R2") {
+      if (is.element(measure, c("R2","R2F"))) {
          if (identical(transf.char, "FALSE") && identical(atransf.char, "FALSE")) {
             lab <- ifelse(short, expression(R^2), "Coefficient of Determination")
          } else {
             lab <- ifelse(short, lab, "Transformed Coefficient of Determination")
          }
       }
-      if (measure == "ZR2") {
+      if (is.element(measure, c("ZR2","ZR2F"))) {
          if (identical(transf.char, "FALSE") && identical(atransf.char, "FALSE")) {
             lab <- ifelse(short, expression(z[R^2]), "z Transformed Coefficient of Determination")
          } else {
@@ -872,6 +986,18 @@
                lab <- ifelse(short, "Odds", "Odds")
          }
       }
+      if (measure == "PRZ") {
+         if (identical(transf.char, "FALSE") && identical(atransf.char, "FALSE")) {
+            lab <- ifelse(short, expression(Phi^{-1}*(p)), "Probit Transformed Proportion") # expression(z[p])
+         } else {
+            lab <- ifelse(short, lab, "Transformed Probit Transformed Proportion")
+            funlist <- lapply(list(pnorm), deparse)
+            if (any(sapply(funlist, identical, atransf.char)))
+               lab <- ifelse(short, "Proportion", "Proportion (probit scale)")
+            if (any(sapply(funlist, identical, transf.char)))
+               lab <- ifelse(short, "Proportion", "Proportion")
+         }
+      }
       if (measure == "PAS") {
          if (identical(transf.char, "FALSE") && identical(atransf.char, "FALSE")) {
             lab <- ifelse(short, expression(arcsin(sqrt(p))), "Arcsine Transformed Proportion")
@@ -943,6 +1069,13 @@
             lab <- ifelse(short, lab, "Transformed Mean")
          }
       }
+      if (measure == "SMN") {
+         if (identical(transf.char, "FALSE") && identical(atransf.char, "FALSE")) {
+            lab <- ifelse(short, "Std. Mean", "Standardized Mean")
+         } else {
+            lab <- ifelse(short, lab, "Transformed Standardized Mean")
+         }
+      }
       if (measure == "MNLN") {
          if (identical(transf.char, "FALSE") && identical(atransf.char, "FALSE")) {
             lab <- ifelse(short, "Log[Mean]", "Log Mean")
@@ -955,13 +1088,6 @@
                lab <- ifelse(short, "Mean", "Mean")
          }
       }
-      if (measure == "SMN") {
-         if (identical(transf.char, "FALSE") && identical(atransf.char, "FALSE")) {
-            lab <- ifelse(short, "SM", "Standardized Mean")
-         } else {
-            lab <- ifelse(short, lab, "Transformed Standardized Mean")
-         }
-      }
       if (measure == "CVLN") {
          if (identical(transf.char, "FALSE") && identical(atransf.char, "FALSE")) {
             lab <- ifelse(short, "Log[CV]", "Log Coefficient of Variation")
@@ -994,7 +1120,7 @@
             lab <- ifelse(short, lab, "Transformed Mean Change")
          }
       }
-      if (is.element(measure, c("SMCC","SMCR","SMCRH"))) {
+      if (is.element(measure, c("SMCC","SMCR","SMCRH","SMCRP","SMCRPH"))) {
          if (identical(transf.char, "FALSE") && identical(atransf.char, "FALSE")) {
             lab <- ifelse(short, "SMC", "Standardized Mean Change")
          } else {
@@ -1083,6 +1209,42 @@
          }
       }
       ######################################################################
+      if (is.element(measure, c("CLES","CLESN","CLESCN"))) {
+         if (identical(transf.char, "FALSE") && identical(atransf.char, "FALSE")) {
+            lab <- ifelse(short, "CLES", "Common Language Effect Size")
+         } else {
+            lab <- ifelse(short, lab, "Transformed Common Language Effect Size")
+         }
+      }
+      ######################################################################
+      if (is.element(measure, c("AUC","AUCN","AUCCN"))) {
+         if (identical(transf.char, "FALSE") && identical(atransf.char, "FALSE")) {
+            lab <- ifelse(short, "AUC", "Area under the Curve")
+         } else {
+            lab <- ifelse(short, lab, "Transformed Area under the Curve")
+         }
+      }
+      ######################################################################
+      if (measure == "HR") {
+         if (identical(transf.char, "FALSE") && identical(atransf.char, "FALSE")) {
+            lab <- ifelse(short, "Log[HR]", "Log Hazard Ratio")
+         } else {
+            lab <- ifelse(short, lab, "Transformed Log Hazard Ratio")
+            funlist <- lapply(list(exp, transf.exp.int), deparse)
+            if (any(sapply(funlist, identical, atransf.char)))
+               lab <- ifelse(short, "HR", "Hazard Ratio (log scale)")
+            if (any(sapply(funlist, identical, transf.char)))
+               lab <- ifelse(short, "HR", "Hazard Ratio")
+         }
+      }
+      if (measure == "HD") {
+         if (identical(transf.char, "FALSE") && identical(atransf.char, "FALSE")) {
+            lab <- ifelse(short, "HD", "Hazard Difference")
+         } else {
+            lab <- ifelse(short, lab, "Transformed Hazard Difference")
+         }
+      }
+      ######################################################################
 
    }
 
@@ -1094,9 +1256,22 @@
 
 ### stuff related to colored/styled output
 
-.get.mstyle <- function(withcrayon) {
+.get.mstyle <- function() {
 
-   if (withcrayon) {
+   crayonloaded <- "crayon" %in% .packages()
+
+   styleopt <- getmfopt("style")
+
+   if (is.logical(styleopt)) {
+
+      if (isTRUE(styleopt)) {
+         styleopt <- NULL
+      } else {
+         crayonloaded <- FALSE
+      }
+   }
+
+   if (crayonloaded) {
 
       if (exists(".mstyle")) {
          .mstyle <- get(".mstyle")
@@ -1104,8 +1279,6 @@
          .mstyle <- list()
       }
 
-      styleopt <- getmfopt("style")
-
       if (!is.null(styleopt))
          .mstyle <- styleopt
 
@@ -1388,13 +1561,16 @@
    if (is.null(x)) # if x is NULL, return x (i.e., NULL)
       return(x)
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    argname <- deparse(substitute(x))
 
    if (length(x) == 0L)
       stop(mstyle$stop(paste0("Argument '", argname, "' is of length 0.")), call.=FALSE)
 
+   if (is.character(x))
+      stop(mstyle$stop(paste0("Argument '", argname, "' is not a logical or numeric vector.")), call.=FALSE)
+
    if (is.logical(x)) {
       if (length(x) != k)
          stop(mstyle$stop(paste0("Length of the '", argname, "' argument (", length(x), ") is not of length k = ", k, ".")), call.=FALSE)
@@ -1436,7 +1612,7 @@
    if (is.null(x) || is.null(subset)) # if x or subset is NULL, return x
       return(x)
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    xname <- deparse(substitute(x))
 
@@ -1476,15 +1652,21 @@
 
 ############################################################################
 
-.chkopt <- function(optimizer, optcontrol) {
+.chkopt <- function(optimizer, optcontrol, ineq=FALSE) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
-   ### set NLOPT_LN_BOBYQA as the default algorithm for nloptr optimizer
+   ### set NLOPT_LN_BOBYQA as the default algorithm for the nloptr optimizer when ineq=FALSE
+   ### and otherwise use NLOPT_LN_COBYLA to allow for nonlinear inequality constraints
    ### and by default use a relative convergence criterion of 1e-8 on the function value
 
-   if (optimizer == "nloptr" && !is.element("algorithm", names(optcontrol)))
-      optcontrol$algorithm <- "NLOPT_LN_BOBYQA"
+   if (optimizer == "nloptr" && !is.element("algorithm", names(optcontrol))) {
+      if (ineq) {
+         optcontrol$algorithm <- "NLOPT_LN_COBYLA"
+      } else {
+         optcontrol$algorithm <- "NLOPT_LN_BOBYQA"
+      }
+   }
 
    if (optimizer == "nloptr" && !is.element("ftol_rel", names(optcontrol)))
       optcontrol$ftol_rel <- 1e-8
@@ -1544,14 +1726,9 @@
          stop(mstyle$stop("Please install the 'alabama' package to use this optimizer."), call.=FALSE)
    }
 
-   if (optimizer == "Rcgmin") {
-      if (!requireNamespace("Rcgmin", quietly=TRUE))
-         stop(mstyle$stop("Please install the 'Rcgmin' package to use this optimizer."), call.=FALSE)
-   }
-
-   if (optimizer == "Rvmmin") {
-      if (!requireNamespace("Rvmmin", quietly=TRUE))
-         stop(mstyle$stop("Please install the 'Rvmmin' package to use this optimizer."), call.=FALSE)
+   if (is.element(optimizer, c("Rcgmin","Rvmmin"))) {
+      if (!requireNamespace("optimx", quietly=TRUE))
+         stop(mstyle$stop(paste0("Please install the 'optimx' package to use this optimizer.")), call.=FALSE)
    }
 
    #########################################################################
@@ -1626,16 +1803,16 @@
 
    if (optimizer == "Rcgmin") {
       par.arg <- "par"
-      optimizer <- "Rcgmin::Rcgmin"
-      #ctrl.arg <- ", gr='grnd', control=optcontrol"
-      ctrl.arg <- ", control=optcontrol"
+      optimizer <- "optimx::Rcgmin"
+      ctrl.arg <- ", gr='grnd', control=optcontrol"
+      #ctrl.arg <- ", control=optcontrol"
    }
 
    if (optimizer == "Rvmmin") {
       par.arg <- "par"
-      optimizer <- "Rvmmin::Rvmmin"
-      #ctrl.arg <- ", gr='grnd', control=optcontrol"
-      ctrl.arg <- ", control=optcontrol"
+      optimizer <- "optimx::Rvmmin"
+      ctrl.arg <- ", gr='grnd', control=optcontrol"
+      #ctrl.arg <- ", control=optcontrol"
    }
 
    if (optimizer == "optimParallel") {
@@ -1650,7 +1827,7 @@
 
 .chkconv <- function(optimizer, opt.res, optcontrol, fun, verbose) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    if (optimizer == "optimParallel::optimParallel" && verbose) {
       tmp <- capture.output(print(opt.res$loginfo))
@@ -1665,7 +1842,7 @@
    if (optimizer == "lbfgsb3c::lbfgsb3c" && is.null(opt.res$convergence)) # special provision for lbfgsb3c in case 'convergence' is missing
       opt.res$convergence <- -99
 
-   if (is.element(optimizer, c("optim","constrOptim","nlminb","dfoptim::hjk","dfoptim::nmk","lbfgsb3c::lbfgsb3c","subplex::subplex","BB::BBoptim","Rsolnp::solnp","alabama::constrOptim.nl","Rcgmin::Rcgmin","Rvmmin:Rvmmin","optimParallel::optimParallel")) && opt.res$convergence != 0)
+   if (is.element(optimizer, c("optim","constrOptim","nlminb","dfoptim::hjk","dfoptim::nmk","lbfgsb3c::lbfgsb3c","subplex::subplex","BB::BBoptim","Rsolnp::solnp","alabama::constrOptim.nl","optimx::Rcgmin","optimx:Rvmmin","optimParallel::optimParallel")) && opt.res$convergence != 0)
       stop(mstyle$stop(paste0("Optimizer (", optimizer, ") did not achieve convergence (convergence = ", opt.res$convergence, ").")), call.=FALSE)
 
    if (is.element(optimizer, c("dfoptim::mads")) && opt.res$convergence > optcontrol$tol)
@@ -1804,7 +1981,7 @@
 
    }
 
-   if (themeopt != "default")
+   if (themeopt != "default" && isFALSE(par("new")))
       par(fg=fg, bg=bg, col=fg, col.axis=fg, col.lab=fg, col.main=fg, col.sub=fg)
 
    invisible()
@@ -1854,6 +2031,7 @@
    } else {
       col <- col + round(light*255)[[pos]]
    }
+
    col[col < 0] <- 0
    col[col > 255] <- 255
 
