@@ -1,6 +1,6 @@
 model.matrix.rma <- function(object, asdf=FALSE, ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(object), must="rma")
 
