@@ -3,7 +3,7 @@ digits, cols, grid=TRUE, pch=19, cex=1, lwd=2, ...) {
 
    #########################################################################
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(x), must="cumul.rma")
 
@@ -58,8 +58,8 @@ digits, cols, grid=TRUE, pch=19, cex=1, lwd=2, ...) {
 
    par.mar <- par("mar")
    par.mar.adj <- par.mar + c(0,0.5,0,0) # need a bit more space on the right for the y-axis label
-   par(mar = par.mar.adj)
-   on.exit(par(mar = par.mar), add=TRUE)
+   par(mar=par.mar.adj)
+   on.exit(par(mar=par.mar), add=TRUE)
 
    if (missing(at))
       at <- NULL
@@ -96,8 +96,8 @@ digits, cols, grid=TRUE, pch=19, cex=1, lwd=2, ...) {
       grid <- TRUE
    }
 
-   lplot <- function(..., addgrid) plot(...)
-   laxis <- function(..., addgrid) axis(...)
+   lplot <- function(..., addgrid, at.lab) plot(...)
+   laxis <- function(..., addgrid, at.lab) axis(...)
 
    #########################################################################
 
@@ -130,6 +130,8 @@ digits, cols, grid=TRUE, pch=19, cex=1, lwd=2, ...) {
       if (is.null(targs)) {
          dat$estim <- sapply(dat$estim, transf)
       } else {
+         if (!is.primitive(transf) && !is.null(targs) && length(formals(transf)) == 1L)
+            stop(mstyle$stop("Function specified via 'transf' does not appear to have an argument for 'targs'."))
          dat$estim <- sapply(dat$estim, transf, targs)
       }
    }
@@ -168,16 +170,24 @@ digits, cols, grid=TRUE, pch=19, cex=1, lwd=2, ...) {
       at <- at[at < par("usr")[2]]
    }
 
-   at.lab <- at
+   if (is.null(ddd$at.lab)) {
 
-   if (is.function(atransf)) {
-      if (is.null(targs)) {
-         at.lab <- fmtx(sapply(at.lab, atransf), digits[[1]], drop0ifint=TRUE)
+      at.lab <- at
+
+      if (is.function(atransf)) {
+         if (is.null(targs)) {
+            at.lab <- fmtx(sapply(at.lab, atransf), digits[[1]], drop0ifint=TRUE)
+         } else {
+            at.lab <- fmtx(sapply(at.lab, atransf, targs), digits[[1]], drop0ifint=TRUE)
+         }
       } else {
-         at.lab <- fmtx(sapply(at.lab, atransf, targs), digits[[1]], drop0ifint=TRUE)
+         at.lab <- fmtx(at.lab, digits[[1]], drop0ifint=TRUE)
       }
+
    } else {
-      at.lab <- fmtx(at.lab, digits[[1]], drop0ifint=TRUE)
+
+      at.lab <- ddd$at.lab
+
    }
 
    ### add x-axis
