@@ -2,7 +2,7 @@ plot.rma.glmm <- function(x, qqplot=FALSE, ...) {
 
    #########################################################################
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(x), must="rma.glmm", notav="rma.glmm")
 
