@@ -1,6 +1,6 @@
 print.gosh.rma <- function(x, digits=x$digits, ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(x), must="gosh.rma")
 
@@ -19,9 +19,9 @@ print.gosh.rma <- function(x, digits=x$digits, ...) {
 
    res.table[,1] <- apply(x$res, 2, mean, na.rm=TRUE)
    res.table[,2] <- apply(x$res, 2, min, na.rm=TRUE)
-   res.table[,3] <- apply(x$res, 2, quantile, .25, na.rm=TRUE)
-   res.table[,4] <- apply(x$res, 2, quantile, .50, na.rm=TRUE)
-   res.table[,5] <- apply(x$res, 2, quantile, .75, na.rm=TRUE)
+   res.table[,3] <- apply(x$res, 2, quantile, 0.25, na.rm=TRUE)
+   res.table[,4] <- apply(x$res, 2, quantile, 0.50, na.rm=TRUE)
+   res.table[,5] <- apply(x$res, 2, quantile, 0.75, na.rm=TRUE)
    res.table[,6] <- apply(x$res, 2, max, na.rm=TRUE)
 
    res.table <- fmtx(res.table, digits[["est"]])
