@@ -1,6 +1,6 @@
 print.hc.rma.uni <- function(x, digits=x$digits, ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(x), must="hc.rma.uni")
 
