@@ -1,6 +1,6 @@
 print.infl.rma.uni <- function(x, digits=x$digits, infonly=FALSE, ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(x), must="infl.rma.uni")
 
