@@ -1,6 +1,6 @@
 print.list.rma <- function(x, digits=x$digits, ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(x), must="list.rma")
 
@@ -56,7 +56,7 @@ print.list.rma <- function(x, digits=x$digits, ...) {
    sav <- out[,seq_len(min.pos-1)]
 
    for (i in seq_len(min.pos-1)) {
-      if (inherits(out[,i], c("integer","logical","factor","character"))) { ### do not apply formating to these classes
+      if (inherits(out[,i], c("integer","logical","factor","character"))) { # do not apply formating to these classes
          out[,i] <- out[,i]
       } else {
          if (names(out)[i] %in% c("pred", "resid"))
@@ -65,7 +65,7 @@ print.list.rma <- function(x, digits=x$digits, ...) {
             out[,i] <- fmtx(out[,i], digits[["se"]])
          if (names(out)[i] %in% c("ci.lb", "ci.ub", "cr.lb", "cr.ub", "pi.lb", "pi.ub"))
             out[,i] <- fmtx(out[,i], digits[["ci"]])
-         if (names(out)[i] %in% c("zval", "Q", "z", "X2"))
+         if (names(out)[i] %in% c("zval", "tval", "Q", "z", "X2"))
             out[,i] <- fmtx(out[,i], digits[["test"]])
          if (names(out)[i] %in% c("pval", "Qp"))
             out[,i] <- fmtx(out[,i], digits[["pval"]])
@@ -73,6 +73,8 @@ print.list.rma <- function(x, digits=x$digits, ...) {
             out[,i] <- fmtx(out[,i], digits[["het"]])
          if (names(out)[i] %in% c("tau2"))
             out[,i] <- fmtx(out[,i], digits[["var"]])
+         if (names(out)[i] %in% c("k"))
+            out[,i] <- fmtx(out[,i], 0)
          # if (names(out)[i] == "rstudent")
          #    out[,i] <- fmtx(out[,i], digits[["test"]])
          # if (names(out)[i] == "dffits")
