@@ -2,7 +2,7 @@ print.profile.rma <- function(x, ...) {
 
    #########################################################################
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(x), must="profile.rma")
 
