@@ -1,6 +1,6 @@
 print.rma.mv <- function(x, digits, showfit=FALSE, signif.stars=getOption("show.signif.stars"), signif.legend=signif.stars, ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(x), must="rma.mv")
 
@@ -19,7 +19,12 @@ print.rma.mv <- function(x, digits, showfit=FALSE, signif.stars=getOption("show.
    if (is.null(ddd$legend)) {
       legend <- ifelse(inherits(x, "robust.rma"), TRUE, FALSE)
    } else {
-      legend <- .isTRUE(ddd$legend)
+      if (is.na(ddd$legend)) { # can suppress legend and legend symbols with legend=NA
+         legend <- FALSE
+         footsym <- rep("", 6)
+      } else {
+         legend <- .isTRUE(ddd$legend)
+      }
    }
 
    .space()
@@ -31,9 +36,9 @@ print.rma.mv <- function(x, digits, showfit=FALSE, signif.stars=getOption("show.
    if (showfit) {
       cat("\n")
       if (x$method == "REML") {
-         fs <- .fcf(x$fit.stats$REML, digits[["fit"]])
+         fs <- fmtx(x$fit.stats$REML, digits[["fit"]])
       } else {
-         fs <- .fcf(x$fit.stats$ML, digits[["fit"]])
+         fs <- fmtx(x$fit.stats$ML, digits[["fit"]])
       }
       names(fs) <- c("logLik", "Deviance", "AIC", "BIC", "AICc")
       cat("\n")
@@ -45,14 +50,14 @@ print.rma.mv <- function(x, digits, showfit=FALSE, signif.stars=getOption("show.
       cat("\n\n")
    }
 
-   sigma2 <- .fcf(x$sigma2, digits[["var"]])
-   tau2   <- .fcf(x$tau2,   digits[["var"]])
-   rho    <- .fcf(x$rho,    digits[["var"]])
-   gamma2 <- .fcf(x$gamma2, digits[["var"]])
-   phi    <- .fcf(x$phi,    digits[["var"]])
-   sigma  <- .fcf(sqrt(x$sigma2), digits[["var"]])
-   tau    <- .fcf(sqrt(x$tau2),   digits[["var"]])
-   gamma  <- .fcf(sqrt(x$gamma2), digits[["var"]])
+   sigma2 <- fmtx(x$sigma2, digits[["var"]])
+   tau2   <- fmtx(x$tau2,   digits[["var"]])
+   rho    <- fmtx(x$rho,    digits[["var"]])
+   gamma2 <- fmtx(x$gamma2, digits[["var"]])
+   phi    <- fmtx(x$phi,    digits[["var"]])
+   sigma  <- fmtx(sqrt(x$sigma2), digits[["var"]])
+   tau    <- fmtx(sqrt(x$tau2),   digits[["var"]])
+   gamma  <- fmtx(sqrt(x$gamma2), digits[["var"]])
 
    cat(mstyle$section("Variance Components:"))
 
@@ -73,7 +78,7 @@ print.rma.mv <- function(x, digits, showfit=FALSE, signif.stars=getOption("show.
          if (length(x$sigma2) == 1L) {
             rownames(vc) <- "sigma^2  "
          } else {
-            rownames(vc) <- paste("sigma^2.", seq_along(x$sigma2), sep="")
+            rownames(vc) <- paste0("sigma^2.", seq_along(x$sigma2))
          }
          tmp <- capture.output(print(vc, quote=FALSE, right=right, print.gap=2))
          .print.table(tmp, mstyle)
@@ -128,7 +133,7 @@ print.rma.mv <- function(x, digits, showfit=FALSE, signif.stars=getOption("show.
             if (length(x$tau2) == 1L) {
                rownames(vc) <- c("tau^2   ", "rho")
             } else {
-               rownames(vc) <- c(paste("tau^2.", seq_along(x$tau2), "  ", sep=""), "rho")
+               rownames(vc) <- c(paste0("tau^2.", seq_along(x$tau2), "  "), "rho")
             }
             if (x$struct[1] == "DIAG")
                vc <- vc[seq_along(tau2),,drop=FALSE]
@@ -148,7 +153,7 @@ print.rma.mv <- function(x, digits, showfit=FALSE, signif.stars=getOption("show.
             if (length(x$g.levels.k) == 1L) {
                rownames(vc) <- c("tau^2")
             } else {
-               rownames(vc) <- paste("tau^2.", seq_along(x$g.levels.k), "  ", sep="")
+               rownames(vc) <- paste0("tau^2.", seq_along(x$g.levels.k), "  ")
             }
             tmp <- capture.output(print(vc, quote=FALSE, right=right, print.gap=2))
             .print.table(tmp, mstyle)
@@ -175,7 +180,7 @@ print.rma.mv <- function(x, digits, showfit=FALSE, signif.stars=getOption("show.
             diag(G.info) <- "-"
 
             vc <- cbind(G, "", G.info)
-            colnames(vc) <- c(paste("rho.", abbreviate(x$g.levels.f[[1]]), sep=""), "", abbreviate(x$g.levels.f[[1]])) ### FIXME: x$g.levels.f[[1]] may be numeric, in which case a wrapping 'header' is not recognized
+            colnames(vc) <- c(paste0("rho.", abbreviate(x$g.levels.f[[1]])), "", abbreviate(x$g.levels.f[[1]])) # FIXME: x$g.levels.f[[1]] may be numeric, in which case a wrapping 'header' is not recognized
             rownames(vc) <- x$g.levels.f[[1]]
             tmp <- capture.output(print(vc, quote=FALSE, right=right, print.gap=2))
             .print.table(tmp, mstyle)
@@ -188,7 +193,7 @@ print.rma.mv <- function(x, digits, showfit=FALSE, signif.stars=getOption("show.
             colnames(vc) <- c("estim", "sqrt", "fixed", "rho:")
             rownames(vc) <- x$g.names[-length(x$g.names)]
 
-            G.info <- .fcf(cov2cor(x$G), digits[["var"]])
+            G.info <- fmtx(cov2cor(x$G), digits[["var"]])
             diag(G.info) <- "-"
             G.info[lower.tri(G.info)] <- ifelse(x$vc.fix$rho, "yes", "no")
             colnames(G.info) <- abbreviate(x$g.names[-length(x$g.names)])
@@ -259,7 +264,7 @@ print.rma.mv <- function(x, digits, showfit=FALSE, signif.stars=getOption("show.
             if (length(x$gamma2) == 1L) {
                rownames(vc) <- c("gamma^2 ", "phi")
             } else {
-               rownames(vc) <- c(paste("gamma^2.", seq_along(x$gamma2), "  ", sep=""), "phi")
+               rownames(vc) <- c(paste0("gamma^2.", seq_along(x$gamma2), "  "), "phi")
             }
             if (x$struct[2] == "DIAG")
                vc <- vc[seq_along(gamma2),,drop=FALSE]
@@ -279,7 +284,7 @@ print.rma.mv <- function(x, digits, showfit=FALSE, signif.stars=getOption("show.
             if (length(x$h.levels.k) == 1L) {
                rownames(vc) <- c("gamma^2")
             } else {
-               rownames(vc) <- paste("gamma^2.", seq_along(x$h.levels.k), "  ", sep="")
+               rownames(vc) <- paste0("gamma^2.", seq_along(x$h.levels.k), "  ")
             }
             tmp <- capture.output(print(vc, quote=FALSE, right=right, print.gap=2))
             .print.table(tmp, mstyle)
@@ -306,7 +311,7 @@ print.rma.mv <- function(x, digits, showfit=FALSE, signif.stars=getOption("show.
             diag(H.info) <- "-"
 
             vc <- cbind(H, "", H.info)
-            colnames(vc) <- c(paste("phi.", abbreviate(x$h.levels.f[[1]]), sep=""), "", abbreviate(x$h.levels.f[[1]])) ### FIXME: x$h.levels.f[[1]] may be numeric, in which case a wrapping 'header' is not recognized
+            colnames(vc) <- c(paste0("phi.", abbreviate(x$h.levels.f[[1]])), "", abbreviate(x$h.levels.f[[1]])) # FIXME: x$h.levels.f[[1]] may be numeric, in which case a wrapping 'header' is not recognized
             rownames(vc) <- x$h.levels.f[[1]]
             tmp <- capture.output(print(vc, quote=FALSE, right=right, print.gap=2))
             .print.table(tmp, mstyle)
@@ -319,7 +324,7 @@ print.rma.mv <- function(x, digits, showfit=FALSE, signif.stars=getOption("show.
             colnames(vc) <- c("estim", "sqrt", "fixed", "phi:")
             rownames(vc) <- x$h.names[-length(x$h.names)]
 
-            H.info <- .fcf(cov2cor(x$H), digits[["var"]])
+            H.info <- fmtx(cov2cor(x$H), digits[["var"]])
             diag(H.info) <- "-"
             H.info[lower.tri(H.info)] <- ifelse(x$vc.fix$phi, "yes", "no")
             colnames(H.info) <- abbreviate(x$h.names[-length(x$h.names)])
@@ -349,11 +354,11 @@ print.rma.mv <- function(x, digits, showfit=FALSE, signif.stars=getOption("show.
       if (x$int.only) {
          cat(mstyle$section("Test for Heterogeneity:"))
          cat("\n")
-         cat(mstyle$result(paste0("Q(df = ", x$k-x$p, ") = ", .fcf(x$QE, digits[["test"]]), ", p-val ", .pval(x$QEp, digits[["pval"]], showeq=TRUE, sep=" "))))
+         cat(mstyle$result(fmtt(x$QE, "Q", df=x$QEdf, pval=x$QEp, digits=digits)))
       } else {
          cat(mstyle$section("Test for Residual Heterogeneity:"))
          cat("\n")
-         cat(mstyle$result(paste0("QE(df = ", x$k-x$p, ") = ", .fcf(x$QE, digits[["test"]]), ", p-val ", .pval(x$QEp, digits[["pval"]], showeq=TRUE, sep=" "))))
+         cat(mstyle$result(fmtt(x$QE, "QE", df=x$QEdf, pval=x$QEp, digits=digits)))
       }
       cat("\n\n")
    }
@@ -371,7 +376,7 @@ print.rma.mv <- function(x, digits, showfit=FALSE, signif.stars=getOption("show.
       if (all(x$tcl[1] == x$tcl)) {
          cat(mstyle$result(x$tcl[1]))
       } else {
-         cat(mstyle$result(paste0(min(x$tcl), "-", max(x$tcl), " (mean: ", .fcf(mean(x$tcl), digits=2), ", median: ", round(median(x$tcl), digits=2), ")")))
+         cat(mstyle$result(paste0(min(x$tcl), "-", max(x$tcl), " (mean: ", fmtx(mean(x$tcl), digits=2), ", median: ", round(median(x$tcl), digits=2), ")")))
       }
       cat("\n\n")
 
@@ -381,19 +386,19 @@ print.rma.mv <- function(x, digits, showfit=FALSE, signif.stars=getOption("show.
       cat(mstyle$section(paste0("Test of Moderators (coefficient", ifelse(x$m == 1, " ", "s "), .format.btt(x$btt),"):", ifelse(inherits(x, "robust.rma"), footsym[1], ""))))
       cat("\n")
       if (is.element(x$test, c("knha","adhoc","t"))) {
-         cat(mstyle$result(paste0("F(df1 = ", x$QMdf[1], ", df2 = ", round(x$QMdf[2], 2), ") = ", .fcf(x$QM, digits[["test"]]), ", p-val ", .pval(x$QMp, digits[["pval"]], showeq=TRUE, sep=" "))))
+         cat(mstyle$result(fmtt(x$QM, "F", df1=x$QMdf[1], df2=x$QMdf[2], pval=x$QMp, digits=digits)))
       } else {
-         cat(mstyle$result(paste0("QM(df = ", x$QMdf[1], ") = ", .fcf(x$QM, digits[["test"]]), ", p-val ", .pval(x$QMp, digits[["pval"]], showeq=TRUE, sep=" "))))
+         cat(mstyle$result(fmtt(x$QM, "QM", df=x$QMdf[1], pval=x$QMp, digits=digits)))
       }
       cat("\n\n")
    }
 
    if (is.element(x$test, c("knha","adhoc","t"))) {
-      res.table <- data.frame(estimate=.fcf(c(x$beta), digits[["est"]]), se=.fcf(x$se, digits[["se"]]), tval=.fcf(x$zval, digits[["test"]]), df=round(x$ddf,2), pval=.pval(x$pval, digits[["pval"]]), ci.lb=.fcf(x$ci.lb, digits[["ci"]]), ci.ub=.fcf(x$ci.ub, digits[["ci"]]), stringsAsFactors=FALSE)
-      if (inherits(x, "robust.rma"))
+      res.table <- data.frame(estimate=fmtx(c(x$beta), digits[["est"]]), se=fmtx(x$se, digits[["se"]]), tval=fmtx(x$zval, digits[["test"]]), df=round(x$ddf,2), pval=fmtp(x$pval, digits[["pval"]]), ci.lb=fmtx(x$ci.lb, digits[["ci"]]), ci.ub=fmtx(x$ci.ub, digits[["ci"]]), stringsAsFactors=FALSE)
+      if (inherits(x, "robust.rma") && footsym[1] != "")
          res.table <- .addfootsym(res.table, 2:7, footsym[1])
    } else {
-      res.table <- data.frame(estimate=.fcf(c(x$beta), digits[["est"]]), se=.fcf(x$se, digits[["se"]]), zval=.fcf(x$zval, digits[["test"]]), pval=.pval(x$pval, digits[["pval"]]), ci.lb=.fcf(x$ci.lb, digits[["ci"]]), ci.ub=.fcf(x$ci.ub, digits[["ci"]]), stringsAsFactors=FALSE)
+      res.table <- data.frame(estimate=fmtx(c(x$beta), digits[["est"]]), se=fmtx(x$se, digits[["se"]]), zval=fmtx(x$zval, digits[["test"]]), pval=fmtp(x$pval, digits[["pval"]]), ci.lb=fmtx(x$ci.lb, digits[["ci"]]), ci.ub=fmtx(x$ci.ub, digits[["ci"]]), stringsAsFactors=FALSE)
    }
    rownames(res.table) <- rownames(x$beta)
    signif <- symnum(x$pval, corr=FALSE, na=FALSE, cutpoints=c(0, 0.001, 0.01, 0.05, 0.1, 1), symbols = c("***", "**", "*", ".", " "))
@@ -402,8 +407,10 @@ print.rma.mv <- function(x, digits, showfit=FALSE, signif.stars=getOption("show.
       colnames(res.table)[ncol(res.table)] <- ""
    }
 
-   if (.isTRUE(ddd$num))
-      rownames(res.table) <- paste0(seq_len(nrow(res.table)), ") ", rownames(res.table))
+   if (.isTRUE(ddd$num)) {
+      width <- nchar(nrow(res.table))
+      rownames(res.table) <- paste0(formatC(seq_len(nrow(res.table)), format="d", width=width), ") ", rownames(res.table))
+   }
 
    if (x$int.only)
       res.table <- res.table[1,]
