@@ -1,18 +1,31 @@
 profile.rma.uni.selmodel <- function(fitted, tau2, delta,
-   xlim, ylim, steps=20, lltol=1e-03, progbar=TRUE, parallel="no", ncpus=1, cl, plot=TRUE, pch=19, refline=TRUE, cline=FALSE, ...) {
+   xlim, ylim, steps=20, lltol=1e-03, progbar=TRUE, parallel="no", ncpus=1, cl, plot=TRUE, ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(fitted), must="rma.uni.selmodel")
 
-   if (steps < 2)
-      stop(mstyle$stop("Argument 'steps' must be >= 2."))
-
    x <- fitted
 
-   if (x$betaspec) ### TODO: consider allowing profiling over beta values as well
+   if (x$betaspec) # TODO: consider allowing profiling over beta values as well
       stop(mstyle$stop("Cannot profile when one or more beta values were fixed."))
 
+   if (x$decreasing || x$type == "stepcon")
+      stop(mstyle$stop("Method not currently implemented for this type of model."))
+
+   if (anyNA(steps))
+      stop(mstyle$stop("No missing values allowed in 'steps' argument."))
+
+   if (length(steps) >= 2L) {
+      if (missing(xlim))
+         xlim <- range(steps)
+      stepseq <- TRUE
+   } else {
+      if (steps < 2)
+         stop(mstyle$stop("Argument 'steps' must be >= 2."))
+      stepseq <- FALSE
+   }
+
    parallel <- match.arg(parallel, c("no", "snow", "multicore"))
 
    if (parallel == "no" && ncpus > 1)
@@ -66,10 +79,15 @@ profile.rma.uni.selmodel <- function(fitted, tau2, delta,
       if (comps == 0)
          stop(mstyle$stop("No components in the model for which a profile likelihood can be constructed."))
 
+      if (!is.null(ddd[["code3"]]))
+         eval(expr = parse(text = ddd[["code3"]]))
+
       if (plot) {
-         if (dev.cur() == 1) {
-            par(mfrow=c(comps, 1))
-            #on.exit(par(mfrow=c(1,1)), add=TRUE)
+         if (comps > 1L) {
+            # if no plotting device is open or mfrow is too small, set mfrow appropriately
+            if (dev.cur() == 1L || prod(par("mfrow")) < comps)
+               par(mfrow=n2mfrow(comps))
+            on.exit(par(mfrow=c(1L,1L)), add=TRUE)
          }
       }
 
@@ -78,10 +96,13 @@ profile.rma.uni.selmodel <- function(fitted, tau2, delta,
 
       if (!is.element(x$method, c("FE","EE","CE")) && !x$tau2.fix) {
          j <- j + 1
+         if (!is.null(ddd[["code4"]]))
+            eval(expr = parse(text = ddd[["code4"]]))
          mc.vc <- mc
          mc.vc$tau2 <- 1
          mc.vc$time <- FALSE
          #mc.vc$fitted <- quote(x)
+         mc.vc[[1]] <- str2lang("metafor::profile.rma.uni.selmodel")
          if (progbar)
             cat(mstyle$verbose(paste("Profiling tau2\n")))
          sav[[j]] <- eval(mc.vc, envir=parent.frame())
@@ -90,10 +111,13 @@ profile.rma.uni.selmodel <- function(fitted, tau2, delta,
       if (any(!x$delta.fix)) {
          for (pos in seq_len(x$deltas)[!x$delta.fix]) {
             j <- j + 1
+            if (!is.null(ddd[["code4"]]))
+               eval(expr = parse(text = ddd[["code4"]]))
             mc.vc <- mc
             mc.vc$delta <- pos
             mc.vc$time <- FALSE
             #mc.vc$fitted <- quote(x)
+            mc.vc[[1]] <- str2lang("metafor::profile.rma.uni.selmodel")
             if (progbar)
                cat(mstyle$verbose(paste("Profiling delta =", pos, "\n")))
             sav[[j]] <- eval(mc.vc, envir=parent.frame())
@@ -117,6 +141,16 @@ profile.rma.uni.selmodel <- function(fitted, tau2, delta,
 
    }
 
+   #########################################################################
+
+   ### round and take unique values
+
+   if (!missing(delta) && is.numeric(delta))
+      delta <- unique(round(delta))
+
+   if (!missing(tau2) && is.numeric(tau2))
+      tau2 <- unique(round(tau2))
+
    ### check if user has specified more than one of these arguments
 
    if (sum(!missing(tau2), !missing(delta)) > 1L)
@@ -164,7 +198,7 @@ profile.rma.uni.selmodel <- function(fitted, tau2, delta,
 
    ### if everything is good so far, get value of the variance component and set 'comp'
 
-   delta.pos <- NA
+   delta.pos <- NA_integer_
 
    if (!missing(tau2)) {
       vc <- x$tau2
@@ -180,26 +214,28 @@ profile.rma.uni.selmodel <- function(fitted, tau2, delta,
 
    #return(list(comp=comp, vc=vc))
 
-   if (missing(xlim)) {
+   #########################################################################
+
+   if (missing(xlim) || is.null(xlim)) {
 
       ### if the user has not specified xlim, set it automatically
 
       if (comp == "tau2") {
          if (is.na(x$se.tau2)) {
             vc.lb <- max(0, vc/4)
-            vc.ub <- min(max(.1, vc*4), x$tau2.max)
+            vc.ub <- min(max(0.1, vc*4), x$tau2.max)
          } else {
-            vc.lb <- max(0, vc - qnorm(.995) * x$se.tau2)
-            vc.ub <- min(max(.1, vc + qnorm(.995) * x$se.tau2), x$tau2.max)
+            vc.lb <- max(0, vc - qnorm(0.995) * x$se.tau2)
+            vc.ub <- min(max(0.1, vc + qnorm(0.995) * x$se.tau2), x$tau2.max)
          }
       }
       if (comp == "delta") {
          if (is.na(x$se.delta[delta])) {
             vc.lb <- max(0, vc/4, x$delta.min[delta])
-            vc.ub <- min(max(.1, vc*4), x$delta.max[delta])
+            vc.ub <- min(max(0.1, vc*4), x$delta.max[delta])
          } else {
-            vc.lb <- max(0, vc - qnorm(.995) * x$se.delta[delta], x$delta.min[delta])
-            vc.ub <- min(max(.1, vc + qnorm(.995) * x$se.delta[delta]), x$delta.max[delta])
+            vc.lb <- max(0, vc - qnorm(0.995) * x$se.delta[delta], x$delta.min[delta])
+            vc.ub <- min(max(0.1, vc + qnorm(0.995) * x$se.delta[delta]), x$delta.max[delta])
          }
       }
 
@@ -230,18 +266,26 @@ profile.rma.uni.selmodel <- function(fitted, tau2, delta,
 
    }
 
-   vcs <- seq(xlim[1], xlim[2], length.out=steps)
+   if (stepseq) {
+      vcs <- steps
+   } else {
+      vcs <- seq(xlim[1], xlim[2], length.out=steps)
+   }
+
    #return(vcs)
 
    if (length(vcs) <= 1L)
       stop(mstyle$stop("Cannot set 'xlim' automatically. Please set this argument manually."))
 
+   if (!is.null(ddd[["code1"]]))
+      eval(expr = parse(text = ddd[["code1"]]))
+
    if (parallel == "no")
-      res <- pbapply::pblapply(vcs, .profile.rma.uni.selmodel, obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE)
+      res <- pbapply::pblapply(vcs, .profile.rma.uni.selmodel, obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE, code2=ddd$code2)
 
    if (parallel == "multicore")
-      res <- pbapply::pblapply(vcs, .profile.rma.uni.selmodel, obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE, cl=ncpus)
-      #res <- parallel::mclapply(vcs, .profile.rma.uni.selmodel, obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE, mc.cores=ncpus)
+      res <- pbapply::pblapply(vcs, .profile.rma.uni.selmodel, obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE, code2=ddd$code2, cl=ncpus)
+      #res <- parallel::mclapply(vcs, .profile.rma.uni.selmodel, obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE, code2=ddd$code2, mc.cores=ncpus)
 
    if (parallel == "snow") {
       if (is.null(cl)) {
@@ -249,28 +293,22 @@ profile.rma.uni.selmodel <- function(fitted, tau2, delta,
          on.exit(parallel::stopCluster(cl), add=TRUE)
       }
       if (.isTRUE(ddd$LB)) {
-         res <- parallel::parLapplyLB(cl, vcs, .profile.rma.uni.selmodel, obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE)
-         #res <- parallel::clusterApplyLB(cl, vcs, .profile.rma.uni.selmodel, obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE)
+         res <- parallel::parLapplyLB(cl, vcs, .profile.rma.uni.selmodel, obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE, code2=ddd$code2)
+         #res <- parallel::clusterApplyLB(cl, vcs, .profile.rma.uni.selmodel, obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE, code2=ddd$code2)
+         #res <- parallel::clusterMap(cl, .profile.rma.uni.selmodel, vcs, MoreArgs=list(obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE, code2=ddd$code2), .scheduling = "dynamic")
       } else {
-         res <- pbapply::pblapply(vcs, .profile.rma.uni.selmodel, obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE, cl=cl)
-         #res <- parallel::parLapply(cl, vcs, .profile.rma.uni.selmodel, obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE)
-         #res <- parallel::clusterApply(cl, vcs, .profile.rma.uni.selmodel, obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE)
+         res <- pbapply::pblapply(vcs, .profile.rma.uni.selmodel, obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE, code2=ddd$code2, cl=cl)
+         #res <- parallel::parLapply(cl, vcs, .profile.rma.uni.selmodel, obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE, code2=ddd$code2)
+         #res <- parallel::clusterApply(cl, vcs, .profile.rma.uni.selmodel, obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE, code2=ddd$code2)
+         #res <- parallel::clusterMap(cl, .profile.rma.uni.selmodel, vcs, MoreArgs=list(obj=x, comp=comp, delta.pos=delta.pos, parallel=parallel, profile=TRUE, code2=ddd$code2))
       }
    }
 
-   lls <- sapply(res, function(x) x$ll)
+   lls   <- sapply(res, function(x) x$ll)
    beta  <- do.call(rbind, lapply(res, function(x) t(x$beta)))
    ci.lb <- do.call(rbind, lapply(res, function(x) t(x$ci.lb)))
    ci.ub <- do.call(rbind, lapply(res, function(x) t(x$ci.ub)))
 
-   #########################################################################
-
-   if (any(lls >= logLik(x) + lltol, na.rm=TRUE))
-      warning(mstyle$warning("At least one profiled log-likelihood value is larger than the log-likelihood of the fitted model."), call.=FALSE)
-
-   if (all(is.na(lls)))
-      warning(mstyle$warning("All model fits failed. Cannot draw profile likelihood plot."), call.=FALSE)
-
    beta  <- data.frame(beta)
    ci.lb <- data.frame(ci.lb)
    ci.ub <- data.frame(ci.ub)
@@ -278,19 +316,35 @@ profile.rma.uni.selmodel <- function(fitted, tau2, delta,
    names(ci.lb) <- rownames(x$beta)
    names(ci.ub) <- rownames(x$beta)
 
+   #########################################################################
+
+   maxll <- c(logLik(x))
+
+   if (any(lls >= maxll + lltol, na.rm=TRUE))
+      warning(mstyle$warning("At least one profiled log-likelihood value is larger than the log-likelihood of the fitted model."), call.=FALSE)
+
+   if (all(is.na(lls)))
+      warning(mstyle$warning("All model fits failed. Cannot draw profile likelihood plot."), call.=FALSE)
+
+   if (.isTRUE(ddd$exp)) {
+      lls <- exp(lls)
+      maxll <- exp(maxll)
+   }
+
    if (missing(ylim)) {
 
-      if (any(!is.na(lls))) {
+      if (any(is.finite(lls))) {
          if (xlim[1] <= vc && xlim[2] >= vc) {
-            ylim <- range(c(logLik(x),lls), na.rm=TRUE)
+            ylim <- range(c(maxll,lls[is.finite(lls)]), na.rm=TRUE)
          } else {
-            ylim <- range(lls, na.rm=TRUE)
+            ylim <- range(lls[is.finite(lls)], na.rm=TRUE)
          }
       } else {
-         ylim <- rep(logLik(x), 2L)
+         ylim <- rep(maxll, 2L)
       }
-      ylim[1] <- ylim[1] - .1
-      ylim[2] <- ylim[2] + .1
+
+      if (!.isTRUE(ddd$exp))
+         ylim <- ylim + c(-0.1, 0.1)
 
    } else {
 
@@ -302,27 +356,27 @@ profile.rma.uni.selmodel <- function(fitted, tau2, delta,
    }
 
    if (comp == "tau2") {
-      xlab <- expression(paste(tau^2, " Value"))
+      xlab  <- expression(paste(tau^2, " Value"))
       title <- expression(paste("Profile Plot for ", tau^2))
    }
    if (comp == "delta") {
       if (x$deltas == 1L) {
-         xlab <- expression(paste(delta, " Value"))
+         xlab  <- expression(paste(delta, " Value"))
          title <- expression(paste("Profile Plot for ", delta))
       } else {
-         xlab <- bquote(delta[.(delta)] ~ "Value")
+         xlab  <- bquote(delta[.(delta)] ~ "Value")
          title <- bquote("Profile Plot for" ~ delta[.(delta)])
       }
    }
 
-   sav <- list(vc=vcs, ll=lls, beta=beta, ci.lb=ci.lb, ci.ub=ci.ub, comps=1, ylim=ylim, method=x$method, vc=vc, maxll=logLik(x), xlab=xlab, title=title)
+   sav <- list(vc=vcs, ll=lls, beta=beta, ci.lb=ci.lb, ci.ub=ci.ub, comps=1, ylim=ylim, method=x$method, vc=vc, maxll=maxll, xlab=xlab, title=title, exp=ddd$exp)
    names(sav)[1] <- switch(comp, tau2="tau2", delta="delta")
    class(sav) <- "profile.rma"
 
    #########################################################################
 
    if (plot)
-      plot(sav, pch=pch, refline=refline, cline=cline, ...)
+      plot(sav, ...)
 
    #########################################################################
 
