@@ -1,6 +1,6 @@
 qqnorm.rma.glmm <- function(y, ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(y), must="rma.glmm", notav="rma.glmm")
 
