@@ -1,6 +1,6 @@
 qqnorm.rma.mv <- function(y, ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(y), must="rma.mv", notav="rma.mv")
 
