@@ -1,6 +1,6 @@
 ranef.rma.uni <- function(object, level, digits, transf, targs, ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(object), must="rma.uni", notav=c("rma.gen", "rma.uni.selmodel"))
 
@@ -11,6 +11,9 @@ ranef.rma.uni <- function(object, level, digits, transf, targs, ...) {
    if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
       stop(mstyle$stop("Unknown 'na.action' specified under options()."))
 
+   if (is.null(x$yi.f) || is.null(x$vi.f) || is.null(x$X.f))
+      stop(mstyle$stop("Information needed to compute the BLUPs is not available in the model object."))
+
    if (missing(level))
       level <- x$level
 
@@ -47,9 +50,11 @@ ranef.rma.uni <- function(object, level, digits, transf, targs, ...) {
    ### see Appendix in: Raudenbush, S. W., & Bryk, A. S. (1985). Empirical
    ### Bayes meta-analysis. Journal of Educational Statistics, 10(2), 75-98
 
-   li <- x$tau2.f / (x$tau2.f + x$vi.f)
+   x$tau2.f <- .expand1(x$tau2.f, x$k.f)
 
-   for (i in seq_len(x$k.f)[x$not.na]) { ### note: skipping NA cases
+   li <- ifelse(is.infinite(x$tau2.f), 1, x$tau2.f / (x$tau2.f + x$vi.f))
+
+   for (i in seq_len(x$k.f)[x$not.na]) { # note: skipping NA cases
       Xi <- matrix(x$X.f[i,], nrow=1)
       if (is.element(x$method, c("FE","EE","CE"))) {
          pred[i]  <- 0
@@ -75,6 +80,8 @@ ranef.rma.uni <- function(object, level, digits, transf, targs, ...) {
          pi.lb <- sapply(pi.lb, transf)
          pi.ub <- sapply(pi.ub, transf)
       } else {
+         if (!is.primitive(transf) && !is.null(targs) && length(formals(transf)) == 1L)
+            stop(mstyle$stop("Function specified via 'transf' does not appear to have an argument for 'targs'."))
          pred  <- sapply(pred, transf, targs)
          se    <- rep(NA_real_, x$k.f)
          pi.lb <- sapply(pi.lb, transf, targs)
