@@ -1,6 +1,6 @@
 replmiss <- function(x, y, data) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    ### check if data argument has been specified
 
@@ -25,15 +25,14 @@ replmiss <- function(x, y, data) {
    if (length(x) == 0L)
       x <- rep(NA_real_, length(y))
 
-   ### in case user specifies a constant to use for replacement
+   ### in case user specified a constant for y to use for replacement
 
-   if (length(y) == 1L)
-      y <- rep(y, length(x))
+   y <- .expand1(y, length(x))
 
    ### check that x and y are of the same length
 
    if (length(x) != length(y))
-      stop(mstyle$stop("Length of 'x' and 'y' is not the same."))
+      stop(mstyle$stop("Length of 'x' and 'y' are not the same."))
 
    #x <- ifelse(is.na(x), y, x) # this is quite a bit slower than the following
    is.na.x <- is.na(x)
