@@ -1,6 +1,6 @@
 residuals.rma <- function(object, type="response", ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(object), must="rma")
 
@@ -9,6 +9,9 @@ residuals.rma <- function(object, type="response", ...) {
    if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
       stop(mstyle$stop("Unknown 'na.action' specified under options()."))
 
+   if (is.null(object$yi.f) || is.null(object$X.f))
+      stop(mstyle$stop("Information needed to compute the residuals is not available in the model object."))
+
    type <- match.arg(type, c("response", "rstandard", "rstudent", "pearson", "cholesky"))
 
    ### for objects of class "rma.mh" and "rma.peto", use rstandard() to get the Pearson residuals
@@ -43,8 +46,8 @@ residuals.rma <- function(object, type="response", ...) {
 
    if (type == "pearson") {
 
-      if (inherits(object, c("rma.mh", "rma.peto", "rma.glmm")))
-         stop(mstyle$stop("Extraction of Pearson residuals not available for objects of class \"rma.mh\", \"rma.peto\", or \"rma.glmm\"."))
+      if (inherits(object, "rma.glmm"))
+         stop(mstyle$stop("Extraction of Pearson residuals not available for objects of class \"rma.glmm\"."))
 
       out <- c(object$yi.f - object$X.f %*% object$beta)
       out[abs(out) < 100 * .Machine$double.eps] <- 0
@@ -89,7 +92,7 @@ residuals.rma <- function(object, type="response", ...) {
          out <- out[object$not.na]
 
       if (na.act == "na.exclude")
-         out[!object$not.na] <- NA
+         out[!object$not.na] <- NA_real_
 
       if (na.act == "na.fail" && any(!object$not.na))
          stop(mstyle$stop("Missing values in results."))
