@@ -1,6 +1,6 @@
 rstandard.rma.mh <- function(model, digits, ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(model), must="rma.mh")
 
@@ -9,6 +9,9 @@ rstandard.rma.mh <- function(model, digits, ...) {
    if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
       stop(mstyle$stop("Unknown 'na.action' specified under options()."))
 
+   if (is.null(model$yi.f))
+      stop(mstyle$stop("Information needed to compute the residuals is not available in the model object."))
+
    x <- model
 
    if (missing(digits)) {
@@ -22,7 +25,7 @@ rstandard.rma.mh <- function(model, digits, ...) {
    resid <- c(x$yi.f - x$beta)
 
    resid[abs(resid) < 100 * .Machine$double.eps] <- 0
-   #resid[abs(resid) < 100 * .Machine$double.eps * median(abs(resid), na.rm=TRUE)] <- 0 ### see lm.influence
+   #resid[abs(resid) < 100 * .Machine$double.eps * median(abs(resid), na.rm=TRUE)] <- 0 # see lm.influence
 
    ### note: these are like Pearson (or semi-standardized) residuals
 
