@@ -1,6 +1,6 @@
 rstandard.rma.mv <- function(model, digits, cluster, ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(model), must="rma.mv", notav="robust.rma")
 
@@ -10,6 +10,9 @@ rstandard.rma.mv <- function(model, digits, cluster, ...) {
    if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
       stop(mstyle$stop("Unknown 'na.action' specified under options()."))
 
+   if (is.null(model$yi) || is.null(model$X))
+      stop(mstyle$stop("Information needed to compute the residuals is not available in the model object."))
+
    x <- model
 
    if (missing(digits)) {
@@ -31,13 +34,12 @@ rstandard.rma.mv <- function(model, digits, cluster, ...) {
 
    ### process cluster variable
 
-   # note: cluster variable is assumed to be of the same length as the size of
-   # the original dataset passed to the model fitting function and so we apply
-   # the same subsetting and removing of missings (if necessary) as was done
-   # during model fitting
+   ### note: cluster variable must be of the same length as the original dataset
+   ###       so we have to apply the same subsetting (if necessary) and removing
+   ###       of NAs as was done during model fitting
 
    if (length(cluster) != x$k.all)
-      stop(mstyle$stop(paste0("Length of variable specified via 'cluster' (", length(cluster), ") does not match length of data (", x$k.all, ").")))
+      stop(mstyle$stop(paste0("Length of the variable specified via 'cluster' (", length(cluster), ") does not match the length of the data (", x$k.all, ").")))
 
    cluster <- .getsubset(cluster, x$subset)
 
@@ -66,7 +68,7 @@ rstandard.rma.mv <- function(model, digits, cluster, ...) {
    ei <- c(x$yi - x$X %*% x$beta)
 
    ei[abs(ei) < 100 * .Machine$double.eps] <- 0
-   #ei[abs(ei) < 100 * .Machine$double.eps * median(abs(ei), na.rm=TRUE)] <- 0 ### see lm.influence
+   #ei[abs(ei) < 100 * .Machine$double.eps * median(abs(ei), na.rm=TRUE)] <- 0 # see lm.influence
 
    ### don't allow this; the SEs of the residuals cannot be estimated consistently for "robust.rma" objects
    #if (inherits(x, "robust.rma")) {
