@@ -1,6 +1,6 @@
 rstudent.rma.mh <- function(model, digits, progbar=FALSE, ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(model), must="rma.mh")
 
@@ -9,6 +9,9 @@ rstudent.rma.mh <- function(model, digits, progbar=FALSE, ...) {
    if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
       stop(mstyle$stop("Unknown 'na.action' specified under options()."))
 
+   if (is.null(model$outdat.f))
+      stop(mstyle$stop("Information needed to compute the residuals is not available in the model object."))
+
    x <- model
 
    if (missing(digits)) {
@@ -19,11 +22,14 @@ rstudent.rma.mh <- function(model, digits, progbar=FALSE, ...) {
 
    ddd <- list(...)
 
-   .chkdots(ddd, c("time"))
+   .chkdots(ddd, c("time", "code1", "code2"))
 
    if (.isTRUE(ddd$time))
       time.start <- proc.time()
 
+   if (!is.null(ddd[["code1"]]))
+      eval(expr = parse(text = ddd[["code1"]]))
+
    #########################################################################
 
    delpred  <- rep(NA_real_, x$k.f)
@@ -43,6 +49,9 @@ rstudent.rma.mh <- function(model, digits, progbar=FALSE, ...) {
       if (progbar)
          pbapply::setpb(pbar, i)
 
+      if (!is.null(ddd[["code2"]]))
+         eval(expr = parse(text = ddd[["code2"]]))
+
       if (!x$not.na[i])
          next
 
@@ -68,7 +77,7 @@ rstudent.rma.mh <- function(model, digits, progbar=FALSE, ...) {
 
    resid <- x$yi.f - delpred
    resid[abs(resid) < 100 * .Machine$double.eps] <- 0
-   #resid[abs(resid) < 100 * .Machine$double.eps * median(abs(resid), na.rm=TRUE)] <- 0 ### see lm.influence
+   #resid[abs(resid) < 100 * .Machine$double.eps * median(abs(resid), na.rm=TRUE)] <- 0 # see lm.influence
    seresid <- sqrt(x$vi.f + vdelpred)
    stresid <- resid / seresid
 
