@@ -7,7 +7,7 @@ tes <- function(x, vi, sei, subset, data,
 
    #########################################################################
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    na.act <- getOption("na.action")
 
@@ -43,6 +43,9 @@ tes <- function(x, vi, sei, subset, data,
 
       .chkclass(class(x), must="rma", notav=c("rma.glmm", "rma.mv", "robust.rma", "rma.ls", "rma.gen", "rma.uni.selmodel"))
 
+      if (is.null(x$yi) || is.null(x$vi))
+         stop(mstyle$stop("Information needed to carry out the test is not available in the model object."))
+
       ### set defaults for digits
 
       if (missing(digits)) {
@@ -99,12 +102,12 @@ tes <- function(x, vi, sei, subset, data,
       }
 
       if (is.null(vi))
-         stop(mstyle$stop("Must specify 'vi' or 'sei' argument."))
+         stop(mstyle$stop("Must specify the 'vi' or 'sei' argument."))
 
       ### check length of yi and vi
 
       if (length(yi) != length(vi))
-         stop(mstyle$stop("Length of 'yi' and 'vi' (or 'sei') is not the same."))
+         stop(mstyle$stop("Length of 'yi' and 'vi' (or 'sei') are not the same."))
 
       ### check 'vi' argument for potential misuse
 
@@ -135,36 +138,11 @@ tes <- function(x, vi, sei, subset, data,
 
       .chkdots(ddd, c("correct", "rel.tol", "subdivisions", "tau2.lb", "find.lim"))
 
-      if (!is.null(ddd$correct)) {
-         correct <- ddd$correct
-      } else {
-         correct <- FALSE
-      }
-
-      if (!is.null(ddd$rel.tol)) {
-         rel.tol <- ddd$rel.tol
-      } else {
-         rel.tol <- .Machine$double.eps^0.25
-      }
-
-      if (!is.null(ddd$subdivisions)) {
-         subdivisions <- ddd$subdivisions
-      } else {
-         subdivisions <- 100L
-      }
-
-      if (!is.null(ddd$tau2.lb)) {
-         tau2.lb <- ddd$tau2.lb
-      } else {
-         #tau2.lb <- 0.0001
-         tau2.lb <- 0
-      }
-
-      if (!is.null(ddd$find.lim)) {
-         find.lim <- ddd$find.lim
-      } else {
-         find.lim <- TRUE
-      }
+      correct      <- .chkddd(ddd$correct, FALSE)
+      rel.tol      <- .chkddd(ddd$rel.tol, .Machine$double.eps^0.25)
+      subdivisions <- .chkddd(ddd$subdivisions, 100L)
+      tau2.lb      <- .chkddd(ddd$tau2.lb, 0) # 0.0001
+      find.lim     <- .chkddd(ddd$find.lim, TRUE)
 
       #########################################################################
 
@@ -196,7 +174,7 @@ tes <- function(x, vi, sei, subset, data,
          }
 
          if (length(theta) != k.f)
-            stop(mstyle$stop("Length of 'theta' and 'yi' is not the same."))
+            stop(mstyle$stop("Length of 'theta' and 'yi' are not the same."))
 
       }
 
