@@ -1,7 +1,7 @@
 to.long <- function(measure, ai, bi, ci, di, n1i, n2i, x1i, x2i, t1i, t2i, m1i, m2i, sd1i, sd2i, xi, mi, ri, ti, sdi, ni,
 data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE, var.names) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    ### check argument specifications
 
@@ -20,14 +20,15 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
                               "RPB","RBIS","D2OR","D2ORN","D2ORL",                 # - transformations to r_PB, r_BIS, and log(OR)
                               "COR","UCOR","ZCOR",                                 # correlations (raw and r-to-z transformed)
                               "PCOR","ZPCOR","SPCOR",                              # partial and semi-partial correlations
-                              "PR","PLN","PLO","PAS","PFT",                        # single proportions (and transformations thereof)
+                              "R2","ZR2","R2F","ZR2F",                             # coefficient of determination (raw and r-to-z transformed)
+                              "PR","PLN","PLO","PRZ","PAS","PFT",                  # single proportions (and transformations thereof)
                               "IR","IRLN","IRS","IRFT",                            # single-group person-time data (and transformations thereof)
-                              "MN","MNLN","CVLN","SDLN","SMN",                     # mean, log(mean), log(CV), log(SD), standardized mean
+                              "MN","SMN","MNLN","CVLN","SDLN",                     # mean, single-group standardized mean, log(mean), log(CV), log(SD),
                               "MC","SMCC","SMCR","SMCRH","ROMC","CVRC","VRC",      # raw/standardized mean change, log(ROM), CVR, and VR for dependent samples
                               "ARAW","AHW","ABT")))                                # alpha (and transformations thereof)
       stop(mstyle$stop("Unknown 'measure' specified."))
 
-   if (is.element(measure, c("CVR","VR","PCOR","ZPCOR","SPCOR","CVLN","SDLN","VRC")))
+   if (is.element(measure, c("CVR","VR","PCOR","ZPCOR","SPCOR","R2","ZR2","R2F","ZR2F","CVLN","SDLN","VRC")))
       stop(mstyle$stop("Function not available for this outcome measure."))
 
    na.act <- getOption("na.action")
@@ -54,6 +55,30 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
          data <- data.frame(data)
    }
 
+   doappend <- FALSE
+
+   if (has.data && is.logical(append) && isTRUE(append)) {
+      doappend <- TRUE
+      appendvars <- seq_len(ncol(data))
+   }
+
+   if (has.data && is.numeric(append)) {
+      doappend <- TRUE
+      append <- unique(round(append))
+      append <- append[which(append >= 1)]
+      append <- append[which(append <= ncol(data))]
+      append <- c(na.omit(append))
+      appendvars <- append
+   }
+
+   if (has.data && is.character(append)) {
+      doappend <- TRUE
+      append <- unique(append)
+      append <- pmatch(append, colnames(data))
+      append <- c(na.omit(append))
+      appendvars <- append
+   }
+
    mf <- match.call()
 
    ### get slab and subset arguments (will be NULL when unspecified)
@@ -89,9 +114,9 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       di <- replmiss(di, n2i-ci)
 
       if (!.all.specified(ai, bi, ci, di))
-         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
+         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required information is specified\n  via the appropriate arguments (i.e., ai, bi, ci, di or ai, n1i, ci, n2i)."))
 
-      k <- length(ai) ### number of outcomes before subsetting
+      k <- length(ai) # number of outcomes before subsetting
 
       if (!is.null(subset)) {
          subset <- .chksubset(subset, k)
@@ -113,17 +138,17 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       if (any(c(n1i < 0, n2i < 0), na.rm=TRUE))
          stop(mstyle$stop("One or more group sizes are negative."))
 
-      ni.u <- ai + bi + ci + di ### unadjusted total sample sizes
+      ni.u <- ai + bi + ci + di # unadjusted total sample sizes
 
       ### if drop00=TRUE, set counts to NA for studies that have no events (or all events) in both arms
 
       if (drop00) {
          id00 <- c(ai == 0L & ci == 0L) | c(bi == 0L & di == 0L)
          id00[is.na(id00)] <- FALSE
-         ai[id00] <- NA
-         bi[id00] <- NA
-         ci[id00] <- NA
-         di[id00] <- NA
+         ai[id00] <- NA_real_
+         bi[id00] <- NA_real_
+         ci[id00] <- NA_real_
+         di[id00] <- NA_real_
       }
 
       if (to == "all") {
@@ -181,12 +206,12 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       t2i <- .getx("t2i", mf=mf, data=data, checknumeric=TRUE)
 
       if (!.all.specified(x1i, x2i, t1i, t2i))
-         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
+         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required information is specified\n  via the appropriate arguments (i.e., x1i, x2i, t1i, t2i)."))
 
       if (!.equal.length(x1i, x2i, t1i, t2i))
          stop(mstyle$stop("Supplied data vectors are not all of the same length."))
 
-      k <- length(x1i) ### number of outcomes before subsetting
+      k <- length(x1i) # number of outcomes before subsetting
 
       if (!is.null(subset)) {
          subset <- .chksubset(subset, k)
@@ -202,15 +227,15 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       if (any(c(t1i, t2i) <= 0, na.rm=TRUE))
          stop(mstyle$stop("One or more person-times are <= 0."))
 
-      ni.u <- t1i + t2i ### unadjusted total sample sizes
+      ni.u <- t1i + t2i # unadjusted total sample sizes
 
       ### if drop00=TRUE, set counts to NA for studies that have no events in both arms
 
       if (drop00) {
          id00 <- c(x1i == 0L & x2i == 0L)
          id00[is.na(id00)] <- FALSE
-         x1i[id00] <- NA
-         x2i[id00] <- NA
+         x1i[id00] <- NA_real_
+         x2i[id00] <- NA_real_
       }
 
       if (to == "all") {
@@ -264,12 +289,12 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       n2i  <- .getx("n2i",  mf=mf, data=data, checknumeric=TRUE)
 
       if (!.all.specified(m1i, m2i, sd1i, sd2i, n1i, n2i))
-         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
+         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required information is specified\n  via the appropriate arguments (i.e., m1i, m2i, sd1i, sd2i, n1i, n2i)."))
 
       if (!.equal.length(m1i, m2i, sd1i, sd2i, n1i, n2i))
          stop(mstyle$stop("Supplied data vectors are not all of the same length."))
 
-      k <- length(n1i) ### number of outcomes before subsetting
+      k <- length(n1i) # number of outcomes before subsetting
 
       if (!is.null(subset)) {
          subset <- .chksubset(subset, k)
@@ -287,7 +312,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       if (any(c(n1i, n2i) <= 0, na.rm=TRUE))
          stop(mstyle$stop("One or more group sizes are <= 0."))
 
-      ni.u <- n1i + n2i ### unadjusted total sample sizes
+      ni.u <- n1i + n2i # unadjusted total sample sizes
 
    }
 
@@ -297,14 +322,17 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
       ri <- .getx("ri", mf=mf, data=data, checknumeric=TRUE)
       ni <- .getx("ni", mf=mf, data=data, checknumeric=TRUE)
+      ti <- .getx("ti", mf=mf, data=data, checknumeric=TRUE)
 
-      if (!.all.specified(ri, ni))
-         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
-
-      if (!.equal.length(ri, ni))
+      if (!.equal.length(ri, ni, ti))
          stop(mstyle$stop("Supplied data vectors are not all of the same length."))
 
-      k <- length(ri) ### number of outcomes before subsetting
+      ri <- replmiss(ri, ti / sqrt(ni - 2 + ti^2))
+
+      if (!.all.specified(ri, ni))
+         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required information is specified\n  via the appropriate arguments (i.e., ri, ni)."))
+
+      k <- length(ri) # number of outcomes before subsetting
 
       if (!is.null(subset)) {
          subset <- .chksubset(subset, k=k)
@@ -318,13 +346,13 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       if (any(ni <= 0, na.rm=TRUE))
          stop(mstyle$stop("One or more sample sizes are <= 0."))
 
-      ni.u <- ni ### unadjusted total sample sizes
+      ni.u <- ni # unadjusted total sample sizes
 
    }
 
    #########################################################################
 
-   if (is.element(measure, c("PR","PLN","PLO","PAS","PFT"))) {
+   if (is.element(measure, c("PR","PLN","PLO","PRZ","PAS","PFT"))) {
 
       xi <- .getx("xi", mf=mf, data=data, checknumeric=TRUE)
       mi <- .getx("mi", mf=mf, data=data, checknumeric=TRUE)
@@ -341,9 +369,9 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       mi <- replmiss(mi, ni-xi)
 
       if (!.all.specified(xi, mi))
-         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
+         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required information is specified\n  via the appropriate arguments (i.e., xi, mi or xi, ni)."))
 
-      k <- length(xi) ### number of outcomes before subsetting
+      k <- length(xi) # number of outcomes before subsetting
 
       if (!is.null(subset)) {
          subset <- .chksubset(subset, k)
@@ -362,7 +390,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       if (any(ni <= 0, na.rm=TRUE))
          stop(mstyle$stop("One or more group sizes are <= 0."))
 
-      ni.u <- ni ### unadjusted total sample sizes
+      ni.u <- ni # unadjusted total sample sizes
 
       if (to == "all") {
 
@@ -411,12 +439,12 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       ti <- .getx("ti", mf=mf, data=data, checknumeric=TRUE)
 
       if (!.all.specified(xi, ti))
-         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
+         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required information is specified\n  via the appropriate arguments (i.e., xi, ti)."))
 
       if (!.equal.length(xi, ti))
          stop(mstyle$stop("Supplied data vectors are not all of the same length."))
 
-      k <- length(xi) ### number of outcomes before subsetting
+      k <- length(xi) # number of outcomes before subsetting
 
       if (!is.null(subset)) {
          subset <- .chksubset(subset, k)
@@ -430,7 +458,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       if (any(ti <= 0, na.rm=TRUE))
          stop(mstyle$stop("One or more person-times are <= 0."))
 
-      ni.u <- ti ### unadjusted total sample sizes
+      ni.u <- ti # unadjusted total sample sizes
 
       if (to == "all") {
 
@@ -470,19 +498,19 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
    #########################################################################
 
-   if (is.element(measure, c("MN","MNLN","SMN"))) {
+   if (is.element(measure, c("MN","SMN","MNLN"))) {
 
       mi  <- .getx("mi",  mf=mf, data=data, checknumeric=TRUE)
       sdi <- .getx("sdi", mf=mf, data=data, checknumeric=TRUE)
       ni  <- .getx("ni",  mf=mf, data=data, checknumeric=TRUE)
 
       if (!.all.specified(mi, sdi, ni))
-         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
+         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required information is specified\n  via the appropriate arguments (i.e., mi, sdi, ni)."))
 
       if (!.equal.length(mi, sdi, ni))
          stop(mstyle$stop("Supplied data vectors are not all of the same length."))
 
-      k <- length(ni) ### number of outcomes before subsetting
+      k <- length(ni) # number of outcomes before subsetting
 
       if (!is.null(subset)) {
          subset <- .chksubset(subset, k)
@@ -500,7 +528,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       if (is.element(measure, c("MNLN","CVLN")) && any(mi < 0, na.rm=TRUE))
          stop(mstyle$stop("One or more means are negative."))
 
-      ni.u <- ni ### unadjusted total sample sizes
+      ni.u <- ni # unadjusted total sample sizes
 
   }
 
@@ -512,15 +540,15 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       m2i  <- .getx("m2i",  mf=mf, data=data, checknumeric=TRUE)
       sd1i <- .getx("sd1i", mf=mf, data=data, checknumeric=TRUE)
       sd2i <- .getx("sd2i", mf=mf, data=data, checknumeric=TRUE)
-      ri   <- .getx("ri",   mf=mf, data=data, checknumeric=TRUE) ### for SMCR, do not need to supply this
+      ri   <- .getx("ri",   mf=mf, data=data, checknumeric=TRUE) # for SMCR, do not need to supply this
       ni   <- .getx("ni",   mf=mf, data=data, checknumeric=TRUE)
 
-      k <- length(m1i) ### number of outcomes before subsetting
+      k <- length(m1i) # number of outcomes before subsetting
 
       if (is.element(measure, c("MC","SMCC","SMCRH","ROMC","CVRC"))) {
 
          if (!.all.specified(m1i, m2i, sd1i, sd2i, ni, ri))
-            stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
+            stop(mstyle$stop("Cannot compute outcomes. Check that all of the required information is specified\n  via the appropriate arguments (i.e., m1i, m2i, sd1i, sd2i, ni, ri)."))
 
          if (!.equal.length(m1i, m2i, sd1i, sd2i, ni, ri))
             stop(mstyle$stop("Supplied data vectors are not all of the same length."))
@@ -528,7 +556,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       } else {
 
          if (!.all.specified(m1i, m2i, sd1i, ni, ri))
-            stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
+            stop(mstyle$stop("Cannot compute outcomes. Check that all of the required information is specified\n  via the appropriate arguments (i.e., m1i, m2i, sd1i, ni, ri)."))
 
          if (!.equal.length(m1i, m2i, sd1i, ni, ri))
             stop(mstyle$stop("Supplied data vectors are not all of the same length."))
@@ -559,7 +587,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       if (any(ni <= 0, na.rm=TRUE))
          stop(mstyle$stop("One or more sample sizes are <= 0."))
 
-      ni.u <- ni ### unadjusted total sample sizes
+      ni.u <- ni # unadjusted total sample sizes
 
    }
 
@@ -572,12 +600,12 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       ni <- .getx("ni", mf=mf, data=data, checknumeric=TRUE)
 
       if (!.all.specified(ai, mi, ni))
-         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
+         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required information is specified\n  via the appropriate arguments (i.e., ai, mi, ni)."))
 
       if (!.equal.length(ai, mi, ni))
          stop(mstyle$stop("Supplied data vectors are not all of the same length."))
 
-      k <- length(ai) ### number of outcomes before subsetting
+      k <- length(ai) # number of outcomes before subsetting
 
       if (!is.null(subset)) {
          subset <- .chksubset(subset, k)
@@ -595,7 +623,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       if (any(ni <= 0, na.rm=TRUE))
          stop(mstyle$stop("One or more sample sizes are <= 0."))
 
-      ni.u <- ni ### unadjusted total sample sizes
+      ni.u <- ni # unadjusted total sample sizes
 
    }
 
@@ -615,7 +643,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
          stop(mstyle$stop("NAs in study labels."))
 
       if (length(slab) != k)
-         stop(mstyle$stop("Study labels not of same length as data."))
+         stop(mstyle$stop(paste0("Length of the 'slab' argument (", length(slab), ") does not correspond to the size of the dataset (", k, ").")))
 
       if (is.factor(slab))
          slab <- as.character(slab)
@@ -657,7 +685,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
             slab <- slab[not.na]
             if (has.data)
                data <- data[not.na,]
-            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
+            warning(mstyle$warning(paste(sum(has.na), ifelse(sum(has.na) > 1, "tables", "table"), "with NAs omitted.")), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -669,7 +697,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
       ### at least one study left?
 
-      if (k < 1)
+      if (k < 1L)
          stop(mstyle$stop("Processing terminated since k = 0."))
 
       ### create long format dataset
@@ -696,8 +724,8 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
          dat[[2]] <- factor(dat[[2]], levels=c(2,1))
          dat[[3]] <- factor(dat[[3]], levels=c(2,1))
 
-         if (has.data && append)
-            dat <- cbind(data[rep(seq_len(k), each=4L),], dat)
+         if (doappend)
+            dat <- cbind(data[rep(seq_len(k), each=4L),appendvars,drop=FALSE], dat)
 
       } else {
 
@@ -720,8 +748,8 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
          dat[[1]] <- factor(dat[[1]])
          dat[[2]] <- factor(dat[[2]], levels=c(2,1))
 
-         if (has.data && append)
-            dat <- cbind(data[rep(seq_len(k), each=2L),], dat)
+         if (doappend)
+            dat <- cbind(data[rep(seq_len(k), each=2L),appendvars,drop=FALSE], dat)
 
       }
 
@@ -747,7 +775,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
             slab <- slab[not.na]
             if (has.data)
                data <- data[not.na,]
-            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
+            warning(mstyle$warning(paste(sum(has.na), ifelse(sum(has.na) > 1, "tables", "table"), "with NAs omitted.")), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -759,7 +787,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
       ### at least one study left?
 
-      if (k < 1)
+      if (k < 1L)
          stop(mstyle$stop("Processing terminated since k = 0."))
 
       ### create long format dataset
@@ -786,8 +814,8 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
          dat[[2]] <- factor(dat[[2]], levels=c(2,1))
          dat[[3]] <- factor(dat[[3]], levels=c(2,1))
 
-         if (has.data && append)
-            dat <- data.frame(data[rep(seq_len(k), each=4L),], dat)
+         if (doappend)
+            dat <- data.frame(data[rep(seq_len(k), each=4L),appendvars,drop=FALSE], dat)
 
       } else {
 
@@ -810,8 +838,8 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
          dat[[1]] <- factor(dat[[1]])
          dat[[2]] <- factor(dat[[2]], levels=c(2,1))
 
-         if (has.data && append)
-            dat <- cbind(data[rep(seq_len(k), each=2L),], dat)
+         if (doappend)
+            dat <- cbind(data[rep(seq_len(k), each=2L),appendvars,drop=FALSE], dat)
 
       }
 
@@ -837,7 +865,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
             slab <- slab[not.na]
             if (has.data)
                data <- data[not.na,]
-            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
+            warning(mstyle$warning(paste(sum(has.na), ifelse(sum(has.na) > 1, "tables", "table"), "with NAs omitted.")), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -849,7 +877,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
       ### at least one study left?
 
-      if (k < 1)
+      if (k < 1L)
          stop(mstyle$stop("Processing terminated since k = 0."))
 
       ### create long format dataset
@@ -876,8 +904,8 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
          dat[[2]] <- factor(dat[[2]], levels=c(2,1))
          dat[[3]] <- factor(dat[[3]], levels=c(2,1))
 
-         if (has.data && append)
-            dat <- cbind(data[rep(seq_len(k), each=4L),], dat)
+         if (doappend)
+            dat <- cbind(data[rep(seq_len(k), each=4L),appendvars,drop=FALSE], dat)
 
       } else {
 
@@ -900,8 +928,8 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
          dat[[1]] <- factor(dat[[1]])
          dat[[2]] <- factor(dat[[2]], levels=c(2,1))
 
-         if (has.data && append)
-            dat <- cbind(data[rep(seq_len(k), each=2L),], dat)
+         if (doappend)
+            dat <- cbind(data[rep(seq_len(k), each=2L),appendvars,drop=FALSE], dat)
 
       }
 
@@ -927,7 +955,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
             slab <- slab[not.na]
             if (has.data)
                data <- data[not.na,]
-            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
+            warning(mstyle$warning(paste(sum(has.na), ifelse(sum(has.na) > 1, "tables", "table"), "with NAs omitted.")), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -939,7 +967,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
       ### at least one study left?
 
-      if (k < 1)
+      if (k < 1L)
          stop(mstyle$stop("Processing terminated since k = 0."))
 
       ### create long format dataset
@@ -961,8 +989,8 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       dat[[1]] <- factor(dat[[1]])
       dat[[2]] <- factor(dat[[2]], levels=c(2,1))
 
-      if (has.data && append)
-         dat <- cbind(data[rep(seq_len(k), each=2L),], dat)
+      if (doappend)
+         dat <- cbind(data[rep(seq_len(k), each=2L),appendvars,drop=FALSE], dat)
 
    }
 
@@ -988,7 +1016,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
             slab <- slab[not.na]
             if (has.data)
                data <- data[not.na,]
-            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
+            warning(mstyle$warning(paste(sum(has.na), ifelse(sum(has.na) > 1, "tables", "table"), "with NAs omitted.")), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -1000,7 +1028,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
       ### at least one study left?
 
-      if (k < 1)
+      if (k < 1L)
          stop(mstyle$stop("Processing terminated since k = 0."))
 
       ### create long format dataset
@@ -1023,8 +1051,8 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
       dat[[1]] <- factor(dat[[1]])
       dat[[2]] <- factor(dat[[2]], levels=c(2,1))
 
-      if (has.data && append)
-         dat <- cbind(data[rep(seq_len(k), each=2L),], dat)
+      if (doappend)
+         dat <- cbind(data[rep(seq_len(k), each=2L),appendvars,drop=FALSE], dat)
 
    }
 
@@ -1046,7 +1074,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
             slab <- slab[not.na]
             if (has.data)
                data <- data[not.na,]
-            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
+            warning(mstyle$warning(paste(sum(has.na), ifelse(sum(has.na) > 1, "tables", "table"), "with NAs omitted.")), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -1058,7 +1086,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
       ### at least one study left?
 
-      if (k < 1)
+      if (k < 1L)
          stop(mstyle$stop("Processing terminated since k = 0."))
 
       ### create long format dataset
@@ -1078,14 +1106,14 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
       dat[[1]] <- factor(dat[[1]])
 
-      if (has.data && append)
-         dat <- cbind(data, dat)
+      if (doappend)
+         dat <- cbind(data[,appendvars,drop=FALSE], dat)
 
    }
 
    #########################################################################
 
-   if (is.element(measure, c("PR","PLN","PLO","PAS","PFT"))) {
+   if (is.element(measure, c("PR","PLN","PLO","PRZ","PAS","PFT"))) {
 
       ### check for NAs in table data and act accordingly
 
@@ -1101,7 +1129,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
             slab <- slab[not.na]
             if (has.data)
                data <- data[not.na,]
-            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
+            warning(mstyle$warning(paste(sum(has.na), ifelse(sum(has.na) > 1, "tables", "table"), "with NAs omitted.")), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -1113,7 +1141,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
       ### at least one study left?
 
-      if (k < 1)
+      if (k < 1L)
          stop(mstyle$stop("Processing terminated since k = 0."))
 
       ### create long format dataset
@@ -1138,8 +1166,8 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
          dat[[1]] <- factor(dat[[1]])
          dat[[2]] <- factor(dat[[2]], levels=c(2,1))
 
-         if (has.data && append)
-            dat <- cbind(data[rep(seq_len(k), each=2L),], dat)
+         if (doappend)
+            dat <- cbind(data[rep(seq_len(k), each=2L),appendvars,drop=FALSE], dat)
 
       } else {
 
@@ -1160,8 +1188,8 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
          dat[[1]] <- factor(dat[[1]])
 
-         if (has.data && append)
-            dat <- cbind(data, dat)
+         if (doappend)
+            dat <- cbind(data[,appendvars,drop=FALSE], dat)
 
       }
 
@@ -1185,7 +1213,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
             slab <- slab[not.na]
             if (has.data)
                data <- data[not.na,]
-            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
+            warning(mstyle$warning(paste(sum(has.na), ifelse(sum(has.na) > 1, "tables", "table"), "with NAs omitted.")), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -1197,7 +1225,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
       ### at least one study left?
 
-      if (k < 1)
+      if (k < 1L)
          stop(mstyle$stop("Processing terminated since k = 0."))
 
       ### create long format dataset
@@ -1217,14 +1245,14 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
       dat[[1]] <- factor(dat[[1]])
 
-      if (has.data && append)
-         dat <- cbind(data, dat)
+      if (doappend)
+         dat <- cbind(data[,appendvars,drop=FALSE], dat)
 
    }
 
    #########################################################################
 
-   if (is.element(measure, c("MN","MNLN","SMN"))) {
+   if (is.element(measure, c("MN","SMN","MNLN"))) {
 
       ### check for NAs in table data and act accordingly
 
@@ -1241,7 +1269,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
             slab <- slab[not.na]
             if (has.data)
                data <- data[not.na,]
-            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
+            warning(mstyle$warning(paste(sum(has.na), ifelse(sum(has.na) > 1, "tables", "table"), "with NAs omitted.")), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -1253,7 +1281,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
       ### at least one study left?
 
-      if (k < 1)
+      if (k < 1L)
          stop(mstyle$stop("Processing terminated since k = 0."))
 
       ### create long format dataset
@@ -1274,8 +1302,8 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
       dat[[1]] <- factor(dat[[1]])
 
-      if (has.data && append)
-         dat <- cbind(data, dat)
+      if (doappend)
+         dat <- cbind(data[,appendvars,drop=FALSE], dat)
 
    }
 
@@ -1306,7 +1334,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
             slab <- slab[not.na]
             if (has.data)
                data <- data[not.na,]
-            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
+            warning(mstyle$warning(paste(sum(has.na), ifelse(sum(has.na) > 1, "tables", "table"), "with NAs omitted.")), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -1318,7 +1346,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
       ### at least one study left?
 
-      if (k < 1)
+      if (k < 1L)
          stop(mstyle$stop("Processing terminated since k = 0."))
 
       ### create long format dataset
@@ -1344,8 +1372,8 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
          dat[[1]] <- factor(dat[[1]])
 
-         if (has.data && append)
-            dat <- cbind(data, dat)
+         if (doappend)
+            dat <- cbind(data[,appendvars,drop=FALSE], dat)
 
       } else {
 
@@ -1367,8 +1395,8 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
          dat[[1]] <- factor(dat[[1]])
 
-         if (has.data && append)
-            dat <- cbind(data, dat)
+         if (doappend)
+            dat <- cbind(data[,appendvars,drop=FALSE], dat)
 
       }
 
@@ -1393,7 +1421,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
             slab <- slab[not.na]
             if (has.data)
                data <- data[not.na,]
-            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
+            warning(mstyle$warning(paste(sum(has.na), ifelse(sum(has.na) > 1, "tables", "table"), "with NAs omitted.")), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -1405,7 +1433,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
       ### at least one study left?
 
-      if (k < 1)
+      if (k < 1L)
          stop(mstyle$stop("Processing terminated since k = 0."))
 
       ### create long format dataset
@@ -1426,8 +1454,8 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, vlong=FALSE, append=TRUE,
 
       dat[[1]] <- factor(dat[[1]])
 
-      if (has.data && append)
-         dat <- data.frame(data, dat)
+      if (doappend)
+         dat <- data.frame(data[,appendvars,drop=FALSE], dat)
 
    }
 
