@@ -1,9 +1,12 @@
 weights.rma.mh <- function(object, type="diagonal", ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(object), must="rma.mh")
 
+   if (is.null(object$outdat))
+      stop(mstyle$stop("Information needed to compute the weights is not available in the model object."))
+
    na.act <- getOption("na.action")
 
    if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
