@@ -1,9 +1,12 @@
 weights.rma.peto <- function(object, type="diagonal", ...) {
 
-   mstyle <- .get.mstyle("crayon" %in% .packages())
+   mstyle <- .get.mstyle()
 
    .chkclass(class(object), must="rma.peto")
 
+   if (is.null(object$outdat))
+      stop(mstyle$stop("Information needed to compute the weights is not available in the model object."))
+
    na.act <- getOption("na.action")
 
    if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
@@ -15,13 +18,13 @@ weights.rma.peto <- function(object, type="diagonal", ...) {
 
    #########################################################################
 
-   n1i <- x$ai + x$bi
-   n2i <- x$ci + x$di
-   Ni  <- x$ai + x$bi + x$ci + x$di
-   xt  <- x$ai + x$ci
-   yt  <- x$bi + x$di
+   n1i <- with(x$outdat, ai + bi)
+   n2i <- with(x$outdat, ci + di)
+   Ni  <- with(x$outdat, ai + bi + ci + di)
+   xt  <- with(x$outdat, ai + ci)
+   yt  <- with(x$outdat, bi + di)
 
-   wi  <- xt * yt * (n1i/Ni) * (n2i/Ni) / (Ni - 1)
+   wi <- xt * yt * (n1i/Ni) * (n2i/Ni) / (Ni - 1)
 
    #########################################################################
 
