@@ -1,6 +1,6 @@
-### library(metafor); library(testthat); Sys.setenv(NOT_CRAN="true")
+### library(metafor); library(testthat); Sys.setenv(NOT_CRAN="true"); Sys.setenv(RUN_VIS_TESTS="true")
 
-### see also: https://www.metafor-project.org/doku.php/analyses:gleser2009
+### see: https://www.metafor-project.org/doku.php/analyses:gleser2009
 
 source("settings.r")
 
@@ -31,7 +31,7 @@ test_that("results are correct for the multiple-treatment studies example with r
 
    V <- bldiag(lapply(split(dat, dat$study), calc.v))
 
-   res <- rma.mv(yi, V, mods = ~ factor(trt) - 1, data=dat, sparse=.sparse)
+   res <- rma.mv(yi, V, mods = ~ 0 + factor(trt), data=dat, sparse=.sparse)
 
    ### compare with results on page 361 (eq. 19.6)
    expect_equivalent(coef(res), c(0.0200, 0.0043, 0.0211), tolerance=.tol[["coef"]])
@@ -62,7 +62,7 @@ test_that("results are correct for the multiple-treatment studies example with l
 
    V <- bldiag(lapply(split(dat, dat$study), calc.v))
 
-   res <- rma.mv(yi, V, mods = ~ factor(trt) - 1, data=dat, sparse=.sparse)
+   res <- rma.mv(yi, V, mods = ~ 0 + factor(trt), data=dat, sparse=.sparse)
 
    ### compare with results on page 363
    expect_equivalent(coef(res), c(0.5099, 0.0044, 0.4301), tolerance=.tol[["coef"]])
@@ -93,7 +93,7 @@ test_that("results are correct for the multiple-treatment studies example with l
 
    V <- bldiag(lapply(split(dat, dat$study), calc.v))
 
-   res <- rma.mv(yi, V, mods = ~ factor(trt) - 1, data=dat, sparse=.sparse)
+   res <- rma.mv(yi, V, mods = ~ 0 + factor(trt), data=dat, sparse=.sparse)
 
    ### compare with results on page 363
    expect_equivalent(coef(res), c(0.4875, 0.0006, 0.4047), tolerance=.tol[["coef"]])
@@ -124,7 +124,7 @@ test_that("results are correct for the multiple-treatment studies example with d
 
    V <- bldiag(lapply(split(dat, dat$study), calc.v))
 
-   res <- rma.mv(yi, V, mods = ~ factor(trt) - 1, data=dat, sparse=.sparse)
+   res <- rma.mv(yi, V, mods = ~ 0 + factor(trt), data=dat, sparse=.sparse)
 
    ### compare with results on page 365
    expect_equivalent(coef(res)*2, c(0.1010, 0.0102, 0.0982), tolerance=.tol[["coef"]])
@@ -168,7 +168,7 @@ test_that("results are correct for the multiple-treatment studies example with s
 
    V <- bldiag(lapply(split(dat, dat$study), calc.v))
 
-   res <- rma.mv(yi, V, mods = ~ factor(trt) - 1, data=dat, sparse=.sparse)
+   res <- rma.mv(yi, V, mods = ~ 0 + factor(trt), data=dat, sparse=.sparse)
 
    ### compare with results on page 367
    expect_equivalent(coef(res), c(2.3743, 1.5702), tolerance=.tol[["coef"]])
@@ -206,7 +206,7 @@ test_that("results are correct for the multiple-endpoint studies example with st
 
    ### fit model
 
-   res <- rma.mv(yi, V, mods = ~ outcome - 1, data=dat, sparse=.sparse)
+   res <- rma.mv(yi, V, mods = ~ 0 + outcome, data=dat, sparse=.sparse)
 
    ### (results for this not given in chapter)
    expect_equivalent(coef(res), c(0.3617, 0.2051), tolerance=.tol[["coef"]])
