@@ -1,4 +1,4 @@
-### library(metafor); library(testthat); Sys.setenv(NOT_CRAN="true")
+### library(metafor); library(testthat); Sys.setenv(NOT_CRAN="true"); Sys.setenv(RUN_VIS_TESTS="true")
 
 context("Checking misc: escalc() function")
 
@@ -64,7 +64,7 @@ test_that("escalc() works correctly for measure='SMD/SMDH/ROM'", {
 
    sav <- escalc(measure="SMDH", m1i=m1i, sd1i=sd1i, n1i=n1i, m2i=m2i, sd2i=sd2i, n2i=n2i, data=dat, subset=1:4)
    expect_equivalent(sav$yi, c(-0.3553, -0.3465, -2.3018, -1.8880), tolerance=.tol[["est"]])
-   expect_equivalent(sav$vi, c( 0.0131,  0.0657,  0.0509,  0.1874), tolerance=.tol[["var"]])
+   expect_equivalent(sav$vi, c( 0.0132,  0.0674,  0.0515,  0.1961), tolerance=.tol[["var"]])
 
    sav <- escalc(measure="ROM", m1i=m1i, sd1i=sd1i, n1i=n1i, m2i=m2i, sd2i=sd2i, n2i=n2i, data=dat, subset=1:4)
    expect_equivalent(sav$yi, c(-0.3102, -0.0715, -0.6202, -0.7303), tolerance=.tol[["est"]])
@@ -72,6 +72,50 @@ test_that("escalc() works correctly for measure='SMD/SMDH/ROM'", {
 
 })
 
+test_that("escalc() works correctly for measure='CLES/AUC/CLESN/AUCN'", {
+
+   # dataset Table 1 from Hanley & McNeil (1982)
+   dat <- data.frame(status = rep(c(0,1), times=c(58,51)), x = c(rep(1:5, times=c(33,6,6,11,2)), rep(1:5, times=c(3,2,2,11,33))))
+   n1 <- sum(dat$status == 1)
+   n0 <- sum(dat$status == 0)
+   x1 <- dat$x[dat$status == 1]
+   x0 <- dat$x[dat$status == 0]
+   mean1 <- mean(x1)
+   mean0 <- mean(x0)
+   sd1   <- sd(x1)
+   sd0   <- sd(x0)
+
+   auc <- (mean(rank(c(x1,x0))[1:n1]) - mean(rank(c(x1,x0))[(n1+1):(n1+n0)])) / (n1+n0) + 1/2
+
+   sav <- escalc(measure="AUC", ai=auc, n1i=n1, n2i=n0)
+   expect_equivalent(sav$yi, 0.893171, tolerance=.tol[["est"]])
+   expect_equivalent(sav$vi, 0.001051, tolerance=.tol[["var"]])
+
+   sav <- escalc(measure="AUC", ai=auc, n1i=n1, n2i=n0, vtype="LS2")
+   expect_equivalent(sav$yi, 0.893171, tolerance=.tol[["est"]])
+   expect_equivalent(sav$vi, 0.001096, tolerance=.tol[["var"]])
+
+   sav <- escalc(measure="AUCN", m1i=mean1, m2i=mean0, sd1i=sd1, sd2i=sd0, n1i=n1, n2i=n0)
+   expect_equivalent(sav$yi, 0.909651, tolerance=.tol[["est"]])
+   expect_equivalent(sav$vi, 0.000717, tolerance=.tol[["var"]])
+
+   sav <- escalc(measure="AUCN", m1i=mean1, m2i=mean0, sd1i=sd1, sd2i=sd0, n1i=n1, n2i=n0)
+   sav <- escalc(measure="AUCN", ai=sav$yi, sd1i=sd1, sd2i=sd0, n1i=n1, n2i=n0)
+   expect_equivalent(sav$yi, 0.909651, tolerance=.tol[["est"]])
+   expect_equivalent(sav$vi, 0.000717, tolerance=.tol[["var"]])
+
+   sav <- escalc(measure="AUCN", m1i=mean1, m2i=mean0, sd1i=sd1, sd2i=sd0, n1i=n1, n2i=n0)
+   sav <- escalc(measure="AUCN", ai=sav$yi, n1i=n1, n2i=n0)
+   expect_equivalent(sav$yi, 0.909651, tolerance=.tol[["est"]])
+   expect_equivalent(sav$vi, 0.000707, tolerance=.tol[["var"]]) # uses vtype="HO"
+
+   sav <- escalc(measure="SMD", m1i=mean1, m2i=mean0, sd1i=sd1, sd2i=sd0, n1i=n1, n2i=n0, correct=FALSE)
+   sav <- escalc(measure="AUCN", di=sav$yi, n1i=n1, n2i=n0)
+   expect_equivalent(sav$yi, 0.908487, tolerance=.tol[["est"]]) # assumes HO
+   expect_equivalent(sav$vi, 0.000717, tolerance=.tol[["var"]]) # uses vtype="HO"
+
+})
+
 test_that("escalc() works correctly for measure='CVR/VR'", {
 
    dat <- dat.normand1999
@@ -169,7 +213,7 @@ test_that("escalc() works correctly for measure='MC/SMCRH'", {
 
    dat <- escalc(measure="SMCRH", m1i=26, m2i=22, sd1i=sqrt(30), sd2i=sqrt(20), ni=60, ri=0.7)
    expect_equivalent(dat$yi, 0.7210, tolerance=.tol[["est"]])
-   expect_equivalent(dat$vi, 0.0129, tolerance=.tol[["var"]])
+   expect_equivalent(dat$vi, 0.0133, tolerance=.tol[["var"]])
 
 })
 
