@@ -1,8 +1,8 @@
-### library(metafor); library(testthat); Sys.setenv(NOT_CRAN="true")
+### library(metafor); library(testthat); Sys.setenv(NOT_CRAN="true"); Sys.setenv(RUN_VIS_TESTS="true")
 
 context("Checking misc: tes() function")
 
-source("tolerances.r") # read in tolerances
+source("settings.r")
 
 test_that("tes() works correctly for 'dat.dorn2007'.", {
 
@@ -16,13 +16,18 @@ test_that("tes() works correctly for 'dat.dorn2007'.", {
    expect_equivalent(sav$X2, 7.065648, tolerance=.tol[["test"]])
    expect_equivalent(sav$pval, 0.003928794, tolerance=.tol[["pval"]])
 
-   sav <- tes(dat$yi, dat$vi, test="binom")
+   sav <- tes(yi, vi, data=dat, test="chi2")
+   expect_equivalent(sav$pval, 0.003928794, tolerance=.tol[["pval"]])
+
+   sav <- tes(yi, vi, data=dat, test="binom")
    expect_equivalent(sav$pval, 0.01159554, tolerance=.tol[["pval"]])
 
-   sav <- tes(dat$yi, dat$vi, test="exact", progbar=FALSE)
+   skip_on_cran()
+
+   sav <- tes(yi, vi, data=dat, test="exact", progbar=FALSE)
    expect_equivalent(sav$pval, 0.007778529, tolerance=.tol[["pval"]])
 
-   res <- rma(yi, vi, data=dat, method="FE")
+   res <- rma(yi, vi, data=dat, method="EE")
    sav <- tes(res, test="chi2")
 
    expect_identical(sav$O, 10L)
@@ -31,3 +36,5 @@ test_that("tes() works correctly for 'dat.dorn2007'.", {
    expect_equivalent(sav$pval, 0.003928794, tolerance=.tol[["pval"]])
 
 })
+
+rm(list=ls())
