@@ -1,4 +1,4 @@
-### library(metafor); library(testthat); Sys.setenv(NOT_CRAN="true")
+### library(metafor); library(testthat); Sys.setenv(NOT_CRAN="true"); Sys.setenv(RUN_VIS_TESTS="true")
 
 context("Checking misc: update() function")
 
@@ -26,11 +26,11 @@ test_that("update() works for rma.mv().", {
 
    dat <- escalc(measure="RR", ai=tpos, bi=tneg, ci=cpos, di=cneg, data=dat.bcg)
 
-   res1 <- rma.mv(yi, vi, data=dat, method="EE", sparse=sparse)
+   res1 <- rma.mv(yi, vi, data=dat, method="EE", sparse=.sparse)
    res2 <- update(res1, random = ~ 1 | trial, method="REML")
-   res3 <- rma.mv(yi, vi, random = ~ 1 | trial, data=dat, method="REML", sparse=sparse)
+   res3 <- rma.mv(yi, vi, random = ~ 1 | trial, data=dat, method="REML", sparse=.sparse)
    res4 <- update(res3, ~ ablat)
-   res5 <- rma.mv(yi, vi, random = ~ 1 | trial, mods = ~ ablat, data=dat, method="REML", sparse=sparse)
+   res5 <- rma.mv(yi, vi, random = ~ 1 | trial, mods = ~ ablat, data=dat, method="REML", sparse=.sparse)
    res2$time <- NULL
    res3$time <- NULL
    res4$time <- NULL
